@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2009-2021  The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -22,36 +21,30 @@
 
 #include <igraph.h>
 
-void vector_print(igraph_vector_t *v) {
-    long int i;
-    for (i = 0; i < igraph_vector_size(v); i++) {
-        printf(" %li", (long int) VECTOR(*v)[i]);
-    }
-    printf("\n");
-}
-
-int main() {
-
+int main(void) {
     igraph_t g;
-    igraph_vector_t vids, layers, parents;
+    igraph_vector_int_t vids, layers, parents;
+
+    /* Initialize the library. */
+    igraph_setup();
 
     igraph_ring(&g, 10, IGRAPH_UNDIRECTED, 0, 0);
 
-    igraph_vector_init(&vids, 0);
-    igraph_vector_init(&layers, 0);
-    igraph_vector_init(&parents, 0);
+    igraph_vector_int_init(&vids, 0);
+    igraph_vector_int_init(&layers, 0);
+    igraph_vector_int_init(&parents, 0);
 
     igraph_bfs_simple(&g, 0, IGRAPH_ALL, &vids, &layers, &parents);
 
-    vector_print(&vids);
-    vector_print(&layers);
-    vector_print(&parents);
+    igraph_vector_int_print(&vids);
+    igraph_vector_int_print(&layers);
+    igraph_vector_int_print(&parents);
 
     igraph_destroy(&g);
 
-    igraph_vector_destroy(&vids);
-    igraph_vector_destroy(&layers);
-    igraph_vector_destroy(&parents);
+    igraph_vector_int_destroy(&vids);
+    igraph_vector_int_destroy(&layers);
+    igraph_vector_int_destroy(&parents);
 
     return 0;
 }
