@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2010-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge MA, 02139 USA
 
@@ -24,6 +23,7 @@
 #include <igraph.h>
 
 int main(void) {
+    igraph_setup();
     igraph_t g;
     igraph_vector_int_list_t sep;
 
@@ -33,7 +33,7 @@ int main(void) {
     igraph_vector_int_list_init(&sep, 0);
     igraph_minimum_size_separators(&g, &sep);
 
-    for (igraph_integer_t i = 0; i < igraph_vector_int_list_size(&sep); i++) {
+    for (igraph_int_t i = 0; i < igraph_vector_int_list_size(&sep); i++) {
         igraph_vector_int_t* v = igraph_vector_int_list_get_ptr(&sep, i);
         igraph_vector_int_print(v);
     }
