@@ -3,8 +3,8 @@
 
 /* This is a callback function suitable for use with igraph_motifs_randesu_callback().
  * It prints each motif it is calld with. */
-igraph_error_t print_motif(const igraph_t *graph, igraph_vector_int_t *vids,
-                          igraph_integer_t isoclass, void* extra) {
+igraph_error_t print_motif(const igraph_t *graph, const igraph_vector_int_t *vids,
+                          igraph_int_t isoclass, void* extra) {
     printf("Found isoclass %2" IGRAPH_PRId ":  ", isoclass);
     igraph_vector_int_print(vids);
     return IGRAPH_SUCCESS; /* Return 'IGRAPH_SUCCESS': do not interrupt the search. */
@@ -15,6 +15,9 @@ int main(void) {
     igraph_t graph;
     igraph_vector_t hist;
 
+    /* Initialize the library. */
+    igraph_setup();
+
     /* Compute the 4-motif distritbuion in Zachary's karate club network. */
 
     igraph_famous(&graph, "Zachary");
@@ -25,14 +28,14 @@ int main(void) {
     /* Compute the total number of motifs (connected 4-vertex subgraphs)
      * so that we can print the normalized distribution. */
     igraph_real_t sum = 0.0;
-    igraph_integer_t n = igraph_vector_size(&hist);
-    for (igraph_integer_t i=0; i < n; i++) {
+    igraph_int_t n = igraph_vector_size(&hist);
+    for (igraph_int_t i=0; i < n; i++) {
         if (!isnan(VECTOR(hist)[i])) {
             sum += VECTOR(hist)[i];
         }
     }
     printf("4-motif distribution:\n");
-    for (igraph_integer_t i=0; i < n; i++) {
+    for (igraph_int_t i=0; i < n; i++) {
         /* Print NaN values in a platform-independent manner: */
         igraph_real_printf(VECTOR(hist)[i] / sum);
         printf(" ");
