@@ -8,6 +8,9 @@ int main(void) {
     igraph_vector_int_t v;
     igraph_vector_int_init_int(&v, 3, 3, 4, 5);
 
+    /* Initialize the library. */
+    igraph_setup();
+
     /* Create a directed symmetric tree with 2 levels -
        3 children in first and 4 children in second level,
        5 children in third level
