@@ -1,8 +1,6 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
-   Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
-   334 Harvard street, Cambridge, MA 02139 USA
+   igraph library.
+   Copyright (C) 2005-2025  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,16 +13,12 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef IGRAPH_ATTRIBUTES_H
 #define IGRAPH_ATTRIBUTES_H
 
-#include "igraph_config.h"
 #include "igraph_decls.h"
 #include "igraph_error.h"
 #include "igraph_datatype.h"
@@ -35,110 +29,140 @@
 #include "igraph_vector_ptr.h"
 #include "igraph_iterators.h"
 
-__BEGIN_DECLS
+IGRAPH_BEGIN_C_DECLS
 
 /* -------------------------------------------------- */
 /* Attributes                                         */
 /* -------------------------------------------------- */
 
-/**
- * \section about_attributes
- *
- * <para>Attributes are numbers or strings (or basically any kind
- * of data) associated with the vertices or edges of a graph, or
- * with the graph itself. E.g. you may label vertices with symbolic names
- * or attach numeric weights to the edges of a graph. </para>
- *
- * <para>igraph attributes are designed to be flexible and extensible.
- * In igraph attributes are implemented via an interface abstraction:
- * any type implementing the functions in the interface, can be used
- * for storing vertex, edge and graph attributes. This means that
- * different attribute implementations can be used together with
- * igraph. This is reasonable: if igraph is used from Python attributes can be
- * of any Python type, from GNU R all R types are allowed. There is an
- * experimental attribute implementation to be used when programming
- * in C, but by default it is currently turned off.</para>
- *
- * <para>First we briefly look over how attribute handlers can be
- * implemented. This is not something a user does every day. It is
- * rather typically the job of the high level interface writers. (But
- * it is possible to write an interface without implementing
- * attributes.) Then we show the experimental C attribute handler.</para>
- */
-
-/**
- * \section about_attribute_table
- * <para>It is possible to attach an attribute handling
- * interface to \a igraph. This is simply a table of functions, of
- * type \ref igraph_attribute_table_t. These functions are invoked to
- * notify the attribute handling code about the structural changes in
- * a graph. See the documentation of this type for details.</para>
- *
- * <para>By default there is no attribute interface attached to \a igraph,
- * to attach one, call \ref igraph_set_attribute_table with your new
- * table. </para>
- *
- */
-
-/**
- * \section about_attribute_combination
- *
- * <para>Several graph operations may collapse multiple vertices or edges into
- * a single one. Attribute combination lists are used to indicate to the attribute
- * handler how to combine the attributes of the original vertices or edges and
- * how to derive the final attribute value that is to be assigned to the collapsed
- * vertex or edge. For example, \ref igraph_simplify() removes loops and combines
- * multiple edges into a single one; in case of a graph with an edge attribute
- * named \c weight the attribute combination list can tell the attribute handler
- * whether the weight of a collapsed edge should be the sum, the mean or some other
- * function of the weights of the original edges that were collapsed into one.</para>
- *
- * <para>One attribute combination list may contain several attribute combination
- * records, one for each vertex or edge attribute that is to be handled during the
- * operation.</para>
- */
-
 /**
  * \typedef igraph_attribute_type_t
- * The possible types of the attributes.
+ * \brief The possible types of the attributes.
+ *
+ * Values of this enum are used by the attribute interface to communicate the
+ * type of an attribute to igraph's C core. When igraph is integrated in a
+ * high-level language, the attribute type reported by the interface may not
+ * necessarily have to match the exact data type in the high-level language as
+ * long as the attribute interface can provide a conversion from the native
+ * high-level attribute value to one of the data types listed here. When the
+ * high-level data type is complex and has no suitable conversion to one of the
+ * atomic igraph attribute types (numeric, string or Boolean), the attribute
+ * interface should report the attribute as having an "object" type, which is
+ * ignored by the C core. See also \ref igraph_attribute_table_t.
  *
- * Note that this is only the
- * type communicated by the attribute interface towards igraph
- * functions. E.g. in the R attribute handler, it is safe to say
- * that all complex R object attributes are strings, as long as this
- * interface is able to serialize them into strings. See also \ref
- * igraph_attribute_table_t.
  * \enumval IGRAPH_ATTRIBUTE_UNSPECIFIED Currently used internally
  *   as a "null value" or "placeholder value" in some algorithms.
  *   Attribute records with this type must not be passed to igraph
  *   functions.
  * \enumval IGRAPH_ATTRIBUTE_NUMERIC Numeric attribute.
  * \enumval IGRAPH_ATTRIBUTE_BOOLEAN Logical values, true or false.
- * \enumval IGRAPH_ATTRIBUTE_STRING Attribute that can be converted to
- *   a string.
+ * \enumval IGRAPH_ATTRIBUTE_STRING String attribute.
  * \enumval IGRAPH_ATTRIBUTE_OBJECT Custom attribute type, to be
  *   used for special data types by client applications. The R and
  *   Python interfaces use this for attributes that hold R or Python
  *   objects. Usually ignored by igraph functions.
  */
-typedef enum { IGRAPH_ATTRIBUTE_UNSPECIFIED = 0,
-               IGRAPH_ATTRIBUTE_DEFAULT IGRAPH_DEPRECATED_ENUMVAL = IGRAPH_ATTRIBUTE_UNSPECIFIED,
-               IGRAPH_ATTRIBUTE_NUMERIC = 1,
-               IGRAPH_ATTRIBUTE_BOOLEAN = 2,
-               IGRAPH_ATTRIBUTE_STRING = 3,
-               IGRAPH_ATTRIBUTE_OBJECT = 127
-             } igraph_attribute_type_t;
+typedef enum {
+    IGRAPH_ATTRIBUTE_UNSPECIFIED = 0,
+    IGRAPH_ATTRIBUTE_NUMERIC = 1,
+    IGRAPH_ATTRIBUTE_BOOLEAN = 2,
+    IGRAPH_ATTRIBUTE_STRING = 3,
+    IGRAPH_ATTRIBUTE_OBJECT = 127
+} igraph_attribute_type_t;
 
+/**
+ * \typedef igraph_attribute_elemtype_t
+ * \brief Types of objects to which attributes can be attached.
+ *
+ * \enumval IGRAPH_ATTRIBUTE_GRAPH Denotes that an attribute belongs to the
+ *   entire graph.
+ * \enumval IGRAPH_ATTRIBUTE_VERTEX Denotes that an attribute belongs to the
+ *   vertices of a graph.
+ * \enumval IGRAPH_ATTRIBUTE_EDGE Denotes that an attribute belongs to the
+ *   edges of a graph.
+ */
+typedef enum {
+    IGRAPH_ATTRIBUTE_GRAPH = 0,
+    IGRAPH_ATTRIBUTE_VERTEX,
+    IGRAPH_ATTRIBUTE_EDGE
+} igraph_attribute_elemtype_t;
+
+/* -------------------------------------------------- */
+/* Attribute records                                  */
+/* -------------------------------------------------- */
+
+/**
+ * \typedef igraph_attribute_record_t
+ * \brief An attribute record holding the name, type and values of an attribute.
+ *
+ * This composite data type is used in the attribute interface to specify a
+ * name-type-value triplet where the name is the name of a graph, vertex or
+ * edge attribute, the type is the corresponding igraph type of the attribute
+ * and the value is a \em vector of attribute values. Note that for graph
+ * attributes we use a vector of length 1. The type of the vector depends on
+ * the attribute type: it is \ref igraph_vector_t for numeric attributes,
+ * \c igraph_strvector_t for string attributes and \c igraph_vector_bool_t
+ * for Boolean attributes.
+ *
+ * </para><para>
+ * The record also stores default values for the attribute. The default values
+ * are used when the value vector of the record is resized with
+ * \ref igraph_attribute_record_resize(). It is important that the record
+ * stores \em one default value only, corresponding to the type of the
+ * attribute record. The default value is \em cleared when the type of the
+ * record is changed.
+ */
 typedef struct igraph_attribute_record_t {
-    const char *name;
+    char *name;
     igraph_attribute_type_t type;
-    const void *value;
+    union {
+        void *as_raw;
+        igraph_vector_t *as_vector;
+        igraph_strvector_t *as_strvector;
+        igraph_vector_bool_t *as_vector_bool;
+    } value;
+    union {
+        igraph_real_t numeric;
+        igraph_bool_t boolean;
+        char *string;
+    } default_value;
 } igraph_attribute_record_t;
 
-typedef enum { IGRAPH_ATTRIBUTE_GRAPH = 0,
-               IGRAPH_ATTRIBUTE_VERTEX,
-               IGRAPH_ATTRIBUTE_EDGE
-             } igraph_attribute_elemtype_t;
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_init(
+    igraph_attribute_record_t *attr, const char* name, igraph_attribute_type_t type
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_init_copy(
+    igraph_attribute_record_t *to, const igraph_attribute_record_t *from
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_check_type(
+    const igraph_attribute_record_t *attr, igraph_attribute_type_t type
+);
+IGRAPH_EXPORT igraph_int_t igraph_attribute_record_size(
+    const igraph_attribute_record_t *attr
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_resize(
+    igraph_attribute_record_t *attr, igraph_int_t new_size
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_set_name(
+    igraph_attribute_record_t *attr, const char* name
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_set_default_numeric(
+    igraph_attribute_record_t *attr, igraph_real_t value
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_set_default_boolean(
+    igraph_attribute_record_t *attr, igraph_bool_t value
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_set_default_string(
+    igraph_attribute_record_t *attr, const char* value
+);
+IGRAPH_EXPORT igraph_error_t igraph_attribute_record_set_type(
+    igraph_attribute_record_t *attr, igraph_attribute_type_t type
+);
+IGRAPH_EXPORT void igraph_attribute_record_destroy(igraph_attribute_record_t *attr);
+
+/* -------------------------------------------------- */
+/* Attribute combinations                             */
+/* -------------------------------------------------- */
 
 /**
  * \typedef igraph_attribute_combination_type_t
@@ -203,90 +227,165 @@ IGRAPH_EXPORT igraph_error_t igraph_attribute_combination_query(const igraph_att
                                                      igraph_attribute_combination_type_t *type,
                                                      igraph_function_pointer_t *func);
 
+/* -------------------------------------------------- */
+/* List of attribute records                          */
+/* -------------------------------------------------- */
+
+#define ATTRIBUTE_RECORD_LIST
+#define BASE_ATTRIBUTE_RECORD
+#include "igraph_pmt.h"
+#include "igraph_typed_list_pmt.h"
+#include "igraph_pmt_off.h"
+#undef BASE_ATTRIBUTE_RECORD
+#undef ATTRIBUTE_RECORD_LIST
+
+/* -------------------------------------------------- */
+/* Attribute handler interface                        */
+/* -------------------------------------------------- */
+
 /**
  * \struct igraph_attribute_table_t
- * \brief Table of functions to perform operations on attributes
+ * \brief Table of functions to perform operations on attributes.
  *
  * This type collects the functions defining an attribute handler.
  * It has the following members:
+ *
  * \member init This function is called whenever a new graph object is
  *    created, right after it is created but before any vertices or
  *    edges are added. It is supposed to set the \c attr member of the \c
- *    igraph_t object. It is expected to return an error code.
+ *    igraph_t object, which is guaranteed to be set to a null pointer
+ *    before this function is called. It is expected to set the \c attr member
+ *    to a non-null value \em or return an error code. Leaving the \c attr
+ *    member at a null value while returning success is invalid and will trigger
+ *    an error in the C core of igraph itself.
  * \member destroy This function is called whenever the graph object
- *    is destroyed, right before freeing the allocated memory.
- * \member copy This function is called when copying a graph with \ref
- *    igraph_copy, after the structure of the graph has been already
- *    copied. It is expected to return an error code.
- * \member add_vertices Called when vertices are added to a
- *    graph, before adding the vertices themselves.
- *    The number of vertices to add is supplied as an
- *    argument. Expected to return an error code.
- * \member permute_vertices Typically called when a new graph is
- *    created based on an existing one, e.g. if vertices are removed
- *    from a graph. The supplied index vector defines which old vertex
- *    a new vertex corresponds to. Its length must be the same as the
- *    number of vertices in the new graph.
+ *    is destroyed, right before freeing the allocated memory. It is supposed
+ *    to do any cleanup operations that are need to dispose of the \c attr
+ *    member of the \c igraph_t object properly. The caller will set the
+ *    \c attr member to a null pointer after this function returns.
+ * \member copy This function is called when the C core wants to populate the
+ *    attributes of a graph from another graph. The structure of the target
+ *    graph is already initialized by the time this function is called, and the
+ *    \c attr member of the graph is set to a null pointer. The function is
+ *    supposed to populate the \c attr member of the target \c igraph_t object
+ *    to a non-null value \em or return an error code. Leaving the \c attr
+ *    member at a null value while returning success is invalid and will trigger
+ *    an error in the C core of igraph itself.
+ * \member add_vertices Called when vertices are added to a graph, after the
+ *    base data structure was modified. The number of vertices that were added is
+ *    supplied as an argument. The function is supposed to set up default values
+ *    for each vertex attribute that is currently registered on the graph, for
+ *    all the newly added vertices. Expected to return an error code.
+ * \member permute_vertices Called when a new graph is created based on an
+ *    existing one such that there is a mapping from the vertices of the new
+ *    graph back to the vertices of the old graph (e.g. if vertices are removed
+ *    from a graph). The supplied index vector defines which old vertex
+ *    a new vertex corresponds to. Its length is the same as the number of
+ *    vertices in the new graph, and for each new vertex it provides the ID
+ *    of the corresponding vertex in the old graph. The function is supposed to
+ *    set up the values of the vertex attributes of the new graph based on the
+ *    attributes of the old graph and the provided index vector. Note that the
+ *    old and the new graph \em may be the same, in which case it is the
+ *    responsibility of the function to ensure that the operation can safely be
+ *    performed in-place. If the two graph instances are \em not the same,
+ *    implementors may safely assume that the new graph has no vertex attributes
+ *    yet (but it may already have graph or edge attributes by the time this
+ *    function is called).
  * \member combine_vertices This function is called when the creation
  *    of a new graph involves a merge (contraction, etc.) of vertices
- *    from another graph. The function is after the new graph was created.
+ *    from another graph. The function is called after the new graph was created.
  *    An argument specifies how several vertices from the old graph map to a
- *    single vertex in the new graph.
- * \member add_edges Called when new edges have been added. The number
- *    of new edges are supplied as well. It is expected to return an
- *    error code.
- * \member permute_edges Typically called when a new graph is created and
- *    some of the new edges should carry the attributes of some of the
- *    old edges. The idx vector shows the mapping between the old edges and
- *    the new ones. Its length is the same as the number of edges in the new
- *    graph, and for each edge it gives the id of the old edge (the edge in
- *    the old graph).
+ *    single vertex in the new graph. It is guaranteed that the old and the
+ *    new graph instances are different when this callback is called.
+ *    Implementors may safely assume that the new graph has no vertex attributes
+ *    yet (but it may already have graph or edge attributes by the time this
+ *    function is called).
+ * \member add_edges Called when new edges are added to a graph, after the
+ *    base data structure was modified. A vector containing the endpoints of the
+ *    new edges are supplied as an argument. The function is supposed to set up
+ *    default values for each edge attribute that is currently registered on the
+ *    graph, for all the newly added edges. Expected to return an error code.
+ * \member permute_edges Called when a new graph is created based on an
+ *    existing one such that some of the edges in the new graph should copy the
+ *    attributes of some edges from the old graph (this also includes the
+ *    deletion of edges). The supplied index vector defines which old edge a new
+ *    edge corresponds to. Its length is the same as the number of edges in the
+ *    new graph, and for each edge it provides the ID of the corresponding edge
+ *    in the old graph. The function is supposed to set up the values of the
+ *    edge attributes of the new graph based on the attributes of the old graph
+ *    and the provided index vector. Note that the old and the new graph \em may
+ *    be the same, in which case it is the responsibility of the function to
+ *    ensure that the operation can safely be performed in-place. If the two
+ *    graph instances are \em not the same, implementors may safely assume that
+ *    the new graph has no edge attributes yet (but it may already have graph or
+ *    vertex attributes by the time this function is called).
  * \member combine_edges This function is called when the creation
  *    of a new graph involves a merge (contraction, etc.) of edges
  *    from another graph. The function is after the new graph was created.
  *    An argument specifies how several edges from the old graph map to a
- *    single edge in the new graph.
+ *    single edge in the new graph. It is guaranteed that the old and the
+ *    new graph instances are different when this callback is called.
+ *    Implementors may safely assume that the new graph has no edge attributes
+ *    yet (but it may already have graph or vertex attributes by the time this
+ *    function is called).
  * \member get_info Query the attributes of a graph, the names and
  *    types should be returned.
  * \member has_attr Check whether a graph has the named
  *    graph/vertex/edge attribute.
- * \member gettype Query the type of a graph/vertex/edge attribute.
+ * \member get_type Query the type of a graph/vertex/edge attribute.
  * \member get_numeric_graph_attr Query a numeric graph attribute. The
- *    value should be placed as the first element of the \p value
- *    vector.
+ *    value should be appended to the provided \p value vector. No assumptions
+ *    should be made about the initial contents of the \p value vector and it is
+ *    not guaranteed to be empty.
  * \member get_string_graph_attr Query a string graph attribute. The
- *    value should be placed as the first element of the \p value
- *    string vector.
+ *    value should be appended to the provided \p value vector. No assumptions
+ *    should be made about the initial contents of the \p value vector and it is
+ *    not guaranteed to be empty.
  * \member get_bool_graph_attr Query a boolean graph attribute. The
- *    value should be placed as the first element of the \p value
- *    boolean vector.
+ *    value should be appended to the provided \p value vector. No assumptions
+ *    should be made about the initial contents of the \p value vector and it is
+ *    not guaranteed to be empty.
  * \member get_numeric_vertex_attr Query a numeric vertex attribute,
- *    for the vertices included in \p vs.
+ *    for the vertices included in \p vs. The attribute values should be
+ *    appended to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  * \member get_string_vertex_attr Query a string vertex attribute,
- *    for the vertices included in \p vs.
+ *    for the vertices included in \p vs. The attribute values should be
+ *    appended to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  * \member get_bool_vertex_attr Query a boolean vertex attribute,
- *    for the vertices included in \p vs.
+ *    for the vertices included in \p vs. The attribute values should be
+ *    appended to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  * \member get_numeric_edge_attr Query a numeric edge attribute, for
- *    the edges included in \p es.
+ *    the edges included in \p es. The attribute values should be appended
+ *    to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  * \member get_string_edge_attr Query a string edge attribute, for the
- *    edges included in \p es.
+ *    the edges included in \p es. The attribute values should be appended
+ *    to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  * \member get_bool_edge_attr Query a boolean edge attribute, for the
- *    edges included in \p es.
- *
- * Note that the <function>get_*_*_attr</function> are allowed to
- * convert the attributes to numeric or string. E.g. if a vertex attribute
- * is a GNU R complex data type, then
- * <function>get_string_vertex_attribute</function> may serialize it
- * into a string, but this probably makes sense only if
- * <function>add_vertices</function> is able to deserialize it.
+ *    the edges included in \p es. The attribute values should be appended
+ *    to the provided \p value vector. No assumptions should be made
+ *    about the initial contents of the \p value vector and it is not guaranteed
+ *    to be empty.
  */
 
 typedef struct igraph_attribute_table_t {
-    igraph_error_t (*init)(igraph_t *graph, igraph_vector_ptr_t *attr);
+    igraph_error_t (*init)(igraph_t *graph, const igraph_attribute_record_list_t *attr);
     void           (*destroy)(igraph_t *graph);
     igraph_error_t (*copy)(igraph_t *to, const igraph_t *from, igraph_bool_t ga,
                            igraph_bool_t va, igraph_bool_t ea);
-    igraph_error_t (*add_vertices)(igraph_t *graph, igraph_integer_t nv, igraph_vector_ptr_t *attr);
+    igraph_error_t (*add_vertices)(
+        igraph_t *graph, igraph_int_t nv,
+        const igraph_attribute_record_list_t *attr
+    );
     igraph_error_t (*permute_vertices)(const igraph_t *graph,
                                        igraph_t *newgraph,
                                        const igraph_vector_int_t *idx);
@@ -294,8 +393,10 @@ typedef struct igraph_attribute_table_t {
                                        igraph_t *newgraph,
                                        const igraph_vector_int_list_t *merges,
                                        const igraph_attribute_combination_t *comb);
-    igraph_error_t (*add_edges)(igraph_t *graph, const igraph_vector_int_t *edges,
-                                igraph_vector_ptr_t *attr);
+    igraph_error_t (*add_edges)(
+        igraph_t *graph, const igraph_vector_int_t *edges,
+        const igraph_attribute_record_list_t *attr
+    );
     igraph_error_t (*permute_edges)(const igraph_t *graph,
                                     igraph_t *newgraph, const igraph_vector_int_t *idx);
     igraph_error_t (*combine_edges)(const igraph_t *graph,
@@ -308,7 +409,7 @@ typedef struct igraph_attribute_table_t {
                                igraph_strvector_t *enames, igraph_vector_int_t *etypes);
     igraph_bool_t (*has_attr)(const igraph_t *graph, igraph_attribute_elemtype_t type,
                               const char *name);
-    igraph_error_t (*gettype)(const igraph_t *graph, igraph_attribute_type_t *type,
+    igraph_error_t (*get_type)(const igraph_t *graph, igraph_attribute_type_t *type,
                               igraph_attribute_elemtype_t elemtype, const char *name);
     igraph_error_t (*get_numeric_graph_attr)(const igraph_t *graph, const char *name,
                                              igraph_vector_t *value);
@@ -336,7 +437,6 @@ typedef struct igraph_attribute_table_t {
                                          igraph_vector_bool_t *value);
 } igraph_attribute_table_t;
 
-IGRAPH_EXPORT IGRAPH_DEPRECATED igraph_attribute_table_t * igraph_i_set_attribute_table(const igraph_attribute_table_t * table);
 IGRAPH_EXPORT igraph_attribute_table_t * igraph_set_attribute_table(const igraph_attribute_table_t * table);
 
 IGRAPH_EXPORT igraph_bool_t igraph_has_attribute_table(void);
@@ -349,17 +449,17 @@ IGRAPH_EXPORT igraph_real_t igraph_cattribute_GAN(const igraph_t *graph, const c
 IGRAPH_EXPORT igraph_bool_t igraph_cattribute_GAB(const igraph_t *graph, const char *name);
 IGRAPH_EXPORT const char* igraph_cattribute_GAS(const igraph_t *graph, const char *name);
 IGRAPH_EXPORT igraph_real_t igraph_cattribute_VAN(const igraph_t *graph, const char *name,
-                                                  igraph_integer_t vid);
+                                                  igraph_int_t vid);
 IGRAPH_EXPORT igraph_bool_t igraph_cattribute_VAB(const igraph_t *graph, const char *name,
-                                                  igraph_integer_t vid);
+                                                  igraph_int_t vid);
 IGRAPH_EXPORT const char* igraph_cattribute_VAS(const igraph_t *graph, const char *name,
-                                                igraph_integer_t vid);
+                                                igraph_int_t vid);
 IGRAPH_EXPORT igraph_real_t igraph_cattribute_EAN(const igraph_t *graph, const char *name,
-                                                  igraph_integer_t eid);
+                                                  igraph_int_t eid);
 IGRAPH_EXPORT igraph_bool_t igraph_cattribute_EAB(const igraph_t *graph, const char *name,
-                                                  igraph_integer_t eid);
+                                                  igraph_int_t eid);
 IGRAPH_EXPORT const char* igraph_cattribute_EAS(const igraph_t *graph, const char *name,
-                                                igraph_integer_t eid);
+                                                igraph_int_t eid);
 
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_VANV(const igraph_t *graph, const char *name,
                                          igraph_vs_t vids, igraph_vector_t *result);
@@ -389,17 +489,17 @@ IGRAPH_EXPORT igraph_error_t igraph_cattribute_GAB_set(igraph_t *graph, const ch
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_GAS_set(igraph_t *graph, const char *name,
                                             const char *value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_VAN_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t vid, igraph_real_t value);
+                                            igraph_int_t vid, igraph_real_t value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_VAB_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t vid, igraph_bool_t value);
+                                            igraph_int_t vid, igraph_bool_t value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_VAS_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t vid, const char *value);
+                                            igraph_int_t vid, const char *value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_EAN_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t eid, igraph_real_t value);
+                                            igraph_int_t eid, igraph_real_t value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_EAB_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t eid, igraph_bool_t value);
+                                            igraph_int_t eid, igraph_bool_t value);
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_EAS_set(igraph_t *graph, const char *name,
-                                            igraph_integer_t eid, const char *value);
+                                            igraph_int_t eid, const char *value);
 
 IGRAPH_EXPORT igraph_error_t igraph_cattribute_VAN_setv(igraph_t *graph, const char *name,
                                              const igraph_vector_t *v);
@@ -826,6 +926,6 @@ IGRAPH_EXPORT void igraph_cattribute_remove_all(igraph_t *graph, igraph_bool_t g
  */
 #define DELALL(graph) (igraph_cattribute_remove_all((graph),1,1,1))
 
-__END_DECLS
+IGRAPH_END_C_DECLS
 
 #endif
