@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2013  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -25,11 +24,11 @@
 #define RESTYPE igraph_vector_int_list_t *res
 #define RESNAME res
 #define SUFFIX
-#define RECORD do {                         \
-        IGRAPH_CHECK(igraph_vector_int_list_push_back_copy(res, R));     \
+#define RECORD do { \
+        IGRAPH_CHECK(igraph_vector_int_list_push_back_copy(res, R)); \
     } while (0)
-#define PREPARE do {                    \
-        igraph_vector_int_list_clear(res);           \
+#define PREPARE do { \
+        igraph_vector_int_list_clear(res); \
     } while (0)
 #define CLEANUP
 #define FOR_LOOP_OVER_VERTICES for (i=0; i<no_of_nodes; i++)
@@ -37,7 +36,7 @@
 #endif
 
 #ifdef IGRAPH_MC_COUNT
-    #define RESTYPE igraph_integer_t *res
+    #define RESTYPE igraph_int_t *res
     #define RESNAME res
     #define SUFFIX _count
     #define RECORD (*res)++
@@ -59,32 +58,32 @@
 #endif
 
 #ifdef IGRAPH_MC_FULL
-#define RESTYPE                 \
-    const igraph_vector_int_t *subset,            \
-    igraph_vector_int_list_t *res,           \
-    igraph_integer_t *no,           \
+#define RESTYPE \
+    const igraph_vector_int_t *subset, \
+    igraph_vector_int_list_t *res, \
+    igraph_int_t *no, \
     FILE *outfile
 #define RESNAME subset, res, no, outfile
 #define SUFFIX _subset
-#define RECORD do {                         \
-        if (res) {                                \
-            IGRAPH_CHECK(igraph_vector_int_list_push_back_copy(res, R));      \
-        }                                 \
-        if (no) { (*no)++; }                              \
-        if (outfile) { igraph_vector_int_fprint(R, outfile); }        \
+#define RECORD do { \
+        if (res) { \
+            IGRAPH_CHECK(igraph_vector_int_list_push_back_copy(res, R)); \
+        } \
+        if (no) { (*no)++; } \
+        if (outfile) { igraph_vector_int_fprint(R, outfile); } \
     } while (0)
-#define PREPARE do {                        \
-        if (res) {                                 \
-            igraph_vector_int_list_clear(res);     \
-        }                             \
-        if (no) { *no=0; }                        \
+#define PREPARE do { \
+        if (res) { \
+            igraph_vector_int_list_clear(res); \
+        } \
+        if (no) { *no=0; } \
     } while (0)
 #define CLEANUP
-#define FOR_LOOP_OVER_VERTICES                  \
-    nn= subset ? igraph_vector_int_size(subset) : no_of_nodes;    \
+#define FOR_LOOP_OVER_VERTICES \
+    nn = subset ? igraph_vector_int_size(subset) : no_of_nodes; \
     for (ii=0; ii<nn; ii++)
-#define FOR_LOOP_OVER_VERTICES_PREPARE do {  \
-        i= subset ? VECTOR(*subset)[ii] : ii;    \
+#define FOR_LOOP_OVER_VERTICES_PREPARE do { \
+        i = subset ? VECTOR(*subset)[ii] : ii; \
     } while (0)
 #endif
 
@@ -114,10 +113,10 @@
 #define RESNAME hist
 #define SUFFIX _hist
 #define RECORD do { \
-        igraph_integer_t hsize = igraph_vector_size(hist); \
+        igraph_int_t hsize = igraph_vector_size(hist); \
         if (clsize > hsize) { \
-            igraph_integer_t hcapacity = igraph_vector_capacity(hist); \
-            igraph_integer_t j; \
+            igraph_int_t hcapacity = igraph_vector_capacity(hist); \
+            igraph_int_t j; \
             igraph_error_t err; \
             if (hcapacity < clsize && clsize < 2*hcapacity) \
                 err = igraph_vector_reserve(hist, 2*hcapacity); \
@@ -138,15 +137,16 @@
 #endif
 
 static igraph_error_t FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
-    igraph_vector_int_t *PX, igraph_integer_t PS, igraph_integer_t PE,
-    igraph_integer_t XS, igraph_integer_t XE, igraph_integer_t oldPS, igraph_integer_t oldXE,
-    igraph_vector_int_t *R,
-    igraph_vector_int_t *pos,
-    igraph_adjlist_t *adjlist,
-    RESTYPE,
-    igraph_vector_int_t *nextv,
-    igraph_vector_int_t *H,
-    igraph_integer_t min_size, igraph_integer_t max_size) {
+        igraph_vector_int_t *PX, igraph_int_t PS, igraph_int_t PE,
+        igraph_int_t XS, igraph_int_t XE, igraph_int_t oldPS, igraph_int_t oldXE,
+        igraph_vector_int_t *R,
+        igraph_vector_int_t *pos,
+        igraph_adjlist_t *adjlist,
+        RESTYPE,
+        igraph_vector_int_t *nextv,
+        igraph_vector_int_t *H,
+        igraph_int_t min_size, igraph_int_t max_size,
+        igraph_int_t max_results, igraph_int_t *result_count) {
 
     igraph_error_t err;
 
@@ -154,18 +154,22 @@ static igraph_error_t FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
 
     if (PS > PE && XS > XE) {
         /* Found a maximum clique, report it */
-        igraph_integer_t clsize = igraph_vector_int_size(R);
+        igraph_int_t clsize = igraph_vector_int_size(R);
         if (min_size <= clsize && (clsize <= max_size || max_size <= 0)) {
             RECORD;
+            *result_count += 1;
+            if (max_results >= 0 && *result_count == max_results) {
+                return IGRAPH_STOP;
+            }
         }
     } else if (PS <= PE) {
         /* Select a pivot element */
-        igraph_integer_t pivot, mynextv;
+        igraph_int_t pivot, mynextv;
         IGRAPH_CHECK(igraph_i_maximal_cliques_select_pivot(
             PX, PS, PE, XS, XE, pos, adjlist, &pivot, nextv, oldPS, oldXE
         ));
         while ((mynextv = igraph_vector_int_pop_back(nextv)) != -1) {
-            igraph_integer_t newPS, newXE;
+            igraph_int_t newPS, newXE;
 
             /* Going down, prepare */
             IGRAPH_CHECK(igraph_i_maximal_cliques_down(
@@ -175,7 +179,8 @@ static igraph_error_t FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
             err = FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
                       PX, newPS, PE, XS, newXE, PS, XE, R,
                       pos, adjlist, RESNAME, nextv, H,
-                      min_size, max_size);
+                      min_size, max_size,
+                      max_results, result_count);
 
             if (err == IGRAPH_STOP) {
                 return err;
@@ -197,27 +202,36 @@ static igraph_error_t FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
     return IGRAPH_SUCCESS;
 }
 
-igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
+igraph_error_t FUNCTION(igraph_i_maximal_cliques, SUFFIX)(
     const igraph_t *graph,
     RESTYPE,
-    igraph_integer_t min_size,
-    igraph_integer_t max_size) {
+    igraph_int_t min_size,
+    igraph_int_t max_size,
+    igraph_int_t max_results) {
 
     /* Implementation details. TODO */
 
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t PX, R, H, pos, nextv;
     igraph_vector_int_t coreness;
     igraph_vector_int_t order;
     igraph_vector_int_t rank; /* TODO: this is not needed */
-    igraph_integer_t i, ii, nn, no_of_nodes = igraph_vcount(graph);
+    igraph_int_t i, ii, nn;
     igraph_adjlist_t adjlist, fulladjlist;
+    igraph_int_t result_count = 0;
     igraph_real_t pgreset = round(no_of_nodes / 100.0), pg = pgreset, pgc = 0;
     igraph_error_t err;
-    IGRAPH_UNUSED(nn);
+
+    IGRAPH_UNUSED(nn); /* not used by all implementations */
 
     if (igraph_is_directed(graph)) {
-        IGRAPH_WARNING("Edge directions are ignored for maximal clique "
-                       "calculation");
+        IGRAPH_WARNING("Edge directions are ignored for maximal clique calculations.");
+    }
+
+    PREPARE;
+
+    if (max_results == 0) {
+        return IGRAPH_SUCCESS;
     }
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&order, no_of_nodes);
@@ -226,7 +240,7 @@ igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
     IGRAPH_CHECK(igraph_coreness(graph, &coreness, /*mode=*/ IGRAPH_ALL));
     IGRAPH_CHECK(igraph_vector_int_sort_ind(&coreness, &order, IGRAPH_ASCENDING));
     for (ii = 0; ii < no_of_nodes; ii++) {
-        igraph_integer_t v = VECTOR(order)[ii];
+        igraph_int_t v = VECTOR(order)[ii];
         VECTOR(rank)[v] = ii;
     }
 
@@ -245,15 +259,13 @@ igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
     IGRAPH_VECTOR_INT_INIT_FINALLY(&pos, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&nextv, 100);
 
-    PREPARE;
-
     FOR_LOOP_OVER_VERTICES {
-        igraph_integer_t v;
-        igraph_integer_t vrank;
+        igraph_int_t v;
+        igraph_int_t vrank;
         igraph_vector_int_t *vneis;
-        igraph_integer_t vdeg;
-        igraph_integer_t Pptr, Xptr, PS, PE, XS, XE;
-        igraph_integer_t j;
+        igraph_int_t vdeg;
+        igraph_int_t Pptr, Xptr, PS, PE, XS, XE;
+        igraph_int_t j;
 
         FOR_LOOP_OVER_VERTICES_PREPARE;
 
@@ -286,7 +298,7 @@ igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
 
         VECTOR(R)[0] = v;
         for (j = 0; j < vdeg; j++) {
-            igraph_integer_t vx = VECTOR(*vneis)[j];
+            igraph_int_t vx = VECTOR(*vneis)[j];
             if (VECTOR(rank)[vx] > vrank) {
                 VECTOR(PX)[Pptr] = vx;
                 VECTOR(pos)[vx] = Pptr + 1;
@@ -308,14 +320,14 @@ igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
             igraph_adjlist_get(&fulladjlist, v)
         ));
         for (j = 0; j <= vdeg - 1; j++) {
-            igraph_integer_t vv = VECTOR(PX)[j];
+            igraph_int_t vv = VECTOR(PX)[j];
             igraph_vector_int_t *fadj = igraph_adjlist_get(&fulladjlist, vv);
             igraph_vector_int_t *radj = igraph_adjlist_get(&adjlist, vv);
-            igraph_integer_t k, fn = igraph_vector_int_size(fadj);
+            igraph_int_t k, fn = igraph_vector_int_size(fadj);
             igraph_vector_int_clear(radj);
             for (k = 0; k < fn; k++) {
-                igraph_integer_t nei = VECTOR(*fadj)[k];
-                igraph_integer_t neipos = VECTOR(pos)[nei] - 1;
+                igraph_int_t nei = VECTOR(*fadj)[k];
+                igraph_int_t neipos = VECTOR(pos)[nei] - 1;
                 if (neipos >= PS && neipos <= XE) {
                     IGRAPH_CHECK(igraph_vector_int_push_back(radj, nei));
                 }
@@ -329,8 +341,8 @@ igraph_error_t FUNCTION(igraph_maximal_cliques, SUFFIX)(
 
         err = FUNCTION(igraph_i_maximal_cliques_bk, SUFFIX)(
                 &PX, PS, PE, XS, XE, PS, XE, &R, &pos,
-                &adjlist, RESNAME, &nextv, &H, min_size,
-                max_size);
+                &adjlist, RESNAME, &nextv, &H,
+                min_size, max_size, max_results, &result_count);
         if (err == IGRAPH_STOP) {
             break;
         } else {
