@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,12 @@
 #include "igraph_bitset.h"
 #include "igraph_memory.h"
 
-igraph_integer_t igraph_i_ctz32(igraph_uint_t x) {
+igraph_int_t igraph_i_ctz32(igraph_uint_t x) {
 #ifdef HAVE__BITSCANFORWARD
     unsigned long index;
     return _BitScanForward(&index, x) ? index : 32;
 #else
-    for (igraph_integer_t i = 0; i < 32; ++i) {
+    for (igraph_int_t i = 0; i < 32; ++i) {
         if (IGRAPH_BIT_MASK(i) & x) {
             return i;
         }
@@ -35,12 +35,12 @@ igraph_integer_t igraph_i_ctz32(igraph_uint_t x) {
 #endif
 }
 
-igraph_integer_t igraph_i_clz32(igraph_uint_t x) {
+igraph_int_t igraph_i_clz32(igraph_uint_t x) {
 #ifdef HAVE_BITSCANREVERSE
     unsigned long index;
     return _BitScanReverse(&index, x) ? 31 - index : 32;
 #else
-    for (igraph_integer_t i = 31; i >= 0; --i) {
+    for (igraph_int_t i = 31; i >= 0; --i) {
         if (IGRAPH_BIT_MASK(i) & x) {
             return 31 - i;
         }
@@ -49,8 +49,8 @@ igraph_integer_t igraph_i_clz32(igraph_uint_t x) {
 #endif
 }
 
-igraph_integer_t igraph_i_popcnt(igraph_uint_t x) {
-    igraph_integer_t result = 0;
+igraph_int_t igraph_i_popcnt(igraph_uint_t x) {
+    igraph_int_t result = 0;
     while (x) {
         result++;
         x = x & (x - 1);
@@ -58,14 +58,14 @@ igraph_integer_t igraph_i_popcnt(igraph_uint_t x) {
     return result;
 }
 
-/* Fallbacks for 64-bit word (and igraph_integer_t/igraph_uint_t) size */
+/* Fallbacks for 64-bit word (and igraph_int_t/igraph_uint_t) size */
 #if IGRAPH_INTEGER_SIZE == 64
-igraph_integer_t igraph_i_ctz64(igraph_uint_t x) {
+igraph_int_t igraph_i_ctz64(igraph_uint_t x) {
 #ifdef HAVE_BITSCANFORWARD64
     unsigned long index;
     return _BitScanForward64(&index, x) ? index : 64;
 #else
-    for (igraph_integer_t i = 0; i < 64; ++i) {
+    for (igraph_int_t i = 0; i < 64; ++i) {
         if (IGRAPH_BIT_MASK(i) & x) {
             return i;
         }
@@ -74,12 +74,12 @@ igraph_integer_t igraph_i_ctz64(igraph_uint_t x) {
 #endif
 }
 
-igraph_integer_t igraph_i_clz64(igraph_uint_t x) {
+igraph_int_t igraph_i_clz64(igraph_uint_t x) {
 #ifdef HAVE_BITSCANREVERSE64
     unsigned long index;
     return _BitScanReverse64(&index, x) ? 63 - index : 64;
 #else
-    for (igraph_integer_t i = 63; i >= 0; --i) {
+    for (igraph_int_t i = 63; i >= 0; --i) {
         if (IGRAPH_BIT_MASK(i) & x) {
             return 63 - i;
         }
@@ -99,7 +99,7 @@ igraph_integer_t igraph_i_clz64(igraph_uint_t x) {
  * difference being the C++ version's size is initialized at compile time.</para>
  *
  * <para>The \type igraph_bitset_t type and use O(n/w) space
- * to store n elements, where w is the bit width of \type igraph_integer_t,
+ * to store n elements, where w is the bit width of \type igraph_int_t,
  * the integer type used throughout the library (either 32 or 64).
  * Sometimes they use more, this is because bitsets can
  * shrink, but even if they shrink, the current implementation does not free a
@@ -146,8 +146,6 @@ igraph_integer_t igraph_i_clz64(igraph_uint_t x) {
  * \function igraph_bitset_init
  * \brief Initializes a bitset object (constructor).
  *
- * \experimental
- *
  * </para><para>
  * Every bitset needs to be initialized before it can be used, and
  * there are a number of initialization functions or otherwise called
@@ -172,10 +170,10 @@ igraph_integer_t igraph_i_clz64(igraph_uint_t x) {
  * w is the word size of the machine (32 or 64).
  */
 
-igraph_error_t igraph_bitset_init(igraph_bitset_t *bitset, igraph_integer_t size) {
-    igraph_integer_t alloc_size = IGRAPH_BIT_NSLOTS(size);
+igraph_error_t igraph_bitset_init(igraph_bitset_t *bitset, igraph_int_t size) {
+    igraph_int_t alloc_size = IGRAPH_BIT_NSLOTS(size);
     bitset->stor_begin = IGRAPH_CALLOC(alloc_size, igraph_uint_t);
-    IGRAPH_CHECK_OOM(bitset->stor_begin, "Cannot initialize bitset");
+    IGRAPH_CHECK_OOM(bitset->stor_begin, "Cannot initialize bitset.");
     bitset->size = size;
     bitset->stor_end = bitset->stor_begin + alloc_size;
     return IGRAPH_SUCCESS;
@@ -186,8 +184,6 @@ igraph_error_t igraph_bitset_init(igraph_bitset_t *bitset, igraph_integer_t size
  * \function igraph_bitset_destroy
  * \brief Destroys a bitset object.
  *
- * \experimental
- *
  * All bitsets initialized by \ref igraph_bitset_init() should be properly
  * destroyed by this function. A destroyed bitset needs to be
  * reinitialized by \ref igraph_bitset_init() or
@@ -210,8 +206,6 @@ void igraph_bitset_destroy(igraph_bitset_t *bitset) {
  * \function igraph_bitset_init_copy
  * \brief Initializes a bitset from another bitset object (constructor).
  *
- * \experimental
- *
  * The contents of the existing bitset object will be copied to
  * the new one.
  *
@@ -230,7 +224,7 @@ igraph_error_t igraph_bitset_init_copy(igraph_bitset_t *dest, const igraph_bitse
     IGRAPH_ASSERT(src != NULL);
     IGRAPH_ASSERT(src->stor_begin != NULL);
     IGRAPH_CHECK(igraph_bitset_init(dest, src->size));
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = VECTOR(*src)[i];
     }
     return IGRAPH_SUCCESS;
@@ -241,8 +235,6 @@ igraph_error_t igraph_bitset_init_copy(igraph_bitset_t *dest, const igraph_bitse
  * \function igraph_bitset_update
  * \brief Update a bitset from another one.
  *
- * \experimental
- *
  * The size and contents of \p dest will be identical to that of \p src.
  *
  * \param dest Pointer to an initialized bitset object. This will be updated.
@@ -261,7 +253,7 @@ igraph_error_t igraph_bitset_update(igraph_bitset_t *dest, const igraph_bitset_t
     IGRAPH_ASSERT(src->stor_begin != NULL);
     IGRAPH_CHECK(igraph_bitset_reserve(dest, src->size));
     dest->size = src->size;
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = VECTOR(*src)[i];
     }
     return IGRAPH_SUCCESS;
@@ -272,8 +264,6 @@ igraph_error_t igraph_bitset_update(igraph_bitset_t *dest, const igraph_bitset_t
  * \function igraph_bitset_capacity
  * \brief Returns the allocated capacity of the bitset.
  *
- * \experimental
- *
  * Note that this might be different from the size of the bitset (as
  * queried by \ref igraph_bitset_size()), and specifies how many elements
  * the bitset can hold, without reallocation.
@@ -287,7 +277,7 @@ igraph_error_t igraph_bitset_update(igraph_bitset_t *dest, const igraph_bitset_t
  * Time complexity: O(1).
  */
 
-igraph_integer_t igraph_bitset_capacity(const igraph_bitset_t *bitset) {
+igraph_int_t igraph_bitset_capacity(const igraph_bitset_t *bitset) {
     return IGRAPH_INTEGER_SIZE * (bitset->stor_end - bitset->stor_begin);
 }
 
@@ -296,15 +286,13 @@ igraph_integer_t igraph_bitset_capacity(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_size
  * \brief Returns the length of the bitset.
  *
- * \experimental
- *
  * \param bitset The bitset object
  * \return The size of the bitset.
  *
  * Time complexity: O(1).
  */
 
-igraph_integer_t igraph_bitset_size(const igraph_bitset_t *bitset) {
+igraph_int_t igraph_bitset_size(const igraph_bitset_t *bitset) {
     return bitset->size;
 }
 
@@ -313,8 +301,6 @@ igraph_integer_t igraph_bitset_size(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_reserve
  * \brief Reserves memory for a bitset.
  *
- * \experimental
- *
  * \a igraph bitsets are flexible, they can grow and
  * shrink. Growing
  * however occasionally needs the data in the bitset to be copied.
@@ -339,8 +325,8 @@ igraph_integer_t igraph_bitset_size(const igraph_bitset_t *bitset) {
  * w is the word size of the machine (32 or 64).
  */
 
-igraph_error_t igraph_bitset_reserve(igraph_bitset_t *bitset, igraph_integer_t capacity) {
-    igraph_integer_t current_capacity;
+igraph_error_t igraph_bitset_reserve(igraph_bitset_t *bitset, igraph_int_t capacity) {
+    igraph_int_t current_capacity;
     igraph_uint_t *tmp;
 
     IGRAPH_ASSERT(bitset != NULL);
@@ -367,8 +353,6 @@ igraph_error_t igraph_bitset_reserve(igraph_bitset_t *bitset, igraph_integer_t c
  * \function igraph_bitset_resize
  * \brief Resizes the bitset.
  *
- * \experimental
- *
  * Note that this function does not free any memory, just sets the
  * size of the bitset to the given one. It may, on the other hand,
  * allocate more memory if the new size is larger than the previous
@@ -392,13 +376,13 @@ igraph_error_t igraph_bitset_reserve(igraph_bitset_t *bitset, igraph_integer_t c
  * w is the word size of the machine (32 or 64).
  */
 
-igraph_error_t igraph_bitset_resize(igraph_bitset_t *bitset, igraph_integer_t new_size) {
+igraph_error_t igraph_bitset_resize(igraph_bitset_t *bitset, igraph_int_t new_size) {
     IGRAPH_ASSERT(bitset != NULL);
     IGRAPH_ASSERT(bitset->stor_begin != NULL);
     IGRAPH_CHECK(igraph_bitset_reserve(bitset, new_size));
 
     if (new_size > bitset->size) {
-        for (igraph_integer_t i = bitset->size; i % IGRAPH_INTEGER_SIZE != 0; ++i) {
+        for (igraph_int_t i = bitset->size; i % IGRAPH_INTEGER_SIZE != 0; ++i) {
             IGRAPH_BIT_CLEAR(*bitset, i);
         }
         memset(bitset->stor_begin + IGRAPH_BIT_NSLOTS(bitset->size), 0,
@@ -414,8 +398,6 @@ igraph_error_t igraph_bitset_resize(igraph_bitset_t *bitset, igraph_integer_t ne
  * \function igraph_bitset_popcount
  * \brief The population count of the bitset.
  *
- * \experimental
- *
  * Returns the number of set bits, also called the population count,
  * of the bitset.
  *
@@ -425,14 +407,14 @@ igraph_error_t igraph_bitset_resize(igraph_bitset_t *bitset, igraph_integer_t ne
  * Time complexity: O(n/w).
  */
 
-igraph_integer_t igraph_bitset_popcount(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+igraph_int_t igraph_bitset_popcount(const igraph_bitset_t *bitset) {
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? ~zero : ((one << final_block_size) - 1);
-    igraph_integer_t count = 0;
+    igraph_int_t count = 0;
 
-    for (igraph_integer_t i = 0; i + 1 < slots; ++i) {
+    for (igraph_int_t i = 0; i + 1 < slots; ++i) {
         count += IGRAPH_POPCOUNT(VECTOR(*bitset)[i]);
     }
     if (bitset->size) {
@@ -447,8 +429,6 @@ igraph_integer_t igraph_bitset_popcount(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_countl_zero
  * \brief The number of leading zeros in the bitset.
  *
- * \experimental
- *
  * Returns the number of leading (starting at the most significant bit)
  * zeros in the bitset before the first one is encountered. If the bitset
  * is all zeros, then its size is returned.
@@ -459,17 +439,17 @@ igraph_integer_t igraph_bitset_popcount(const igraph_bitset_t *bitset) {
  * Time complexity: O(n/w).
  */
 
-igraph_integer_t igraph_bitset_countl_zero(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t padding = IGRAPH_INTEGER_SIZE - final_block_size;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+igraph_int_t igraph_bitset_countl_zero(const igraph_bitset_t *bitset) {
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t padding = IGRAPH_INTEGER_SIZE - final_block_size;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0;
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? ~zero : ((one << final_block_size) - one);
 
     if (bitset->size && (mask & VECTOR(*bitset)[slots - 1]) != 0) {
         return IGRAPH_CLZ(mask & VECTOR(*bitset)[slots - 1]) - padding;
     }
-    for (igraph_integer_t i = 1; i < slots; ++i) {
+    for (igraph_int_t i = 1; i < slots; ++i) {
         if (VECTOR(*bitset)[slots - i - 1] != 0) {
             return IGRAPH_INTEGER_SIZE * i + IGRAPH_CLZ(VECTOR(*bitset)[slots - i - 1]) - padding;
         }
@@ -483,8 +463,6 @@ igraph_integer_t igraph_bitset_countl_zero(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_countl_one
  * \brief The number of leading ones in the bitset.
  *
- * \experimental
- *
  * Returns the number of leading ones (starting at the most significant bit)
  * in the bitset before the first zero is encountered.
  * If the bitset is all ones, then its size is returned.
@@ -495,17 +473,17 @@ igraph_integer_t igraph_bitset_countl_zero(const igraph_bitset_t *bitset) {
  * Time complexity: O(n/w).
  */
 
-igraph_integer_t igraph_bitset_countl_one(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t padding = IGRAPH_INTEGER_SIZE - final_block_size;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+igraph_int_t igraph_bitset_countl_one(const igraph_bitset_t *bitset) {
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t padding = IGRAPH_INTEGER_SIZE - final_block_size;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? zero : ~((one << final_block_size) - one);
 
     if (bitset->size && (mask | VECTOR(*bitset)[slots - 1]) != ~zero) {
         return IGRAPH_CLO(mask | VECTOR(*bitset)[slots - 1]) - padding;
     }
-    for (igraph_integer_t i = 1; i < slots; ++i) {
+    for (igraph_int_t i = 1; i < slots; ++i) {
         if (VECTOR(*bitset)[slots - i - 1] != ~zero) {
             return IGRAPH_INTEGER_SIZE * i + IGRAPH_CLO(VECTOR(*bitset)[slots - i - 1]) - padding;
         }
@@ -519,8 +497,6 @@ igraph_integer_t igraph_bitset_countl_one(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_countr_zero
  * \brief The number of trailing zeros in the bitset.
  *
- * \experimental
- *
  * Returns the number of trailing (starting at the least significant bit)
  * zeros in the bitset before the first one is encountered.
  * If the bitset is all zeros, then its size is returned.
@@ -531,13 +507,13 @@ igraph_integer_t igraph_bitset_countl_one(const igraph_bitset_t *bitset) {
  * Time complexity: O(n/w).
  */
 
-igraph_integer_t igraph_bitset_countr_zero(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+igraph_int_t igraph_bitset_countr_zero(const igraph_bitset_t *bitset) {
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? ~zero : ((one << final_block_size) - one);
 
-    for (igraph_integer_t i = 0; i + 1 < slots; ++i) {
+    for (igraph_int_t i = 0; i + 1 < slots; ++i) {
         if (VECTOR(*bitset)[i] != zero) {
             return IGRAPH_INTEGER_SIZE * i + IGRAPH_CTZ(VECTOR(*bitset)[i]);
         }
@@ -554,8 +530,6 @@ igraph_integer_t igraph_bitset_countr_zero(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_countr_one
  * \brief The number of trailing ones in the bitset.
  *
- * \experimental
- *
  * Returns the number of trailing ones (starting at the least significant bit)
  * in the bitset before the first zero is encountered.
  * If the bitset is all ones, then its size is returned.
@@ -566,13 +540,13 @@ igraph_integer_t igraph_bitset_countr_zero(const igraph_bitset_t *bitset) {
  * Time complexity: O(n/w).
  */
 
-igraph_integer_t igraph_bitset_countr_one(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+igraph_int_t igraph_bitset_countr_one(const igraph_bitset_t *bitset) {
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? zero : ~((one << final_block_size) - one);
 
-    for (igraph_integer_t i = 0; i + 1 < slots; ++i) {
+    for (igraph_int_t i = 0; i + 1 < slots; ++i) {
         if (VECTOR(*bitset)[i] != ~zero) {
             return IGRAPH_INTEGER_SIZE * i + IGRAPH_CTO(VECTOR(*bitset)[i]);
         }
@@ -589,8 +563,6 @@ igraph_integer_t igraph_bitset_countr_one(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_is_all_zero
  * \brief Are all bits zeros?
  *
- * \experimental
- *
  * \param bitset The bitset object to test.
  * \return True if none of the bits are set.
  *
@@ -598,12 +570,12 @@ igraph_integer_t igraph_bitset_countr_one(const igraph_bitset_t *bitset) {
  */
 
 igraph_bool_t igraph_bitset_is_all_zero(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? ~zero : ((one << final_block_size) - one);
 
-    for (igraph_integer_t i = 0; i < slots - 1; i++) {
+    for (igraph_int_t i = 0; i < slots - 1; i++) {
         if (VECTOR(*bitset)[i] != zero) {
             return false;
         }
@@ -619,8 +591,6 @@ igraph_bool_t igraph_bitset_is_all_zero(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_is_all_one
  * \brief Are all bits ones?
  *
- * \experimental
- *
  * \param bitset The bitset object to test.
  * \return True if all of the bits are set.
  *
@@ -628,12 +598,12 @@ igraph_bool_t igraph_bitset_is_all_zero(const igraph_bitset_t *bitset) {
  */
 
 igraph_bool_t igraph_bitset_is_all_one(const igraph_bitset_t *bitset) {
-    const igraph_integer_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
-    const igraph_integer_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
+    const igraph_int_t final_block_size = bitset->size % IGRAPH_INTEGER_SIZE ? bitset->size % IGRAPH_INTEGER_SIZE : IGRAPH_INTEGER_SIZE;
+    const igraph_int_t slots = IGRAPH_BIT_NSLOTS(bitset->size);
     const igraph_uint_t one = 1, zero = 0; /* to avoid the need to cast 1 and 0 to igraph_uint_t below */
     const igraph_uint_t mask = final_block_size == IGRAPH_INTEGER_SIZE ? zero : ~((one << final_block_size) - one);
 
-    for (igraph_integer_t i = 0; i < slots - 1; i++) {
+    for (igraph_int_t i = 0; i < slots - 1; i++) {
         if (VECTOR(*bitset)[i] != ~zero) {
             return false;
         }
@@ -649,8 +619,6 @@ igraph_bool_t igraph_bitset_is_all_one(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_is_any_zero
  * \brief Are any bits zeros?
  *
- * \experimental
- *
  * \param bitset The bitset object to test.
  * \return True if at least one bit is zero.
  *
@@ -666,8 +634,6 @@ igraph_bool_t igraph_bitset_is_any_zero(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_is_any_one
  * \brief Are any bits ones?
  *
- * \experimental
- *
  * \param bitset The bitset object to test.
  * \return True if at least one bit is one.
  *
@@ -683,8 +649,6 @@ igraph_bool_t igraph_bitset_is_any_one(const igraph_bitset_t *bitset) {
  * \function igraph_bitset_or
  * \brief Bitwise OR of two bitsets.
  *
- * \experimental
- *
  * Applies a bitwise or to the contents of two bitsets and stores it in an
  * already initialized bitset. The destination bitset may be equal to one
  * (or even both) of the sources. When working with bitsets, it is common
@@ -701,7 +665,7 @@ igraph_bool_t igraph_bitset_is_any_one(const igraph_bitset_t *bitset) {
 
 void igraph_bitset_or(igraph_bitset_t *dest,
                       const igraph_bitset_t *src1, const igraph_bitset_t *src2) {
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = VECTOR(*src1)[i] | VECTOR(*src2)[i];
     }
 }
@@ -711,8 +675,6 @@ void igraph_bitset_or(igraph_bitset_t *dest,
  * \function igraph_bitset_and
  * \brief Bitwise AND of two bitsets.
  *
- * \experimental
- *
  * Applies a bitwise and to the contents of two bitsets and stores it in an
  * already initialized bitset. The destination bitset may be equal to one
  * (or even both) of the sources. When working with bitsets, it is common
@@ -728,7 +690,7 @@ void igraph_bitset_or(igraph_bitset_t *dest,
  */
 
 void igraph_bitset_and(igraph_bitset_t *dest, const igraph_bitset_t *src1, const igraph_bitset_t *src2) {
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = VECTOR(*src1)[i] & VECTOR(*src2)[i];
     }
 }
@@ -738,8 +700,6 @@ void igraph_bitset_and(igraph_bitset_t *dest, const igraph_bitset_t *src1, const
  * \function igraph_bitset_xor
  * \brief Bitwise XOR of two bitsets.
  *
- * \experimental
- *
  * Applies a bitwise xor to the contents of two bitsets and stores it in
  * an already initialized bitset. The destination bitset may be equal to
  * one (or even both) of the sources. When working with bitsets, it is common
@@ -756,7 +716,7 @@ void igraph_bitset_and(igraph_bitset_t *dest, const igraph_bitset_t *src1, const
 
 void igraph_bitset_xor(igraph_bitset_t *dest,
                        const igraph_bitset_t *src1, const igraph_bitset_t *src2) {
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = VECTOR(*src1)[i] ^ VECTOR(*src2)[i];
     }
 }
@@ -766,8 +726,6 @@ void igraph_bitset_xor(igraph_bitset_t *dest,
  * \function igraph_bitset_not
  * \brief Bitwise negation of a bitset.
  *
- * \experimental
- *
  * Applies a bitwise not to the contents of a bitset and stores it in an
  * already initialized bitset. The destination bitset may be equal to the
  * source. When working with bitsets, it is common that those created are
@@ -781,7 +739,7 @@ void igraph_bitset_xor(igraph_bitset_t *dest,
  */
 
 void igraph_bitset_not(igraph_bitset_t *dest, const igraph_bitset_t *src) {
-    for (igraph_integer_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
+    for (igraph_int_t i = 0; i < IGRAPH_BIT_NSLOTS(dest->size); ++i) {
         VECTOR(*dest)[i] = ~VECTOR(*src)[i];
     }
 }
@@ -791,8 +749,6 @@ void igraph_bitset_not(igraph_bitset_t *dest, const igraph_bitset_t *src) {
  * \function igraph_bitset_fill
  * \brief Fills a bitset with a constant value.
  *
- * \experimental
- *
  * Sets all bits of a bitset to the same value.
  *
  * \param bitset The bitset object to modify.
@@ -814,8 +770,6 @@ void igraph_bitset_fill(igraph_bitset_t *bitset, igraph_bool_t value) {
  * \function igraph_bitset_null
  * \brief Clears all bits in a bitset.
  *
- * \experimental
- *
  * \param bitset The bitset object to clear all bits in.
  *
  * \sa \ref igraph_bitset_fill()
@@ -832,8 +786,6 @@ void igraph_bitset_null(igraph_bitset_t *bitset) {
  * \function igraph_bitset_fprint
  * \brief Prints the bits of a bitset.
  *
- * \experimental
- *
  * Outputs the contents of a bitset to a file.
  * The bitset is written from index n-1 to index 0, left to right,
  * such that index 0 is the least significant bit and index n-1 is
@@ -852,7 +804,7 @@ void igraph_bitset_null(igraph_bitset_t *bitset) {
  * Time complexity: O(n).
  */
 igraph_error_t igraph_bitset_fprint(const igraph_bitset_t *bitset, FILE *file) {
-    for (igraph_integer_t i = bitset->size - 1; i >= 0; i--) {
+    for (igraph_int_t i = bitset->size - 1; i >= 0; i--) {
         fputc(IGRAPH_BIT_TEST(*bitset, i) ? '1' : '0', file);
     }
     return IGRAPH_SUCCESS;
