@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -21,8 +20,9 @@
 
 */
 
-#include "igraph_types.h"
 #include "igraph_strvector.h"
+
+#include "igraph_types.h"
 #include "igraph_memory.h"
 #include "igraph_error.h"
 
@@ -62,31 +62,17 @@
  * All elements of the string vector are set to the empty string.
  *
  * \param sv Pointer to an initialized string vector.
- * \param len The (initial) length of the string vector.
+ * \param size The (initial) length of the string vector.
  * \return Error code.
  *
  * Time complexity: O(\p len).
  */
 
-igraph_error_t igraph_strvector_init(igraph_strvector_t *sv, igraph_integer_t size) {
+igraph_error_t igraph_strvector_init(igraph_strvector_t *sv, igraph_int_t size) {
 
-    sv->stor_begin = IGRAPH_CALLOC(size, char*);
+    sv->stor_begin = IGRAPH_CALLOC(size, const char *);
     IGRAPH_CHECK_OOM(sv->stor_begin, "Cannot initialize string vector.");
 
-    for (igraph_integer_t i = 0; i < size; i++) {
-        sv->stor_begin[i] = IGRAPH_CALLOC(1, char);
-        if (sv->stor_begin[i] == NULL) {
-            /* LCOV_EXCL_START */
-            for (igraph_integer_t j = 0; j < i; j++) {
-                IGRAPH_FREE(sv->stor_begin[j]);
-            }
-            IGRAPH_FREE(sv->stor_begin);
-            IGRAPH_ERROR("Cannot initialize string vector.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-            /* LCOV_EXCL_STOP */
-        }
-        sv->stor_begin[i][0] = '\0';
-    }
-
     sv->stor_end = sv->stor_begin + size;
     sv->end = sv->stor_end;
 
@@ -107,10 +93,9 @@ igraph_error_t igraph_strvector_init(igraph_strvector_t *sv, igraph_integer_t si
  */
 
 void igraph_strvector_destroy(igraph_strvector_t *sv) {
-    char **ptr;
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
-    for (ptr = sv->stor_begin; ptr < sv->end; ptr++) {
+    for (const char **ptr = sv->stor_begin; ptr < sv->end; ptr++) {
         IGRAPH_FREE(*ptr);
     }
     IGRAPH_FREE(sv->stor_begin);
@@ -119,10 +104,9 @@ void igraph_strvector_destroy(igraph_strvector_t *sv) {
 /**
  * \ingroup strvector
  * \function igraph_strvector_get
- * \brief Retrieves an element of the string vector.
+ * \brief Retrieves an element of a string vector.
  *
- * Query an element of a string vector. See also the \ref STR macro
- * for an easier way.
+ * Query an element of a string vector. The returned string must not be modified.
  *
  * \param sv The input string vector.
  * \param idx The index of the element to query.
@@ -130,11 +114,10 @@ void igraph_strvector_destroy(igraph_strvector_t *sv) {
  * Time complexity: O(1).
  */
 
-const char *igraph_strvector_get(const igraph_strvector_t *sv, igraph_integer_t idx) {
+const char *igraph_strvector_get(const igraph_strvector_t *sv, igraph_int_t idx) {
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
-    IGRAPH_ASSERT(sv->stor_begin[idx] != NULL);
-    return sv->stor_begin[idx];
+    return sv->stor_begin[idx] ? sv->stor_begin[idx] : "";
 }
 
 /**
@@ -154,7 +137,7 @@ const char *igraph_strvector_get(const igraph_strvector_t *sv, igraph_integer_t
  * depending on the memory management, if reallocation is needed.
  */
 
-igraph_error_t igraph_strvector_set(igraph_strvector_t *sv, igraph_integer_t idx,
+igraph_error_t igraph_strvector_set(igraph_strvector_t *sv, igraph_int_t idx,
                          const char *value) {
     return igraph_strvector_set_len(sv, idx, value, strlen(value));
 }
@@ -176,20 +159,22 @@ igraph_error_t igraph_strvector_set(igraph_strvector_t *sv, igraph_integer_t idx
  * Time complexity: O(l), the length of the new string. Maybe more,
  * depending on the memory management, if reallocation is needed.
  */
-igraph_error_t igraph_strvector_set_len(igraph_strvector_t *sv, igraph_integer_t idx,
+igraph_error_t igraph_strvector_set_len(igraph_strvector_t *sv, igraph_int_t idx,
                           const char *value, size_t len) {
-    char *tmp;
-
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
-    IGRAPH_ASSERT(sv->stor_begin[idx] != NULL);
 
-    tmp = IGRAPH_REALLOC(sv->stor_begin[idx], len + 1, char);
-    IGRAPH_CHECK_OOM(tmp, "Cannot reserve space for new item in string vector.");
+    if (sv->stor_begin[idx] == NULL) {
+        sv->stor_begin[idx] = strndup(value, len);
+        IGRAPH_CHECK_OOM(sv->stor_begin[idx], "Cannot reserve space for new item in string vector.");
+    } else {
+        char *tmp = IGRAPH_REALLOC(sv->stor_begin[idx], len + 1, char);
+        IGRAPH_CHECK_OOM(tmp, "Cannot reserve space for new item in string vector.");
 
-    sv->stor_begin[idx] = tmp;
-    memcpy(sv->stor_begin[idx], value, len * sizeof(char));
-    sv->stor_begin[idx][len] = '\0';
+        memcpy(tmp, value, len * sizeof(char));
+        tmp[len] = '\0';
+        sv->stor_begin[idx] = tmp;
+    }
 
     return IGRAPH_SUCCESS;
 }
@@ -207,9 +192,9 @@ igraph_error_t igraph_strvector_set_len(igraph_strvector_t *sv, igraph_integer_t
  */
 
 void igraph_strvector_remove_section(
-        igraph_strvector_t *sv, igraph_integer_t from, igraph_integer_t to) {
-    igraph_integer_t size = igraph_strvector_size(sv);
-    igraph_integer_t i;
+        igraph_strvector_t *sv, igraph_int_t from, igraph_int_t to) {
+    igraph_int_t size = igraph_strvector_size(sv);
+    igraph_int_t i;
 
     if (from < 0) {
         from = 0;
@@ -242,7 +227,7 @@ void igraph_strvector_remove_section(
  * Time complexity: O(n), the length of the string.
  */
 
-void igraph_strvector_remove(igraph_strvector_t *sv, igraph_integer_t elem) {
+void igraph_strvector_remove(igraph_strvector_t *sv, igraph_int_t elem) {
     igraph_strvector_remove_section(sv, elem, elem + 1);
 }
 
@@ -262,20 +247,25 @@ void igraph_strvector_remove(igraph_strvector_t *sv, igraph_integer_t elem) {
 
 igraph_error_t igraph_strvector_init_copy(igraph_strvector_t *to,
                                           const igraph_strvector_t *from) {
-    igraph_integer_t from_size = igraph_strvector_size(from);
+    igraph_int_t from_size = igraph_strvector_size(from);
 
-    to->stor_begin = IGRAPH_CALLOC(from_size, char*);
+    to->stor_begin = IGRAPH_CALLOC(from_size, const char *);
     IGRAPH_CHECK_OOM(to->stor_begin, "Cannot copy string vector.");
 
-    for (igraph_integer_t i = 0; i < from_size; i++) {
-        to->stor_begin[i] = strdup(igraph_strvector_get(from, i));
+    for (igraph_int_t i = 0; i < from_size; i++) {
+        /* If the string in the 'from' vector is empty, we represent it as NULL.
+         * The NULL value was already set by IGRAPH_CALLOC(). */
+        if (from->stor_begin[i] == NULL || from->stor_begin[i][0] == '\0') {
+            continue;
+        }
+        to->stor_begin[i] = strdup(from->stor_begin[i]);
         if (to->stor_begin[i] == NULL) {
             /* LCOV_EXCL_START */
-            for (igraph_integer_t j = 0; j < i; j++) {
+            for (igraph_int_t j = 0; j < i; j++) {
                 IGRAPH_FREE(to->stor_begin[j]);
             }
             IGRAPH_FREE(to->stor_begin);
-            IGRAPH_ERROR("Cannot copy string vector.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
+            IGRAPH_ERROR("Cannot copy string vector.", IGRAPH_ENOMEM);
             /* LCOV_EXCL_STOP */
         }
     }
@@ -286,18 +276,6 @@ igraph_error_t igraph_strvector_init_copy(igraph_strvector_t *to,
     return IGRAPH_SUCCESS;
 }
 
-/**
- * \ingroup strvector
- * \function igraph_strvector_copy
- * \brief Initialization by copying (deprecated alias).
- *
- * \deprecated-by igraph_strvector_init_copy 0.10.0
- */
-
-igraph_error_t igraph_strvector_copy(igraph_strvector_t *to,
-                          const igraph_strvector_t *from) {
-    return igraph_strvector_init_copy(to, from);
-}
 
 /**
  * \function igraph_strvector_append
@@ -319,28 +297,41 @@ igraph_error_t igraph_strvector_copy(igraph_strvector_t *to,
  */
 
 igraph_error_t igraph_strvector_append(igraph_strvector_t *to,
-                            const igraph_strvector_t *from) {
-    igraph_integer_t len1 = igraph_strvector_size(to), len2 = igraph_strvector_size(from);
-    igraph_integer_t newlen;
+                                       const igraph_strvector_t *from) {
+    const igraph_int_t to_size = igraph_strvector_size(to);
+    const igraph_int_t from_size = igraph_strvector_size(from);
+    const igraph_int_t to_capacity = igraph_strvector_capacity(to);
+    igraph_int_t new_to_size;
     igraph_bool_t error = false;
-    char *tmp;
+    const char *tmp;
 
-    IGRAPH_SAFE_ADD(len1, len2, &newlen);
-    IGRAPH_CHECK(igraph_strvector_reserve(to, newlen));
+    IGRAPH_SAFE_ADD(to_size, from_size, &new_to_size);
 
-    for (igraph_integer_t i = 0; i < len2; i++) {
-        tmp = strdup(igraph_strvector_get(from, i));
-        if (!tmp) {
-            error = true;
-            break;
-        } else {
-            *(to->end) = tmp;
-            to->end++;
+    if (to_capacity < new_to_size) {
+        igraph_int_t new_to_capacity = to_capacity < IGRAPH_INTEGER_MAX/2 ? to_capacity * 2 : IGRAPH_INTEGER_MAX;
+        if (new_to_capacity < new_to_size) {
+            new_to_capacity = new_to_size;
         }
+        IGRAPH_CHECK(igraph_strvector_reserve(to, new_to_capacity));
+    }
+
+    for (igraph_int_t i = 0; i < from_size; i++) {
+        if (from->stor_begin[i] == NULL || from->stor_begin[i][0] == '\0') {
+            /* Represent empty strings as NULL. */
+            tmp = NULL;
+        } else {
+            tmp = strdup(from->stor_begin[i]);
+            if (tmp == NULL) {
+                error = true;
+                break;
+            }
+        }
+        *(to->end) = tmp;
+        to->end++;
     }
 
     if (error) {
-        igraph_strvector_resize(to, len1); /* always shrinks */
+        igraph_strvector_resize(to, to_size); /* always shrinks */
         IGRAPH_ERROR("Cannot append string vector.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
 
@@ -367,8 +358,8 @@ igraph_error_t igraph_strvector_append(igraph_strvector_t *to,
  *   where l1 and l2 are the lengths of \p to and \from respectively.
  */
 igraph_error_t igraph_strvector_merge(igraph_strvector_t *to, igraph_strvector_t *from) {
-    char **p1, **p2, **pe;
-    igraph_integer_t newlen;
+    const char **p1, **p2, **pe;
+    igraph_int_t newlen;
 
     IGRAPH_SAFE_ADD(igraph_strvector_size(to), igraph_strvector_size(from), &newlen);
     IGRAPH_CHECK(igraph_strvector_reserve(to, newlen));
@@ -400,9 +391,9 @@ igraph_error_t igraph_strvector_merge(igraph_strvector_t *to, igraph_strvector_t
  */
 
 void igraph_strvector_clear(igraph_strvector_t *sv) {
-    igraph_integer_t n = igraph_strvector_size(sv);
+    igraph_int_t n = igraph_strvector_size(sv);
 
-    for (igraph_integer_t i = 0; i < n; i++) {
+    for (igraph_int_t i = 0; i < n; i++) {
         IGRAPH_FREE(sv->stor_begin[i]);
     }
     sv->end = sv->stor_begin;
@@ -425,30 +416,18 @@ void igraph_strvector_clear(igraph_strvector_t *sv) {
  * smaller, maybe less, depending on memory management.
  */
 
-igraph_error_t igraph_strvector_resize(igraph_strvector_t *sv, igraph_integer_t newsize) {
-    igraph_integer_t toadd = newsize - igraph_strvector_size(sv);
-    igraph_integer_t oldsize = igraph_strvector_size(sv);
+igraph_error_t igraph_strvector_resize(igraph_strvector_t *sv, igraph_int_t newsize) {
+    igraph_int_t toadd = newsize - igraph_strvector_size(sv);
+    igraph_int_t oldsize = igraph_strvector_size(sv);
 
     if (newsize < oldsize) {
-        for (igraph_integer_t i = newsize; i < oldsize; i++) {
+        for (igraph_int_t i = newsize; i < oldsize; i++) {
             IGRAPH_FREE(sv->stor_begin[i]);
         }
         sv->end = sv->stor_begin + newsize;
     } else if (newsize > oldsize) {
         IGRAPH_CHECK(igraph_strvector_reserve(sv, newsize));
-
-        for (igraph_integer_t i = 0; i < toadd; i++) {
-            sv->stor_begin[oldsize + i] = IGRAPH_CALLOC(1, char);
-            if (sv->stor_begin[oldsize + i] == NULL) {
-                /* LCOV_EXCL_START */
-                for (igraph_integer_t j = 0; j < i; j++) {
-                    IGRAPH_FREE(sv->stor_begin[oldsize + j]);
-                }
-                IGRAPH_ERROR("Cannot resize string vector.", IGRAPH_ENOMEM);
-                /* LCOV_EXCL_STOP */
-            }
-            sv->stor_begin[oldsize + i][0] = '\0';
-        }
+        memset(sv->stor_begin + oldsize, 0, toadd * sizeof(const char *));
         sv->end = sv->stor_begin + newsize;
     }
 
@@ -466,7 +445,7 @@ igraph_error_t igraph_strvector_resize(igraph_strvector_t *sv, igraph_integer_t
  * Time complexity: O(1).
  */
 
-igraph_integer_t igraph_strvector_capacity(const igraph_strvector_t *sv) {
+igraph_int_t igraph_strvector_capacity(const igraph_strvector_t *sv) {
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
     return sv->stor_end - sv->stor_begin;
@@ -499,15 +478,14 @@ igraph_integer_t igraph_strvector_capacity(const igraph_strvector_t *sv) {
  * O(n), n is the new allocated size of the vector.
  */
 
-igraph_error_t igraph_strvector_reserve(igraph_strvector_t *sv, igraph_integer_t capacity) {
-    igraph_integer_t current_capacity = igraph_strvector_capacity(sv);
-    char **tmp;
+igraph_error_t igraph_strvector_reserve(igraph_strvector_t *sv, igraph_int_t capacity) {
+    igraph_int_t current_capacity = igraph_strvector_capacity(sv);
 
     if (capacity <= current_capacity) {
         return IGRAPH_SUCCESS;
     }
 
-    tmp = IGRAPH_REALLOC(sv->stor_begin, capacity, char *);
+    const char **tmp = IGRAPH_REALLOC(sv->stor_begin, capacity, const char *);
     IGRAPH_CHECK_OOM(tmp, "Cannot reserve space for new items in string vector.");
 
     sv->end = tmp + (sv->end - sv->stor_begin);
@@ -533,14 +511,12 @@ igraph_error_t igraph_strvector_reserve(igraph_strvector_t *sv, igraph_integer_t
  */
 
 void igraph_strvector_resize_min(igraph_strvector_t *sv) {
-    igraph_integer_t size;
-    char **tmp;
     if (sv->stor_end == sv->end) {
         return;
     }
 
-    size = (sv->end - sv->stor_begin);
-    tmp = IGRAPH_REALLOC(sv->stor_begin, size, char *);
+    const igraph_int_t size = (sv->end - sv->stor_begin);
+    const char **tmp = IGRAPH_REALLOC(sv->stor_begin, size, const char *);
 
     if (tmp != NULL) {
         sv->stor_begin = tmp;
@@ -559,7 +535,7 @@ void igraph_strvector_resize_min(igraph_strvector_t *sv) {
  * Time complexity: O(1).
  */
 
-igraph_integer_t igraph_strvector_size(const igraph_strvector_t *sv) {
+igraph_int_t igraph_strvector_size(const igraph_strvector_t *sv) {
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
     return sv->end - sv->stor_begin;
@@ -569,13 +545,13 @@ igraph_integer_t igraph_strvector_size(const igraph_strvector_t *sv) {
  * Ensures that the vector has at least one extra slot at the end of its
  * allocated storage area.
  */
-static igraph_error_t igraph_i_strvector_expand_if_full(igraph_strvector_t *sv) {
+static igraph_error_t strvector_expand_if_full(igraph_strvector_t *sv) {
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
 
     if (sv->stor_end == sv->end) {
-        igraph_integer_t old_size = igraph_strvector_size(sv);
-        igraph_integer_t new_size = old_size < IGRAPH_INTEGER_MAX/2 ? old_size * 2 : IGRAPH_INTEGER_MAX;
+        igraph_int_t old_size = igraph_strvector_size(sv);
+        igraph_int_t new_size = old_size < IGRAPH_INTEGER_MAX/2 ? old_size * 2 : IGRAPH_INTEGER_MAX;
         if (old_size == IGRAPH_INTEGER_MAX) {
             IGRAPH_ERROR("Cannot add new item to string vector, already at maximum size.", IGRAPH_EOVERFLOW);
         }
@@ -602,8 +578,8 @@ static igraph_error_t igraph_i_strvector_expand_if_full(igraph_strvector_t *sv)
  */
 
 igraph_error_t igraph_strvector_push_back(igraph_strvector_t *sv, const char *value) {
-    IGRAPH_CHECK(igraph_i_strvector_expand_if_full(sv));
-    char *tmp = strdup(value);
+    IGRAPH_CHECK(strvector_expand_if_full(sv));
+    const char *tmp = strdup(value);
     IGRAPH_CHECK_OOM(tmp, "Cannot push new string to string vector.");
     *sv->end = tmp;
     sv->end++;
@@ -627,43 +603,39 @@ igraph_error_t igraph_strvector_push_back(igraph_strvector_t *sv, const char *va
 
 igraph_error_t igraph_strvector_push_back_len(
         igraph_strvector_t *sv,
-        const char *value, igraph_integer_t len) {
+        const char *value, size_t len) {
 
-    IGRAPH_CHECK(igraph_i_strvector_expand_if_full(sv));
-    char *tmp = strndup(value, len);
-    if (! tmp) {
-        IGRAPH_ERROR("Cannot add string to string vector.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    IGRAPH_CHECK(strvector_expand_if_full(sv));
+    const char *tmp = strndup(value, len);
+    IGRAPH_CHECK_OOM(tmp, "Insufficient memory to push to string vector.");
     *sv->end = tmp;
     sv->end++;
 
     return IGRAPH_SUCCESS;
 }
 
-/**
- * \ingroup strvector
- * \function igraph_strvector_add
- * \brief Adds an element to the back of a string vector (deprecated alias).
- *
- * \deprecated-by igraph_strvector_push_back 0.10.0
- */
-
-igraph_error_t igraph_strvector_add(igraph_strvector_t *sv, const char *value) {
-    return igraph_strvector_push_back(sv, value);
-}
 
 /**
  * \ingroup strvector
- * \function igraph_strvector_set2
- * \brief Sets an element of the string vector given a buffer and its size (deprecated alias).
+ * \function igraph_strvector_print
+ * \brief Prints a string vector to a file.
  *
- * \deprecated-by igraph_strvector_set_len 0.10.0
+ * \param sv The string vector.
+ * \param file The file to write to.
+ * \param sep The separator to print between strings.
+ * \return Error code.
  */
+igraph_error_t igraph_strvector_fprint(const igraph_strvector_t *sv, FILE *file,
+                                       const char *sep) {
 
-igraph_error_t igraph_strvector_set2(
-    igraph_strvector_t *sv, igraph_integer_t idx, const char *value, size_t len
-) {
-    return igraph_strvector_set_len(sv, idx, value, len);
+    const igraph_int_t n = igraph_strvector_size(sv);
+    if (n != 0) {
+        fprintf(file, "%s", igraph_strvector_get(sv, 0));
+    }
+    for (igraph_int_t i = 1; i < n; i++) {
+        fprintf(file, "%s%s", sep, igraph_strvector_get(sv, i));
+    }
+    return IGRAPH_SUCCESS;
 }
 
 /**
@@ -672,26 +644,19 @@ igraph_error_t igraph_strvector_set2(
  * \brief Prints a string vector.
  *
  * \param sv The string vector.
- * \param file The file to write to.
  * \param sep The separator to print between strings.
  * \return Error code.
  */
-igraph_error_t igraph_strvector_print(const igraph_strvector_t *sv, FILE *file,
-                           const char *sep) {
-
-    igraph_integer_t n = igraph_strvector_size(sv);
-    if (n != 0) {
-        fprintf(file, "%s", STR(*sv, 0));
-    }
-    for (igraph_integer_t i = 1; i < n; i++) {
-        fprintf(file, "%s%s", sep, STR(*sv, i));
-    }
-    return IGRAPH_SUCCESS;
+#ifndef USING_R
+igraph_error_t igraph_strvector_print(const igraph_strvector_t *sv,
+                                      const char *sep) {
+    return igraph_strvector_fprint(sv, stdout, sep);
 }
+#endif
 
 /**
  * \ingroup strvector
- * \function igraph_strvector_set
+ * \function igraph_strvector_index
  * \brief Takes elements at given positions from a string vector.
  *
  * \param sv   The string vector.
@@ -703,14 +668,71 @@ igraph_error_t igraph_strvector_index(const igraph_strvector_t *sv,
                            igraph_strvector_t *newv,
                            const igraph_vector_int_t *idx) {
 
-    igraph_integer_t newlen = igraph_vector_int_size(idx);
+    igraph_int_t newlen = igraph_vector_int_size(idx);
     IGRAPH_CHECK(igraph_strvector_resize(newv, newlen));
 
-    for (igraph_integer_t i = 0; i < newlen; i++) {
-        igraph_integer_t j = VECTOR(*idx)[i];
+    for (igraph_int_t i = 0; i < newlen; i++) {
+        igraph_int_t j = VECTOR(*idx)[i];
         const char *str = igraph_strvector_get(sv, j);
         IGRAPH_CHECK(igraph_strvector_set(newv, i, str));
     }
 
     return IGRAPH_SUCCESS;
 }
+
+/**
+ * \ingroup strvector
+ * \function igraph_strvector_update
+ * \brief Updates a string vector from another one.
+ *
+ * After this operation the contents of \p to will be exactly the same
+ * as that of \p from. The vector \p to will be resized if it was originally
+ * shorter or longer than \p from.
+ *
+ * \param to The string vector to update.
+ * \param from The string vector to update from.
+ * \return Error code.
+ */
+igraph_error_t igraph_strvector_update(
+    igraph_strvector_t *to, const igraph_strvector_t *from
+) {
+    igraph_strvector_clear(to);
+    IGRAPH_CHECK(igraph_strvector_append(to, from));
+    return IGRAPH_SUCCESS;
+}
+
+/**
+ * \ingroup strvector
+ * \function igraph_strvector_swap
+ * \brief Swaps all elements of two string vectors.
+ *
+ * \param v1 The first string vector.
+ * \param v2 The second string vector.
+ *
+ * Time complexity: O(1).
+ */
+void igraph_strvector_swap(igraph_strvector_t *v1, igraph_strvector_t *v2) {
+    igraph_strvector_t tmp;
+
+    tmp = *v1;
+    *v1 = *v2;
+    *v2 = tmp;
+}
+
+/**
+ * \function igraph_strvector_swap_elements
+ * \brief Swap two elements in a string vector.
+ *
+ * Note that currently no range checking is performed.
+ *
+ * \param sv The string vector.
+ * \param i Index of the first element.
+ * \param j Index of the second element (may be the same as the first one).
+ *
+ * Time complexity: O(1).
+ */
+void igraph_strvector_swap_elements(igraph_strvector_t *sv, igraph_int_t i, igraph_int_t j) {
+    const char *tmp = sv->stor_begin[i];
+    sv->stor_begin[i] = sv->stor_begin[j];
+    sv->stor_begin[j] = tmp;
+}
