@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 #include "igraph_games.h"
 
 #include "igraph_adjlist.h"
+#include "igraph_bitset_list.h"
 #include "igraph_constructors.h"
 #include "igraph_conversion.h"
 #include "igraph_graphicality.h"
@@ -33,16 +34,17 @@
 #include "games/degree_sequence_vl/degree_sequence_vl.h"
 #include "math/safe_intop.h"
 
-static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *graph,
-                                       const igraph_vector_int_t *out_seq,
-                                       const igraph_vector_int_t *in_seq) {
+static igraph_error_t configuration(
+        igraph_t *graph,
+        const igraph_vector_int_t *out_seq,
+        const igraph_vector_int_t *in_seq) {
 
-    const igraph_bool_t directed = (in_seq != NULL && igraph_vector_int_size(in_seq) != 0);
-    igraph_integer_t outsum = 0, insum = 0;
+    const igraph_bool_t directed = (in_seq != NULL);
+    igraph_int_t outsum = 0, insum = 0;
     igraph_bool_t graphical;
-    igraph_integer_t no_of_nodes, no_of_edges;
-    igraph_integer_t *bag1, *bag2;
-    igraph_integer_t bagp1 = 0, bagp2 = 0;
+    igraph_int_t no_of_nodes, no_of_edges;
+    igraph_int_t *bag1, *bag2;
+    igraph_int_t bagp1 = 0, bagp2 = 0;
     igraph_vector_int_t edges;
 
     IGRAPH_CHECK(igraph_is_graphical(out_seq, in_seq, IGRAPH_LOOPS_SW | IGRAPH_MULTI_SW, &graphical));
@@ -59,21 +61,21 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *grap
     no_of_nodes = igraph_vector_int_size(out_seq);
     no_of_edges = directed ? outsum : outsum / 2;
 
-    bag1 = IGRAPH_CALLOC(outsum, igraph_integer_t);
+    bag1 = IGRAPH_CALLOC(outsum, igraph_int_t);
     IGRAPH_CHECK_OOM(bag1, "Insufficient memory for sampling from configuration model.");
     IGRAPH_FINALLY(igraph_free, bag1);
 
-    for (igraph_integer_t i = 0; i < no_of_nodes; i++) {
-        for (igraph_integer_t j = 0; j < VECTOR(*out_seq)[i]; j++) {
+    for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+        for (igraph_int_t j = 0; j < VECTOR(*out_seq)[i]; j++) {
             bag1[bagp1++] = i;
         }
     }
     if (directed) {
-        bag2 = IGRAPH_CALLOC(insum, igraph_integer_t);
+        bag2 = IGRAPH_CALLOC(insum, igraph_int_t);
         IGRAPH_CHECK_OOM(bag2, "Insufficient memory for sampling from configuration model.");
         IGRAPH_FINALLY(igraph_free, bag2);
-        for (igraph_integer_t i = 0; i < no_of_nodes; i++) {
-            for (igraph_integer_t j = 0; j < VECTOR(*in_seq)[i]; j++) {
+        for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+            for (igraph_int_t j = 0; j < VECTOR(*in_seq)[i]; j++) {
                 bag2[bagp2++] = i;
             }
         }
@@ -82,12 +84,10 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *grap
     IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 0);
     IGRAPH_CHECK(igraph_vector_int_reserve(&edges, no_of_edges * 2));
 
-    RNG_BEGIN();
-
     if (directed) {
-        for (igraph_integer_t i = 0; i < no_of_edges; i++) {
-            igraph_integer_t from = RNG_INTEGER(0, bagp1 - 1);
-            igraph_integer_t to = RNG_INTEGER(0, bagp2 - 1);
+        for (igraph_int_t i = 0; i < no_of_edges; i++) {
+            igraph_int_t from = RNG_INTEGER(0, bagp1 - 1);
+            igraph_int_t to = RNG_INTEGER(0, bagp2 - 1);
             igraph_vector_int_push_back(&edges, bag1[from]); /* safe, already reserved */
             igraph_vector_int_push_back(&edges, bag2[to]);   /* ditto */
             bag1[from] = bag1[bagp1 - 1];
@@ -95,9 +95,9 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *grap
             bagp1--; bagp2--;
         }
     } else {
-        for (igraph_integer_t i = 0; i < no_of_edges; i++) {
-            igraph_integer_t from = RNG_INTEGER(0, bagp1 - 1);
-            igraph_integer_t to;
+        for (igraph_int_t i = 0; i < no_of_edges; i++) {
+            igraph_int_t from = RNG_INTEGER(0, bagp1 - 1);
+            igraph_int_t to;
             igraph_vector_int_push_back(&edges, bag1[from]); /* safe, already reserved */
             bag1[from] = bag1[bagp1 - 1];
             bagp1--;
@@ -108,8 +108,6 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *grap
         }
     }
 
-    RNG_END();
-
     IGRAPH_FREE(bag1);
     IGRAPH_FINALLY_CLEAN(1);
     if (directed) {
@@ -124,8 +122,9 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration(igraph_t *grap
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_i_degree_sequence_game_fast_heur_undirected(
-        igraph_t *graph, const igraph_vector_int_t *seq) {
+static igraph_error_t fast_heur_undirected(
+        igraph_t *graph,
+        const igraph_vector_int_t *seq) {
 
     igraph_vector_int_t stubs;
     igraph_vector_int_t *neis;
@@ -133,9 +132,9 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_undirected(
     igraph_set_t incomplete_vertices;
     igraph_adjlist_t al;
     igraph_bool_t finished, failed;
-    igraph_integer_t from, to, dummy;
-    igraph_integer_t i, j, k;
-    igraph_integer_t no_of_nodes, outsum = 0;
+    igraph_int_t from, to, dummy;
+    igraph_int_t i, j, k;
+    igraph_int_t no_of_nodes, outsum = 0;
     igraph_bool_t graphical;
     int iter = 0;
 
@@ -157,9 +156,6 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_undirected(
     IGRAPH_CHECK(igraph_set_init(&incomplete_vertices, 0));
     IGRAPH_FINALLY(igraph_set_destroy, &incomplete_vertices);
 
-    /* Start the RNG */
-    RNG_BEGIN();
-
     /* Outer loop; this will try to construct a graph several times from scratch
      * until it finally succeeds. */
     finished = false;
@@ -245,9 +241,6 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_undirected(
         }
     }
 
-    /* Finish the RNG */
-    RNG_END();
-
     /* Clean up */
     igraph_set_destroy(&incomplete_vertices);
     igraph_vector_int_destroy(&residual_degrees);
@@ -267,8 +260,10 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_undirected(
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_i_degree_sequence_game_fast_heur_directed(igraph_t *graph,
-        const igraph_vector_int_t *out_seq, const igraph_vector_int_t *in_seq) {
+static igraph_error_t fast_heur_directed(
+        igraph_t *graph,
+        const igraph_vector_int_t *out_seq,
+        const igraph_vector_int_t *in_seq) {
 
     igraph_adjlist_t al;
     igraph_bool_t deg_seq_ok, failed, finished;
@@ -277,9 +272,9 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_directed(igraph_t
     igraph_vector_int_t *neis;
     igraph_vector_int_t residual_in_degrees, residual_out_degrees;
     igraph_set_t incomplete_in_vertices, incomplete_out_vertices;
-    igraph_integer_t from, to;
-    igraph_integer_t i, j, k;
-    igraph_integer_t no_of_nodes, outsum;
+    igraph_int_t from, to;
+    igraph_int_t i, j, k;
+    igraph_int_t no_of_nodes, outsum;
     int iter = 0;
 
     IGRAPH_CHECK(igraph_is_graphical(out_seq, in_seq, IGRAPH_SIMPLE_SW, &deg_seq_ok));
@@ -305,9 +300,6 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_directed(igraph_t
     IGRAPH_CHECK(igraph_set_init(&incomplete_in_vertices, 0));
     IGRAPH_FINALLY(igraph_set_destroy, &incomplete_in_vertices);
 
-    /* Start the RNG */
-    RNG_BEGIN();
-
     /* Outer loop; this will try to construct a graph several times from scratch
      * until it finally succeeds. */
     finished = false;
@@ -390,9 +382,6 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_directed(igraph_t
         }
     }
 
-    /* Finish the RNG */
-    RNG_END();
-
     /* Clean up */
     igraph_set_destroy(&incomplete_in_vertices);
     igraph_set_destroy(&incomplete_out_vertices);
@@ -415,73 +404,49 @@ static igraph_error_t igraph_i_degree_sequence_game_fast_heur_directed(igraph_t
 /* swap two elements of a vector_int */
 #define SWAP_INT_ELEM(vec, i, j) \
     { \
-        igraph_integer_t temp; \
+        igraph_int_t temp; \
         temp = VECTOR(vec)[i]; \
         VECTOR(vec)[i] = VECTOR(vec)[j]; \
         VECTOR(vec)[j] = temp; \
     }
 
-static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_undirected(igraph_t *graph, const igraph_vector_int_t *degseq) {
-    igraph_vector_int_t stubs;
-    igraph_vector_int_t edges;
-    igraph_bool_t graphical;
+/* Uses a set to check for multi-edges. Efficient for larger graphs, frugal with memory. */
+static igraph_error_t configuration_simple_undirected_set(
+        const igraph_vector_int_t *degseq,
+        igraph_vector_int_t *stubs,
+        igraph_int_t vcount, igraph_int_t stub_count) {
+
+    const igraph_int_t ecount = stub_count / 2;
     igraph_vector_ptr_t adjlist;
-    igraph_integer_t i, j;
-    igraph_integer_t vcount, ecount, stub_count;
     int iter = 0;
 
-    IGRAPH_CHECK(igraph_is_graphical(degseq, NULL, IGRAPH_SIMPLE_SW, &graphical));
-    if (!graphical) {
-        IGRAPH_ERROR("No simple undirected graph can realize the given degree sequence.", IGRAPH_EINVAL);
-    }
-
-    stub_count = igraph_vector_int_sum(degseq);
-    ecount = stub_count / 2;
-    vcount = igraph_vector_int_size(degseq);
-
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&stubs, stub_count);
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, stub_count);
-
-    /* Fill stubs vector. */
-    {
-        igraph_integer_t k = 0;
-        for (i = 0; i < vcount; ++i) {
-            igraph_integer_t deg = VECTOR(*degseq)[i];
-            for (j = 0; j < deg; ++j) {
-                VECTOR(stubs)[k++] = i;
-            }
-        }
-    }
-
     /* Build an adjacency list in terms of sets; used to check for multi-edges. */
     IGRAPH_CHECK(igraph_vector_ptr_init(&adjlist, vcount));
     IGRAPH_VECTOR_PTR_SET_ITEM_DESTRUCTOR(&adjlist, igraph_set_destroy);
     IGRAPH_FINALLY(igraph_vector_ptr_destroy_all, &adjlist);
-    for (i = 0; i < vcount; ++i) {
+    for (igraph_int_t i = 0; i < vcount; ++i) {
         igraph_set_t *set = IGRAPH_CALLOC(1, igraph_set_t);
-        IGRAPH_CHECK_OOM(set, "Insufficient memory for configuation model (simple graphs).");
+        IGRAPH_CHECK_OOM(set, "Insufficient memory for configuration model (simple graphs).");
         IGRAPH_CHECK(igraph_set_init(set, 0));
         VECTOR(adjlist)[i] = set;
         IGRAPH_CHECK(igraph_set_reserve(set, VECTOR(*degseq)[i]));
     }
 
-    RNG_BEGIN();
-
     for (;;) {
         igraph_bool_t success = true;
 
         /* Shuffle stubs vector with Fisher-Yates and check for self-loops and multi-edges as we go. */
-        for (i = 0; i < ecount; ++i) {
-            igraph_integer_t k, from, to;
+        for (igraph_int_t i = 0; i < ecount; ++i) {
+            igraph_int_t k, from, to;
 
             k = RNG_INTEGER(2*i, stub_count-1);
-            SWAP_INT_ELEM(stubs, 2*i, k);
+            SWAP_INT_ELEM(*stubs, 2*i, k);
 
             k = RNG_INTEGER(2*i+1, stub_count-1);
-            SWAP_INT_ELEM(stubs, 2*i+1, k);
+            SWAP_INT_ELEM(*stubs, 2*i+1, k);
 
-            from = VECTOR(stubs)[2*i];
-            to   = VECTOR(stubs)[2*i+1];
+            from = VECTOR(*stubs)[2*i];
+            to   = VECTOR(*stubs)[2*i+1];
 
             /* self-loop, fail */
             if (from == to) {
@@ -498,10 +463,6 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_undirec
             /* sets are already reserved */
             igraph_set_add((igraph_set_t *) VECTOR(adjlist)[to], from);
             igraph_set_add((igraph_set_t *) VECTOR(adjlist)[from], to);
-
-            /* register edge */
-            VECTOR(edges)[2 * i]   = from;
-            VECTOR(edges)[2 * i + 1] = to;
         }
 
         if (success) {
@@ -509,37 +470,140 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_undirec
         }
 
         /* Clear adjacency list. */
-        for (j = 0; j < vcount; ++j) {
+        for (igraph_int_t j = 0; j < vcount; ++j) {
             igraph_set_clear((igraph_set_t *) VECTOR(adjlist)[j]);
         }
 
         IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 8);
     }
 
-    RNG_END();
-
     igraph_vector_ptr_destroy_all(&adjlist);
+    IGRAPH_FINALLY_CLEAN(1);
+
+    return IGRAPH_SUCCESS;
+}
+
+/* Uses a bitset to check for multi-edges. Efficient for smaller graphs. */
+static igraph_error_t configuration_simple_undirected_bitset(
+        igraph_vector_int_t *stubs,
+        igraph_int_t vcount, igraph_int_t stub_count) {
+
+    const igraph_int_t ecount = stub_count / 2;
+    igraph_bitset_list_t adjlist;
+    int iter = 0;
+
+    /* Build an adjacency list in terms of bitsets; used to check for multi-edges. */
+    IGRAPH_BITSET_LIST_INIT_FINALLY(&adjlist, vcount);
+    for (igraph_int_t i = 0; i < vcount; ++i) {
+        IGRAPH_CHECK(igraph_bitset_resize(igraph_bitset_list_get_ptr(&adjlist, i), vcount));
+    }
+
+    for (;;) {
+        igraph_bool_t success = true;
+
+        /* Shuffle stubs vector with Fisher-Yates and check for self-loops and multi-edges as we go. */
+        for (igraph_int_t i = 0; i < ecount; ++i) {
+            igraph_int_t k, from, to;
+
+            k = RNG_INTEGER(2*i, stub_count-1);
+            SWAP_INT_ELEM(*stubs, 2*i, k);
+
+            k = RNG_INTEGER(2*i+1, stub_count-1);
+            SWAP_INT_ELEM(*stubs, 2*i+1, k);
+
+            from = VECTOR(*stubs)[2*i];
+            to   = VECTOR(*stubs)[2*i+1];
+
+            /* self-loop, fail */
+            if (from == to) {
+                success = false;
+                break;
+            }
+
+            /* multi-edge, fail */
+            if (IGRAPH_BIT_TEST(*igraph_bitset_list_get_ptr(&adjlist, to), from)) {
+                success = false;
+                break;
+            }
+
+            /* sets are already reserved */
+            IGRAPH_BIT_SET(*igraph_bitset_list_get_ptr(&adjlist, to), from);
+            IGRAPH_BIT_SET(*igraph_bitset_list_get_ptr(&adjlist, from), to);
+        }
+
+        if (success) {
+            break;
+        }
+
+        /* Clear adjacency list. */
+        for (igraph_int_t j = 0; j < vcount; ++j) {
+            igraph_bitset_null(igraph_bitset_list_get_ptr(&adjlist, j));
+        }
+
+        IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 8);
+    }
+
+    igraph_bitset_list_destroy(&adjlist);
+    IGRAPH_FINALLY_CLEAN(1);
+
+    return IGRAPH_SUCCESS;
+}
+
+static igraph_error_t configuration_simple_undirected(
+        igraph_t *graph,
+        const igraph_vector_int_t *degseq) {
+
+    igraph_vector_int_t stubs;
+    igraph_bool_t graphical;
+    igraph_int_t vcount, stub_count;
+
+    IGRAPH_CHECK(igraph_is_graphical(degseq, NULL, IGRAPH_SIMPLE_SW, &graphical));
+    if (!graphical) {
+        IGRAPH_ERROR("No simple undirected graph can realize the given degree sequence.", IGRAPH_EINVAL);
+    }
+
+    stub_count = igraph_vector_int_sum(degseq);
+    vcount = igraph_vector_int_size(degseq);
+
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&stubs, stub_count);
+
+    /* Fill stubs vector. */
+    {
+        igraph_int_t k = 0;
+        for (igraph_int_t i = 0; i < vcount; ++i) {
+            igraph_int_t deg = VECTOR(*degseq)[i];
+            for (igraph_int_t j = 0; j < deg; ++j) {
+                VECTOR(stubs)[k++] = i;
+            }
+        }
+    }
+
+    /* Tradeoff between speed and memory: Choose set vs bitset implementation. */
+    if (vcount > 1024) {
+        IGRAPH_CHECK(configuration_simple_undirected_set(degseq, &stubs, vcount, stub_count));
+    } else {
+        IGRAPH_CHECK(configuration_simple_undirected_bitset(&stubs, vcount, stub_count));
+    }
+
+    IGRAPH_CHECK(igraph_create(graph, &stubs, vcount, IGRAPH_UNDIRECTED));
+
     igraph_vector_int_destroy(&stubs);
-    IGRAPH_FINALLY_CLEAN(2);
-
-    IGRAPH_CHECK(igraph_create(graph, &edges, vcount, IGRAPH_UNDIRECTED));
-
-    igraph_vector_int_destroy(&edges);
     IGRAPH_FINALLY_CLEAN(1);
 
     return IGRAPH_SUCCESS;
 }
 
 
-static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_directed(
-    igraph_t *graph, const igraph_vector_int_t *out_deg, const igraph_vector_int_t *in_deg) {
+static igraph_error_t configuration_simple_directed(
+        igraph_t *graph,
+        const igraph_vector_int_t *out_deg,
+        const igraph_vector_int_t *in_deg) {
 
     igraph_vector_int_t out_stubs, in_stubs;
     igraph_vector_int_t edges;
+    igraph_vector_int_t vertex_done;
     igraph_bool_t graphical;
-    igraph_vector_ptr_t adjlist;
-    igraph_integer_t i, j;
-    igraph_integer_t vcount, ecount;
+    igraph_int_t vcount, ecount;
     int iter = 0;
 
     IGRAPH_CHECK(igraph_is_graphical(out_deg, in_deg, IGRAPH_SIMPLE_SW, &graphical));
@@ -550,49 +614,46 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_directe
     ecount = igraph_vector_int_sum(out_deg);
     vcount = igraph_vector_int_size(out_deg);
 
+    /* In the directed case, checking for multi-edges can be done efficiently for as
+     * long as only one of the in-/out-stub vectors is shuffled. Here we shuffle
+     * the out-stub vector, keeping in the in-stubs in their original order, thus
+     * processing vertices to connect *to* in order. For each vertex v, we mark
+     * vertex_done[v] to indicate that it has already been connected *to* the
+     * current vertex. When moving on to the next vertex, instead of nulling the
+     * vertex_done vector, we simply change the mark we use. */
+
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 2 * ecount);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&out_stubs, ecount);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&in_stubs, ecount);
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 2 * ecount);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&vertex_done, vcount);
 
     /* Fill in- and out-stubs vectors. */
     {
-        igraph_integer_t k = 0, l = 0;
-        for (i = 0; i < vcount; ++i) {
-            igraph_integer_t dout, din;
+        igraph_int_t k = 0, l = 0;
+        for (igraph_int_t i = 0; i < vcount; ++i) {
+            igraph_int_t dout, din;
 
             dout = VECTOR(*out_deg)[i];
-            for (j = 0; j < dout; ++j) {
+            for (igraph_int_t j = 0; j < dout; ++j) {
                 VECTOR(out_stubs)[k++] = i;
             }
 
             din  = VECTOR(*in_deg)[i];
-            for (j = 0; j < din; ++j) {
+            for (igraph_int_t j = 0; j < din; ++j) {
                 VECTOR(in_stubs)[l++] = i;
             }
         }
     }
 
-    /* Build an adjacency list in terms of sets; used to check for multi-edges. */
-    IGRAPH_CHECK(igraph_vector_ptr_init(&adjlist, vcount));
-    IGRAPH_VECTOR_PTR_SET_ITEM_DESTRUCTOR(&adjlist, igraph_set_destroy);
-    IGRAPH_FINALLY(igraph_vector_ptr_destroy_all, &adjlist);
-    for (i = 0; i < vcount; ++i) {
-        igraph_set_t *set = IGRAPH_CALLOC(1, igraph_set_t);
-        IGRAPH_CHECK_OOM(set, "Insufficient memory for directed configuration model (simple graphs).");
-        IGRAPH_CHECK(igraph_set_init(set, 0));
-        VECTOR(adjlist)[i] = set;
-        IGRAPH_CHECK(igraph_set_reserve(set, VECTOR(*out_deg)[i]));
-    }
-
-    RNG_BEGIN();
+    igraph_int_t vertex_done_mark = 1;
 
     for (;;) {
         igraph_bool_t success = true;
+        igraph_int_t previous_to = -1;
 
         /* Shuffle out-stubs vector with Fisher-Yates and check for self-loops and multi-edges as we go. */
-        for (i = 0; i < ecount; ++i) {
-            igraph_integer_t k, from, to;
-            igraph_set_t *set;
+        for (igraph_int_t i = 0; i < ecount; ++i) {
+            igraph_int_t k, from, to;
 
             k = RNG_INTEGER(i, ecount-1);
             SWAP_INT_ELEM(out_stubs, i, k);
@@ -606,38 +667,36 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_directe
                 break;
             }
 
+            /* have we moved on to the next vertex? */
+            if (to != previous_to) {
+                vertex_done_mark++;
+                previous_to = to;
+            }
+
             /* multi-edge, fail */
-            set = (igraph_set_t *) VECTOR(adjlist)[from];
-            if (igraph_set_contains(set, to)) {
+            if (VECTOR(vertex_done)[from] == vertex_done_mark) {
                 success = false;
                 break;
             }
 
-            /* sets are already reserved */
-            igraph_set_add(set, to);
-
-            /* register edge */
-            VECTOR(edges)[2 * i]   = from;
-            VECTOR(edges)[2 * i + 1] = to;
+            VECTOR(vertex_done)[from] = vertex_done_mark;
         }
 
         if (success) {
             break;
         }
 
-        /* Clear adjacency list. */
-        for (j = 0; j < vcount; ++j) {
-            igraph_set_clear((igraph_set_t *) VECTOR(adjlist)[j]);
-        }
-
         IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 8);
     }
 
-    RNG_END();
+    for (igraph_int_t i=0; i < ecount; i++) {
+        VECTOR(edges)[2*i]   = VECTOR(out_stubs)[i];
+        VECTOR(edges)[2*i+1] = VECTOR(in_stubs)[i];
+    }
 
-    igraph_vector_ptr_destroy_all(&adjlist);
-    igraph_vector_int_destroy(&out_stubs);
+    igraph_vector_int_destroy(&vertex_done);
     igraph_vector_int_destroy(&in_stubs);
+    igraph_vector_int_destroy(&out_stubs);
     IGRAPH_FINALLY_CLEAN(3);
 
     IGRAPH_CHECK(igraph_create(graph, &edges, vcount, IGRAPH_DIRECTED));
@@ -650,13 +709,14 @@ static igraph_error_t igraph_i_degree_sequence_game_configuration_simple_directe
 
 #undef SWAP_INT_ELEM
 
-igraph_error_t igraph_i_degree_sequence_game_edge_switching(
+igraph_error_t edge_switching(
         igraph_t *graph,
-        const igraph_vector_int_t *out_seq, const igraph_vector_int_t *in_seq) {
+        const igraph_vector_int_t *out_seq,
+        const igraph_vector_int_t *in_seq) {
 
     IGRAPH_CHECK(igraph_realize_degree_sequence(graph, out_seq, in_seq, IGRAPH_SIMPLE_SW, IGRAPH_REALIZE_DEGSEQ_INDEX));
     IGRAPH_FINALLY(igraph_destroy, graph);
-    IGRAPH_CHECK(igraph_rewire(graph, 10 * igraph_ecount(graph), IGRAPH_REWIRING_SIMPLE));
+    IGRAPH_CHECK(igraph_rewire(graph, 10 * igraph_ecount(graph), IGRAPH_SIMPLE_SW, NULL));
     IGRAPH_FINALLY_CLEAN(1);
     return IGRAPH_SUCCESS;
 }
@@ -690,13 +750,10 @@ igraph_error_t igraph_i_degree_sequence_game_edge_switching(
  * https://doi.org/10.1088/2632-072x/abced5.
  *
  * \param graph Pointer to an uninitialized graph object.
- * \param out_deg The degree sequence for an undirected graph (if
- *        \p in_seq is \c NULL or of length zero), or the out-degree
- *        sequence of a directed graph (if \p in_deq is not
- *        of length zero).
- * \param in_deg It is either a zero-length vector or
- *        \c NULL (if an undirected
- *        graph is generated), or the in-degree sequence.
+ * \param out_degrees A vector of integers specifying the degree sequence for
+ *     undirected graphs or the out-degree sequence for directed graphs.
+ * \param in_degrees A vector of integers specifying the in-degree sequence for
+ *     directed graphs. For undirected graphs, it must be \c NULL.
  * \param method The method to generate the graph. Possible values:
  *        \clist
  *          \cli IGRAPH_DEGSEQ_CONFIGURATION
@@ -771,36 +828,35 @@ igraph_error_t igraph_i_degree_sequence_game_edge_switching(
  * \example examples/simple/igraph_degree_sequence_game.c
  */
 
-igraph_error_t igraph_degree_sequence_game(igraph_t *graph, const igraph_vector_int_t *out_deg,
-                                const igraph_vector_int_t *in_deg,
-                                igraph_degseq_t method) {
-    if (in_deg && igraph_vector_int_empty(in_deg) && !igraph_vector_int_empty(out_deg)) {
-        in_deg = NULL;
-    }
+igraph_error_t igraph_degree_sequence_game(
+        igraph_t *graph,
+        const igraph_vector_int_t *out_degrees,
+        const igraph_vector_int_t *in_degrees,
+        igraph_degseq_t method) {
 
     switch (method) {
     case IGRAPH_DEGSEQ_CONFIGURATION:
-        return igraph_i_degree_sequence_game_configuration(graph, out_deg, in_deg);
+        return configuration(graph, out_degrees, in_degrees);
 
     case IGRAPH_DEGSEQ_VL:
-        return igraph_degree_sequence_game_vl(graph, out_deg, in_deg);
+        return igraph_i_degree_sequence_game_vl(graph, out_degrees, in_degrees);
 
     case IGRAPH_DEGSEQ_FAST_HEUR_SIMPLE:
-        if (! in_deg) {
-            return igraph_i_degree_sequence_game_fast_heur_undirected(graph, out_deg);
+        if (! in_degrees) {
+            return fast_heur_undirected(graph, out_degrees);
         } else {
-            return igraph_i_degree_sequence_game_fast_heur_directed(graph, out_deg, in_deg);
+            return fast_heur_directed(graph, out_degrees, in_degrees);
         }
 
     case IGRAPH_DEGSEQ_CONFIGURATION_SIMPLE:
-        if (! in_deg) {
-            return igraph_i_degree_sequence_game_configuration_simple_undirected(graph, out_deg);
+        if (! in_degrees) {
+            return configuration_simple_undirected(graph, out_degrees);
         } else {
-            return igraph_i_degree_sequence_game_configuration_simple_directed(graph, out_deg, in_deg);
+            return configuration_simple_directed(graph, out_degrees, in_degrees);
         }
 
     case IGRAPH_DEGSEQ_EDGE_SWITCHING_SIMPLE:
-        return igraph_i_degree_sequence_game_edge_switching(graph, out_deg, in_deg);
+        return edge_switching(graph, out_degrees, in_degrees);
 
     default:
         IGRAPH_ERROR("Invalid degree sequence game method.", IGRAPH_EINVAL);
