@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2021 The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -42,7 +40,7 @@
  * \param graph Pointer to an uninitialized graph object.
  * \param nodes The number of vertices in the graph.
  * \param radius The radius within which the vertices will be connected.
- * \param torus Logical constant. If true, periodic boundary conditions
+ * \param torus Boolean constant. If true, periodic boundary conditions
  *        will be used, i.e. the vertices are assumed to be on a torus
  *        instead of a square.
  * \param x An initialized vector or \c NULL. If not \c NULL, the points'
@@ -55,11 +53,11 @@
  *
  * \example examples/simple/igraph_grg_game.c
  */
-igraph_error_t igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
+igraph_error_t igraph_grg_game(igraph_t *graph, igraph_int_t nodes,
                     igraph_real_t radius, igraph_bool_t torus,
                     igraph_vector_t *x, igraph_vector_t *y) {
 
-    igraph_integer_t i;
+    igraph_int_t i;
     igraph_vector_t myx, myy, *xx = &myx, *yy = &myy;
     igraph_vector_int_t edges;
     igraph_real_t r2;
@@ -91,22 +89,18 @@ igraph_error_t igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
         IGRAPH_VECTOR_INIT_FINALLY(yy, nodes);
     }
 
-    RNG_BEGIN();
-
     for (i = 0; i < nodes; i++) {
         VECTOR(*xx)[i] = RNG_UNIF01();
         VECTOR(*yy)[i] = RNG_UNIF01();
     }
 
-    RNG_END();
-
     igraph_vector_sort(xx);
 
     if (!torus) {
         for (i = 0; i < nodes; i++) {
             igraph_real_t xx1 = VECTOR(*xx)[i];
             igraph_real_t yy1 = VECTOR(*yy)[i];
-            igraph_integer_t j = i + 1;
+            igraph_int_t j = i + 1;
             igraph_real_t dx, dy;
 
             IGRAPH_ALLOW_INTERRUPTION();
@@ -125,7 +119,7 @@ igraph_error_t igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
         for (i = 0; i < nodes; i++) {
             igraph_real_t xx1 = VECTOR(*xx)[i];
             igraph_real_t yy1 = VECTOR(*yy)[i];
-            igraph_integer_t j = i + 1;
+            igraph_int_t j = i + 1;
             igraph_real_t dx, dy;
 
             IGRAPH_ALLOW_INTERRUPTION();
