@@ -45,8 +45,7 @@
      KEYWORD = 260,
      LISTOPEN = 261,
      LISTCLOSE = 262,
-     EOFF = 263,
-     ERROR = 264
+     ERROR = 263
    };
 #endif
 /* Tokens.  */
@@ -56,8 +55,7 @@
 #define KEYWORD 260
 #define LISTOPEN 261
 #define LISTCLOSE 262
-#define EOFF 263
-#define ERROR 264
+#define ERROR 263
 
 
 
@@ -66,12 +64,9 @@
 typedef union YYSTYPE
 
 {
-   struct {
-      char *s;
-      int len;
-   } str;
-   void *tree;
-   double real;
+   char *str;
+   igraph_gml_tree_t *tree;
+   igraph_real_t real;
 }
 /* Line 1529 of yacc.c.  */
 
