@@ -58,7 +58,7 @@
 typedef union YYSTYPE
 
 {
-  igraph_integer_t edgenum;
+  igraph_int_t edgenum;
   igraph_real_t weightnum;
 }
 /* Line 1529 of yacc.c.  */
