@@ -146,7 +146,7 @@
 typedef union YYSTYPE
 
 {
-  igraph_integer_t intnum;
+  igraph_int_t intnum;
   igraph_real_t    realnum;
   struct {
     char *str;
