@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -21,7 +20,7 @@
 
 */
 
-#include "igraph_topology.h"
+#include "igraph_isomorphism.h"
 
 #include "igraph_adjlist.h"
 #include "igraph_interface.h"
@@ -144,20 +143,20 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
     igraph_isocompat_t *edge_compat_fn, void *arg
 ) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph1);
-    igraph_integer_t no_of_edges = igraph_ecount(graph1);
+    igraph_int_t no_of_nodes = igraph_vcount(graph1);
+    igraph_int_t no_of_edges = igraph_ecount(graph1);
     igraph_vector_int_t mycore_1, mycore_2, *core_1 = &mycore_1, *core_2 = &mycore_2;
     igraph_vector_int_t in_1, in_2, out_1, out_2;
-    igraph_integer_t in_1_size = 0, in_2_size = 0, out_1_size = 0, out_2_size = 0;
+    igraph_int_t in_1_size = 0, in_2_size = 0, out_1_size = 0, out_2_size = 0;
     igraph_vector_int_t *inneis_1, *inneis_2, *outneis_1, *outneis_2;
-    igraph_integer_t matched_nodes = 0;
-    igraph_integer_t depth;
-    igraph_integer_t cand1, cand2;
-    igraph_integer_t last1, last2;
+    igraph_int_t matched_nodes = 0;
+    igraph_int_t depth;
+    igraph_int_t cand1, cand2;
+    igraph_int_t last1, last2;
     igraph_stack_int_t path;
     igraph_lazy_adjlist_t inadj1, inadj2, outadj1, outadj2;
     igraph_vector_int_t indeg1, indeg2, outdeg1, outdeg2;
-    igraph_integer_t vsize;
+    igraph_int_t vsize;
 
     IGRAPH_CHECK(igraph_i_perform_vf2_pre_checks(graph1, graph2));
 
@@ -275,7 +274,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
     depth = 0; last1 = -1; last2 = -1;
     while (depth >= 0) {
-        igraph_integer_t i;
+        igraph_int_t i;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -381,7 +380,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_1)[i];
+                    igraph_int_t node = VECTOR(*inneis_1)[i];
                     if (VECTOR(in_1)[node] == depth) {
                         VECTOR(in_1)[node] = 0;
                         in_1_size -= 1;
@@ -393,7 +392,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_1)[i];
+                    igraph_int_t node = VECTOR(*outneis_1)[i];
                     if (VECTOR(out_1)[node] == depth) {
                         VECTOR(out_1)[node] = 0;
                         out_1_size -= 1;
@@ -405,7 +404,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_2)[i];
+                    igraph_int_t node = VECTOR(*inneis_2)[i];
                     if (VECTOR(in_2)[node] == depth) {
                         VECTOR(in_2)[node] = 0;
                         in_2_size -= 1;
@@ -417,7 +416,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_2)[i];
+                    igraph_int_t node = VECTOR(*outneis_2)[i];
                     if (VECTOR(out_2)[node] == depth) {
                         VECTOR(out_2)[node] = 0;
                         out_2_size -= 1;
@@ -431,7 +430,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
         } else {
             /**************************************************************/
             /* step forward if worth, check if worth first */
-            igraph_integer_t xin1 = 0, xin2 = 0, xout1 = 0, xout2 = 0;
+            igraph_int_t xin1 = 0, xin2 = 0, xout1 = 0, xout2 = 0;
             igraph_bool_t end = false;
 
             inneis_1 = igraph_lazy_adjlist_get(&inadj1, cand1);
@@ -456,14 +455,14 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
             vsize = igraph_vector_int_size(inneis_1);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*inneis_1)[i];
+                igraph_int_t node = VECTOR(*inneis_1)[i];
                 if (VECTOR(*core_1)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_1)[node];
+                    igraph_int_t node2 = VECTOR(*core_1)[node];
                     /* check if there is a node2->cand2 edge */
                     if (!igraph_vector_int_contains_sorted(inneis_2, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, node, cand1, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, node2, cand2, IGRAPH_DIRECTED,
@@ -488,14 +487,14 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(outneis_1);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*outneis_1)[i];
+                igraph_int_t node = VECTOR(*outneis_1)[i];
                 if (VECTOR(*core_1)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_1)[node];
+                    igraph_int_t node2 = VECTOR(*core_1)[node];
                     /* check if there is a cand2->node2 edge */
                     if (!igraph_vector_int_contains_sorted(outneis_2, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, cand1, node, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, cand2, node2, IGRAPH_DIRECTED,
@@ -520,14 +519,14 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(inneis_2);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*inneis_2)[i];
+                igraph_int_t node = VECTOR(*inneis_2)[i];
                 if (VECTOR(*core_2)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_2)[node];
+                    igraph_int_t node2 = VECTOR(*core_2)[node];
                     /* check if there is a node2->cand1 edge */
                     if (!igraph_vector_int_contains_sorted(inneis_1, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, node2, cand1, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, node, cand2, IGRAPH_DIRECTED,
@@ -552,14 +551,14 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(outneis_2);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*outneis_2)[i];
+                igraph_int_t node = VECTOR(*outneis_2)[i];
                 if (VECTOR(*core_2)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_2)[node];
+                    igraph_int_t node2 = VECTOR(*core_2)[node];
                     /* check if there is a cand1->node2 edge */
                     if (!igraph_vector_int_contains_sorted(outneis_1, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, cand1, node2, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, cand2, node, IGRAPH_DIRECTED,
@@ -611,7 +610,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_1)[i];
+                    igraph_int_t node = VECTOR(*inneis_1)[i];
                     if (VECTOR(in_1)[node] == 0 && VECTOR(*core_1)[node] < 0) {
                         VECTOR(in_1)[node] = depth;
                         in_1_size += 1;
@@ -623,7 +622,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_1)[i];
+                    igraph_int_t node = VECTOR(*outneis_1)[i];
                     if (VECTOR(out_1)[node] == 0 && VECTOR(*core_1)[node] < 0) {
                         VECTOR(out_1)[node] = depth;
                         out_1_size += 1;
@@ -635,7 +634,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_2)[i];
+                    igraph_int_t node = VECTOR(*inneis_2)[i];
                     if (VECTOR(in_2)[node] == 0 && VECTOR(*core_2)[node] < 0) {
                         VECTOR(in_2)[node] = depth;
                         in_2_size += 1;
@@ -647,7 +646,7 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_2)[i];
+                    igraph_int_t node = VECTOR(*outneis_2)[i];
                     if (VECTOR(out_2)[node] == 0 && VECTOR(*core_2)[node] < 0) {
                         VECTOR(out_2)[node] = depth;
                         out_2_size += 1;
@@ -697,26 +696,6 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
     return IGRAPH_SUCCESS;
 }
 
-/**
- * \function igraph_isomorphic_function_vf2
- * \brief The generic VF2 interface (deprecated alias).
- *
- * \deprecated-by igraph_get_isomorphisms_vf2_callback 0.10.0
- */
-igraph_error_t igraph_isomorphic_function_vf2(
-    const igraph_t *graph1, const igraph_t *graph2,
-    const igraph_vector_int_t *vertex_color1, const igraph_vector_int_t *vertex_color2,
-    const igraph_vector_int_t *edge_color1, const igraph_vector_int_t *edge_color2,
-    igraph_vector_int_t *map12, igraph_vector_int_t *map21,
-    igraph_isohandler_t *isohandler_fn, igraph_isocompat_t *node_compat_fn,
-    igraph_isocompat_t *edge_compat_fn, void *arg
-) {
-    return igraph_get_isomorphisms_vf2_callback(
-        graph1, graph2, vertex_color1, vertex_color2, edge_color1, edge_color2,
-        map12, map21, isohandler_fn, node_compat_fn, edge_compat_fn, arg
-    );
-}
-
 typedef struct {
     igraph_isocompat_t *node_compat_fn, *edge_compat_fn;
     void *arg, *carg;
@@ -725,8 +704,8 @@ typedef struct {
 static igraph_bool_t igraph_i_isocompat_node_cb(
         const igraph_t *graph1,
         const igraph_t *graph2,
-        const igraph_integer_t g1_num,
-        const igraph_integer_t g2_num,
+        const igraph_int_t g1_num,
+        const igraph_int_t g2_num,
         void *arg) {
     igraph_i_iso_cb_data_t *data = arg;
     return data->node_compat_fn(graph1, graph2, g1_num, g2_num, data->carg);
@@ -735,8 +714,8 @@ static igraph_bool_t igraph_i_isocompat_node_cb(
 static igraph_bool_t igraph_i_isocompat_edge_cb(
         const igraph_t *graph1,
         const igraph_t *graph2,
-        const igraph_integer_t g1_num,
-        const igraph_integer_t g2_num,
+        const igraph_int_t g1_num,
+        const igraph_int_t g2_num,
         void *arg) {
     igraph_i_iso_cb_data_t *data = arg;
     return data->edge_compat_fn(graph1, graph2, g1_num, g2_num, data->carg);
@@ -779,7 +758,7 @@ static igraph_error_t igraph_i_isomorphic_vf2_cb(
  *   colors as well. Supply a null pointer here if your graphs are not
  *   edge-colored.
  * \param edge_color2 The edge color vector for the second graph.
- * \param iso Pointer to a logical constant, the result of the
+ * \param iso Pointer to a Boolean constant, the result of the
  *    algorithm will be placed here.
  * \param map12 Pointer to an initialized vector or a NULL pointer. If not
  *    a NULL pointer then the mapping from \p graph1 to \p graph2 is
@@ -845,7 +824,7 @@ static igraph_error_t igraph_i_count_isomorphisms_vf2_cb(
     void *arg
 ) {
     igraph_i_iso_cb_data_t *data = arg;
-    igraph_integer_t *count = data->arg;
+    igraph_int_t *count = data->arg;
     IGRAPH_UNUSED(map12); IGRAPH_UNUSED(map21);
     *count += 1;
     return IGRAPH_SUCCESS;
@@ -885,7 +864,7 @@ static igraph_error_t igraph_i_count_isomorphisms_vf2_cb(
  *   \p edge_compat_fn.
  * \return Error code.
  *
- * \sa igraph_count_automorphisms()
+ * \sa igraph_count_automorphisms_bliss()
  *
  * Time complexity: exponential.
  */
@@ -895,7 +874,7 @@ igraph_error_t igraph_count_isomorphisms_vf2(const igraph_t *graph1, const igrap
                                   const igraph_vector_int_t *vertex_color2,
                                   const igraph_vector_int_t *edge_color1,
                                   const igraph_vector_int_t *edge_color2,
-                                  igraph_integer_t *count,
+                                  igraph_int_t *count,
                                   igraph_isocompat_t *node_compat_fn,
                                   igraph_isocompat_t *edge_compat_fn,
                                   void *arg) {
@@ -1050,22 +1029,22 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
     igraph_isocompat_t *edge_compat_fn, void *arg
 ) {
 
-    igraph_integer_t no_of_nodes1 = igraph_vcount(graph1),
+    igraph_int_t no_of_nodes1 = igraph_vcount(graph1),
              no_of_nodes2 = igraph_vcount(graph2);
-    igraph_integer_t no_of_edges1 = igraph_ecount(graph1),
+    igraph_int_t no_of_edges1 = igraph_ecount(graph1),
              no_of_edges2 = igraph_ecount(graph2);
     igraph_vector_int_t mycore_1, mycore_2, *core_1 = &mycore_1, *core_2 = &mycore_2;
     igraph_vector_int_t in_1, in_2, out_1, out_2;
-    igraph_integer_t in_1_size = 0, in_2_size = 0, out_1_size = 0, out_2_size = 0;
+    igraph_int_t in_1_size = 0, in_2_size = 0, out_1_size = 0, out_2_size = 0;
     igraph_vector_int_t *inneis_1, *inneis_2, *outneis_1, *outneis_2;
-    igraph_integer_t matched_nodes = 0;
-    igraph_integer_t depth;
-    igraph_integer_t cand1, cand2;
-    igraph_integer_t last1, last2;
+    igraph_int_t matched_nodes = 0;
+    igraph_int_t depth;
+    igraph_int_t cand1, cand2;
+    igraph_int_t last1, last2;
     igraph_stack_int_t path;
     igraph_lazy_adjlist_t inadj1, inadj2, outadj1, outadj2;
     igraph_vector_int_t indeg1, indeg2, outdeg1, outdeg2;
-    igraph_integer_t vsize;
+    igraph_int_t vsize;
 
     IGRAPH_CHECK(igraph_i_perform_vf2_pre_checks(graph1, graph2));
 
@@ -1152,7 +1131,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
     depth = 0; last1 = -1; last2 = -1;
     while (depth >= 0) {
-        igraph_integer_t i;
+        igraph_int_t i;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -1258,7 +1237,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_1)[i];
+                    igraph_int_t node = VECTOR(*inneis_1)[i];
                     if (VECTOR(in_1)[node] == depth) {
                         VECTOR(in_1)[node] = 0;
                         in_1_size -= 1;
@@ -1270,7 +1249,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_1)[i];
+                    igraph_int_t node = VECTOR(*outneis_1)[i];
                     if (VECTOR(out_1)[node] == depth) {
                         VECTOR(out_1)[node] = 0;
                         out_1_size -= 1;
@@ -1282,7 +1261,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_2)[i];
+                    igraph_int_t node = VECTOR(*inneis_2)[i];
                     if (VECTOR(in_2)[node] == depth) {
                         VECTOR(in_2)[node] = 0;
                         in_2_size -= 1;
@@ -1294,7 +1273,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_2)[i];
+                    igraph_int_t node = VECTOR(*outneis_2)[i];
                     if (VECTOR(out_2)[node] == depth) {
                         VECTOR(out_2)[node] = 0;
                         out_2_size -= 1;
@@ -1308,7 +1287,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
         } else {
             /**************************************************************/
             /* step forward if worth, check if worth first */
-            igraph_integer_t xin1 = 0, xin2 = 0, xout1 = 0, xout2 = 0;
+            igraph_int_t xin1 = 0, xin2 = 0, xout1 = 0, xout2 = 0;
             igraph_bool_t end = false;
 
             inneis_1 = igraph_lazy_adjlist_get(&inadj1, cand1);
@@ -1333,7 +1312,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
             vsize = igraph_vector_int_size(inneis_1);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*inneis_1)[i];
+                igraph_int_t node = VECTOR(*inneis_1)[i];
                 if (VECTOR(*core_1)[node] < 0) {
                     if (VECTOR(in_1)[node] != 0) {
                         xin1++;
@@ -1345,7 +1324,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(outneis_1);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*outneis_1)[i];
+                igraph_int_t node = VECTOR(*outneis_1)[i];
                 if (VECTOR(*core_1)[node] < 0) {
                     if (VECTOR(in_1)[node] != 0) {
                         xin1++;
@@ -1357,14 +1336,14 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(inneis_2);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*inneis_2)[i];
+                igraph_int_t node = VECTOR(*inneis_2)[i];
                 if (VECTOR(*core_2)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_2)[node];
+                    igraph_int_t node2 = VECTOR(*core_2)[node];
                     /* check if there is a node2->cand1 edge */
                     if (!igraph_vector_int_contains_sorted(inneis_1, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, node2, cand1, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, node, cand2, IGRAPH_DIRECTED,
@@ -1389,14 +1368,14 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
             }
             vsize = igraph_vector_int_size(outneis_2);
             for (i = 0; !end && i < vsize; i++) {
-                igraph_integer_t node = VECTOR(*outneis_2)[i];
+                igraph_int_t node = VECTOR(*outneis_2)[i];
                 if (VECTOR(*core_2)[node] >= 0) {
-                    igraph_integer_t node2 = VECTOR(*core_2)[node];
+                    igraph_int_t node2 = VECTOR(*core_2)[node];
                     /* check if there is a cand1->node2 edge */
                     if (!igraph_vector_int_contains_sorted(outneis_1, node2)) {
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
-                        igraph_integer_t eid1, eid2;
+                        igraph_int_t eid1, eid2;
                         igraph_get_eid(graph1, &eid1, cand1, node2, IGRAPH_DIRECTED,
                                        /*error=*/ true);
                         igraph_get_eid(graph2, &eid2, cand2, node, IGRAPH_DIRECTED,
@@ -1448,7 +1427,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_1)[i];
+                    igraph_int_t node = VECTOR(*inneis_1)[i];
                     if (VECTOR(in_1)[node] == 0 && VECTOR(*core_1)[node] < 0) {
                         VECTOR(in_1)[node] = depth;
                         in_1_size += 1;
@@ -1460,7 +1439,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_1);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_1)[i];
+                    igraph_int_t node = VECTOR(*outneis_1)[i];
                     if (VECTOR(out_1)[node] == 0 && VECTOR(*core_1)[node] < 0) {
                         VECTOR(out_1)[node] = depth;
                         out_1_size += 1;
@@ -1472,7 +1451,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(inneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*inneis_2)[i];
+                    igraph_int_t node = VECTOR(*inneis_2)[i];
                     if (VECTOR(in_2)[node] == 0 && VECTOR(*core_2)[node] < 0) {
                         VECTOR(in_2)[node] = depth;
                         in_2_size += 1;
@@ -1484,7 +1463,7 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
 
                 vsize = igraph_vector_int_size(outneis_2);
                 for (i = 0; i < vsize; i++) {
-                    igraph_integer_t node = VECTOR(*outneis_2)[i];
+                    igraph_int_t node = VECTOR(*outneis_2)[i];
                     if (VECTOR(out_2)[node] == 0 && VECTOR(*core_2)[node] < 0) {
                         VECTOR(out_2)[node] = depth;
                         out_2_size += 1;
@@ -1545,26 +1524,6 @@ static igraph_error_t igraph_i_subisomorphic_vf2_cb(
     return IGRAPH_STOP;
 }
 
-/**
- * \function igraph_subisomorphic_function_vf2
- * \brief Generic VF2 function for subgraph isomorphism problems (deprecated alias).
- *
- * \deprecated-by igraph_get_subisomorphisms_vf2_callback 0.10.0
- */
-igraph_error_t igraph_subisomorphic_function_vf2(
-    const igraph_t *graph1, const igraph_t *graph2,
-    const igraph_vector_int_t *vertex_color1, const igraph_vector_int_t *vertex_color2,
-    const igraph_vector_int_t *edge_color1, const igraph_vector_int_t *edge_color2,
-    igraph_vector_int_t *map12, igraph_vector_int_t *map21,
-    igraph_isohandler_t *isohandler_fn, igraph_isocompat_t *node_compat_fn,
-    igraph_isocompat_t *edge_compat_fn, void *arg
-) {
-    return igraph_get_subisomorphisms_vf2_callback(
-        graph1, graph2, vertex_color1, vertex_color2, edge_color1, edge_color2,
-        map12, map21, isohandler_fn, node_compat_fn, edge_compat_fn, arg
-    );
-}
-
 /**
  * \function igraph_subisomorphic_vf2
  * Decide subgraph isomorphism using VF2
@@ -1646,7 +1605,7 @@ static igraph_error_t igraph_i_count_subisomorphisms_vf2_cb(
     void *arg
 ) {
     igraph_i_iso_cb_data_t *data = arg;
-    igraph_integer_t *count = data->arg;
+    igraph_int_t *count = data->arg;
     IGRAPH_UNUSED(map12); IGRAPH_UNUSED(map21);
     *count += 1;
     return IGRAPH_SUCCESS;
@@ -1695,7 +1654,7 @@ igraph_error_t igraph_count_subisomorphisms_vf2(const igraph_t *graph1, const ig
                                      const igraph_vector_int_t *vertex_color2,
                                      const igraph_vector_int_t *edge_color1,
                                      const igraph_vector_int_t *edge_color2,
-                                     igraph_integer_t *count,
+                                     igraph_int_t *count,
                                      igraph_isocompat_t *node_compat_fn,
                                      igraph_isocompat_t *edge_compat_fn,
                                      void *arg) {
