@@ -44,9 +44,6 @@ using namespace std;
 #include "igraph_interface.h"
 #include "igraph_progress.h"
 #include "core/interruption.h"
-#ifdef MUSE_MPI
-    #include <mpi.h>
-#endif
 
 namespace drl {
 
@@ -175,11 +172,11 @@ graph::graph(const igraph_t *igraph,
     // scan .int file for node info
     highest_sim = 1.0;
     num_nodes = igraph_vcount(igraph);
-    igraph_integer_t no_of_edges = igraph_ecount(igraph);
-    for (igraph_integer_t i = 0; i < num_nodes; i++) {
+    igraph_int_t no_of_edges = igraph_ecount(igraph);
+    for (igraph_int_t i = 0; i < num_nodes; i++) {
         id_catalog[i] = 1;
     }
-    map<igraph_integer_t, igraph_integer_t>::iterator cat_iter;
+    map<igraph_int_t, igraph_int_t>::iterator cat_iter;
     for ( cat_iter = id_catalog.begin();
           cat_iter != id_catalog.end(); cat_iter++) {
         cat_iter->second = cat_iter->first;
@@ -194,9 +191,9 @@ graph::graph(const igraph_t *igraph,
     }
 
     // read .int file for graph info
-    igraph_integer_t node_1, node_2;
+    igraph_int_t node_1, node_2;
     igraph_real_t weight;
-    for (igraph_integer_t i = 0; i < no_of_edges; i++) {
+    for (igraph_int_t i = 0; i < no_of_edges; i++) {
         node_1 = IGRAPH_FROM(igraph, i);
         node_2 = IGRAPH_TO(igraph, i);
         weight = weights ? VECTOR(*weights)[i] : 1.0 ;
@@ -395,17 +392,17 @@ void graph::init_parms ( int rand_seed, float edge_cut, float real_parm ) {
     cut_rate = ( cut_length_start - cut_length_end ) / 400.0;
 
     // finally set the number of iterations to leave .real coords fixed
-    igraph_integer_t full_comp_iters;
+    igraph_int_t full_comp_iters;
     full_comp_iters = liquid.iterations + expansion.iterations +
                       cooldown.iterations + crunch.iterations + 3;
 
     // adjust real parm to iterations (do not enter simmer halfway)
     if ( real_parm < 0 ) {
-        real_iterations = (igraph_integer_t)real_parm;
+        real_iterations = (igraph_int_t)real_parm;
     } else if ( real_parm == 1) {
         real_iterations = full_comp_iters + simmer.iterations + 100;
     } else {
-        real_iterations = (igraph_integer_t)(real_parm * full_comp_iters);
+        real_iterations = (igraph_int_t)(real_parm * full_comp_iters);
     }
 
     tot_iterations = 0;
@@ -489,8 +486,8 @@ void graph::init_parms(const igraph_layout_drl_options_t *options) {
 // }
 
 int graph::read_real(const igraph_matrix_t *real_mat) {
-    igraph_integer_t n = igraph_matrix_nrow(real_mat);
-    for (igraph_integer_t i = 0; i < n; i++) {
+    igraph_int_t n = igraph_matrix_nrow(real_mat);
+    for (igraph_int_t i = 0; i < n; i++) {
         positions[id_catalog[i]].x = MATRIX(*real_mat, i, 0);
         positions[id_catalog[i]].y = MATRIX(*real_mat, i, 1);
         positions[id_catalog[i]].fixed = false;
@@ -818,7 +815,7 @@ int graph::ReCompute( ) {
 
 void graph::update_nodes ( ) {
 
-    vector<igraph_integer_t> node_indices;           // node list of nodes currently being updated
+    vector<igraph_int_t> node_indices;           // node list of nodes currently being updated
     float old_positions[2 * MAX_PROCS]; // positions before update
     float new_positions[2 * MAX_PROCS]; // positions after update
 
@@ -831,9 +828,9 @@ void graph::update_nodes ( ) {
 
     // next we calculate the number of nodes there would be if the
     // num_nodes by num_procs schedule grid were perfectly square
-    igraph_integer_t square_num_nodes = (igraph_integer_t)(num_procs + num_procs * floor ((float)(num_nodes - 1) / (float)num_procs ));
+    igraph_int_t square_num_nodes = (igraph_int_t)(num_procs + num_procs * floor ((float)(num_nodes - 1) / (float)num_procs ));
 
-    for ( igraph_integer_t i = myid; i < square_num_nodes; i += num_procs ) {
+    for ( igraph_int_t i = myid; i < square_num_nodes; i += num_procs ) {
 
         // get old positions
         get_positions ( node_indices, old_positions );
@@ -842,31 +839,10 @@ void graph::update_nodes ( ) {
         get_positions ( node_indices, new_positions );
 
         if ( i < num_nodes ) {
-
-            // advance random sequence according to myid
-            for ( size_t j = 0; j < 2 * myid; j++ ) {
-                RNG_UNIF01();
-            }
-            // rand();
-
             // calculate node energy possibilities
             if ( !(positions[i].fixed && real_fixed) ) {
                 update_node_pos ( i, old_positions, new_positions );
             }
-
-            // advance random sequence for next iteration
-            for ( size_t j = 2 * myid; j < 2 * (node_indices.size() - 1); j++ ) {
-                RNG_UNIF01();
-            }
-            // rand();
-
-        } else {
-            // advance random sequence according to use by
-            // the other processors
-            for ( size_t j = 0; j < 2 * (node_indices.size()); j++ ) {
-                RNG_UNIF01();
-            }
-            //rand();
         }
 
         // check if anything was actually updated (e.g. everything was fixed)
@@ -878,11 +854,6 @@ void graph::update_nodes ( ) {
 
         // update positions across processors (if not all fixed)
         if ( !all_fixed ) {
-#ifdef MUSE_MPI
-            MPI_Allgather ( &new_positions[2 * myid], 2, MPI_FLOAT,
-                            new_positions, 2, MPI_FLOAT, MPI_COMM_WORLD );
-#endif
-
             // update positions (old to new)
             update_density ( node_indices, old_positions, new_positions );
         }
@@ -919,7 +890,7 @@ void graph::update_nodes ( ) {
 // The get_positions function takes the node_indices list
 // and returns the corresponding positions in an array.
 
-void graph::get_positions ( vector<igraph_integer_t> &node_indices,
+void graph::get_positions ( vector<igraph_int_t> &node_indices,
                             float return_positions[2 * MAX_PROCS]  ) {
 
     // fill positions
@@ -935,7 +906,7 @@ void graph::get_positions ( vector<igraph_integer_t> &node_indices,
 // of active processes at this level for use by the random number
 // generators.
 
-void graph::update_node_pos ( igraph_integer_t node_ind,
+void graph::update_node_pos ( igraph_int_t node_ind,
                               float old_positions[2 * MAX_PROCS],
                               float new_positions[2 * MAX_PROCS] ) {
 
@@ -1007,7 +978,7 @@ void graph::update_node_pos ( igraph_integer_t node_ind,
 // updates the positions by subtracting the old positions and adding the
 // new positions to the density grid.
 
-void graph::update_density ( vector<igraph_integer_t> &node_indices,
+void graph::update_density ( vector<igraph_int_t> &node_indices,
                              float old_positions[2 * MAX_PROCS],
                              float new_positions[2 * MAX_PROCS] ) {
 
@@ -1033,13 +1004,13 @@ void graph::update_density ( vector<igraph_integer_t> &node_indices,
 * original code by B. Wylie.                *
 *********************************************/
 
-float graph::Compute_Node_Energy( igraph_integer_t node_ind ) {
+float graph::Compute_Node_Energy( igraph_int_t node_ind ) {
 
     /* Want to expand 4th power range of attraction */
     float attraction_factor = attraction * attraction *
                               attraction * attraction * 2e-2;
 
-    map <igraph_integer_t, float>::iterator EI;
+    map <igraph_int_t, float>::iterator EI;
     float x_dis, y_dis;
     float energy_distance, weight;
     float node_energy = 0;
@@ -1090,9 +1061,9 @@ float graph::Compute_Node_Energy( igraph_integer_t node_ind ) {
 * originally written by B. Wylie             *
 *********************************************/
 
-void graph::Solve_Analytic( igraph_integer_t node_ind, float &pos_x, float &pos_y ) {
+void graph::Solve_Analytic( igraph_int_t node_ind, float &pos_x, float &pos_y ) {
 
-    map <igraph_integer_t, float>::iterator EI;
+    map <igraph_int_t, float>::iterator EI;
     float total_weight = 0;
     float x_dis, y_dis, x_cen = 0, y_cen = 0;
     float x = 0, y = 0, dis;
@@ -1133,7 +1104,7 @@ void graph::Solve_Analytic( igraph_integer_t node_ind, float &pos_x, float &pos_
     float num_connections = sqrt((double)neighbors[node_ind].size());
     float maxLength = 0;
 
-    map<igraph_integer_t, float>::iterator maxIndex;
+    map<igraph_int_t, float>::iterator maxIndex;
 
     // Go through nodes edges... cutting if necessary
     for (EI = maxIndex = neighbors[node_ind].begin();
@@ -1251,11 +1222,7 @@ float graph::get_tot_energy ( ) {
     //for ( i = positions.begin(); i != positions.end(); i++ )
     //  tot_energy += i->energy;
 
-#ifdef MUSE_MPI
-    MPI_Reduce ( &my_tot_energy, &tot_energy, 1, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD );
-#else
     tot_energy = my_tot_energy;
-#endif
 
     return tot_energy;
 
@@ -1287,17 +1254,17 @@ float graph::get_tot_energy ( ) {
 
 // }
 
-int graph::draw_graph(igraph_matrix_t *res) {
+igraph_error_t graph::draw_graph(igraph_matrix_t *res) {
     while (ReCompute()) {
         IGRAPH_ALLOW_INTERRUPTION();
     }
-    igraph_integer_t n = positions.size();
+    igraph_int_t n = positions.size();
     IGRAPH_CHECK(igraph_matrix_resize(res, n, 2));
-    for (size_t i = 0; i < n; i++) {
+    for (igraph_int_t i = 0; i < n; i++) {
         MATRIX(*res, i, 0) = positions[i].x;
         MATRIX(*res, i, 1) = positions[i].y;
     }
-    return 0;
+    return IGRAPH_SUCCESS;
 }
 
 } // namespace drl
