@@ -1,8 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
-   Copyright (C) 2003-2020  The igraph development team
+   igraph library.
+   Copyright (C) 2003-2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "igraph_layout.h"
@@ -34,7 +29,7 @@
 static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
                               igraph_matrix_t *res,
                               igraph_bool_t use_seed,
-                              igraph_integer_t niter,
+                              igraph_int_t niter,
                               igraph_real_t start_temp,
                               const igraph_vector_t *weight,
                               const igraph_vector_t *minx,
@@ -42,9 +37,8 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
                               const igraph_vector_t *miny,
                               const igraph_vector_t *maxy) {
 
-    igraph_integer_t no_nodes = igraph_vcount(graph);
-    igraph_integer_t no_edges = igraph_ecount(graph);
-    igraph_integer_t i;
+    const igraph_int_t vcount = igraph_vcount(graph);
+    const igraph_int_t ecount = igraph_ecount(graph);
     igraph_vector_t dispx, dispy;
     igraph_real_t temp = start_temp;
     igraph_real_t difftemp = start_temp / niter;
@@ -53,21 +47,17 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
 
     IGRAPH_CHECK(igraph_is_connected(graph, &conn, IGRAPH_WEAK));
     if (!conn) {
-        C = no_nodes * sqrt(no_nodes);
+        C = vcount * sqrt(vcount);
     }
 
-    RNG_BEGIN();
-
     if (!use_seed) {
         igraph_i_layout_random_bounded(graph, res, minx, maxx, miny, maxy);
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&dispx, no_nodes);
-    IGRAPH_VECTOR_INIT_FINALLY(&dispy, no_nodes);
-
-    for (i = 0; i < niter; i++) {
-        igraph_integer_t v, u, e;
+    IGRAPH_VECTOR_INIT_FINALLY(&dispx, vcount);
+    IGRAPH_VECTOR_INIT_FINALLY(&dispy, vcount);
 
+    for (igraph_int_t i = 0; i < niter; i++) {
         IGRAPH_ALLOW_INTERRUPTION();
 
         /* calculate repulsive forces, we have a special version
@@ -75,8 +65,8 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
         igraph_vector_null(&dispx);
         igraph_vector_null(&dispy);
         if (conn) {
-            for (v = 0; v < no_nodes; v++) {
-                for (u = v + 1; u < no_nodes; u++) {
+            for (igraph_int_t v = 0; v < vcount; v++) {
+                for (igraph_int_t u = v + 1; u < vcount; u++) {
                     igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
                     igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
                     igraph_real_t dlen = dx * dx + dy * dy;
@@ -94,8 +84,8 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
                 }
             }
         } else {
-            for (v = 0; v < no_nodes; v++) {
-                for (u = v + 1; u < no_nodes; u++) {
+            for (igraph_int_t v = 0; v < vcount; v++) {
+                for (igraph_int_t u = v + 1; u < vcount; u++) {
                     igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
                     igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
                     igraph_real_t dlen, rdlen;
@@ -118,10 +108,10 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
         }
 
         /* calculate attractive forces */
-        for (e = 0; e < no_edges; e++) {
+        for (igraph_int_t e = 0; e < ecount; e++) {
             /* each edge is an ordered pair of vertices v and u */
-            igraph_integer_t v = IGRAPH_FROM(graph, e);
-            igraph_integer_t u = IGRAPH_TO(graph, e);
+            igraph_int_t v = IGRAPH_FROM(graph, e);
+            igraph_int_t u = IGRAPH_TO(graph, e);
             igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
             igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
             igraph_real_t w = weight ? VECTOR(*weight)[e] : 1.0;
@@ -134,7 +124,7 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
 
         /* limit max displacement to temperature t and prevent from
            displacement outside frame */
-        for (v = 0; v < no_nodes; v++) {
+        for (igraph_int_t v = 0; v < vcount; v++) {
             igraph_real_t dx = VECTOR(dispx)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t dy = VECTOR(dispy)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t displen = sqrt(dx * dx + dy * dy);
@@ -165,8 +155,6 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
         temp -= difftemp;
     }
 
-    RNG_END();
-
     igraph_vector_destroy(&dispx);
     igraph_vector_destroy(&dispy);
     IGRAPH_FINALLY_CLEAN(2);
@@ -177,24 +165,21 @@ static igraph_error_t igraph_layout_i_fr(const igraph_t *graph,
 static igraph_error_t igraph_layout_i_grid_fr(
         const igraph_t *graph,
         igraph_matrix_t *res, igraph_bool_t use_seed,
-        igraph_integer_t niter, igraph_real_t start_temp,
+        igraph_int_t niter, igraph_real_t start_temp,
         const igraph_vector_t *weight, const igraph_vector_t *minx,
         const igraph_vector_t *maxx, const igraph_vector_t *miny,
         const igraph_vector_t *maxy) {
 
-    igraph_integer_t no_nodes = igraph_vcount(graph);
-    igraph_integer_t no_edges = igraph_ecount(graph);
-    igraph_real_t width = sqrt(no_nodes), height = width;
+    const igraph_int_t vcount = igraph_vcount(graph);
+    const igraph_int_t ecount = igraph_ecount(graph);
+    const igraph_real_t width = sqrt(vcount), height = width;
     igraph_2dgrid_t grid;
     igraph_vector_t dispx, dispy;
     igraph_real_t temp = start_temp;
     igraph_real_t difftemp = start_temp / niter;
     igraph_2dgrid_iterator_t vidit;
-    igraph_integer_t i;
     const igraph_real_t cellsize = 2.0;
 
-    RNG_BEGIN();
-
     if (!use_seed) {
         igraph_i_layout_random_bounded(graph, res, minx, maxx, miny, maxy);
     }
@@ -205,15 +190,15 @@ static igraph_error_t igraph_layout_i_grid_fr(
     IGRAPH_FINALLY(igraph_2dgrid_destroy, &grid);
 
     /* place vertices on grid */
-    for (i = 0; i < no_nodes; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         igraph_2dgrid_add2(&grid, i);
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&dispx, no_nodes);
-    IGRAPH_VECTOR_INIT_FINALLY(&dispy, no_nodes);
+    IGRAPH_VECTOR_INIT_FINALLY(&dispx, vcount);
+    IGRAPH_VECTOR_INIT_FINALLY(&dispy, vcount);
 
-    for (i = 0; i < niter; i++) {
-        igraph_integer_t v, u, e;
+    for (igraph_int_t i = 0; i < niter; i++) {
+        igraph_int_t v, u;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -242,9 +227,9 @@ static igraph_error_t igraph_layout_i_grid_fr(
         }
 
         /* attraction */
-        for (e = 0; e < no_edges; e++) {
-            igraph_integer_t v = IGRAPH_FROM(graph, e);
-            igraph_integer_t u = IGRAPH_TO(graph, e);
+        for (igraph_int_t e = 0; e < ecount; e++) {
+            igraph_int_t v = IGRAPH_FROM(graph, e);
+            igraph_int_t u = IGRAPH_TO(graph, e);
             igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
             igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
             igraph_real_t w = weight ? VECTOR(*weight)[e] : 1.0;
@@ -256,7 +241,7 @@ static igraph_error_t igraph_layout_i_grid_fr(
         }
 
         /* update */
-        for (v = 0; v < no_nodes; v++) {
+        for (v = 0; v < vcount; v++) {
             igraph_real_t dx = VECTOR(dispx)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t dy = VECTOR(dispy)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t displen = sqrt(dx * dx + dy * dy);
@@ -328,7 +313,7 @@ static igraph_error_t igraph_layout_i_grid_fr(
  * \param graph Pointer to an initialized graph object.
  * \param res Pointer to an initialized matrix object. This will
  *        contain the result and will be resized as needed.
- * \param use_seed Logical, if true the supplied values in the
+ * \param use_seed If true the supplied values in the
  *        \p res argument are used as an initial layout, if
  *        false a random initial layout is used.
  * \param niter The number of iterations to do. A reasonable
@@ -343,10 +328,11 @@ static igraph_error_t igraph_layout_i_grid_fr(
  *        IGRAPH_LAYOUT_AUTOGRID. The last one uses the grid based
  *        version only for large graphs, currently the ones with
  *        more than 1000 vertices.
- * \param weights Pointer to a vector containing edge weights,
- *        the attraction along the edges will be multiplied by these.
- *        Weights must be positive.
- *        It will be ignored if it is a null-pointer.
+ * \param weights Pointer to a vector containing edge weights. Weights must
+ *        be positive. If \c NULL, all edges are assumed to have weight 1.
+ *        The attraction along the edges will be multiplied by the weights,
+ *        resulting in vertices connected by a high-weight edge being placed
+ *        closer together.
  * \param minx Pointer to a vector, or a \c NULL pointer. If not a
  *        \c NULL pointer then the vector gives the minimum
  *        \quote x \endquote coordinate for every vertex.
@@ -367,7 +353,7 @@ static igraph_error_t igraph_layout_i_grid_fr(
 igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
                                        igraph_matrix_t *res,
                                        igraph_bool_t use_seed,
-                                       igraph_integer_t niter,
+                                       igraph_int_t niter,
                                        igraph_real_t start_temp,
                                        igraph_layout_grid_t grid,
                                        const igraph_vector_t *weights,
@@ -376,40 +362,40 @@ igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
                                        const igraph_vector_t *miny,
                                        const igraph_vector_t *maxy) {
 
-    igraph_integer_t no_nodes = igraph_vcount(graph);
-    igraph_integer_t no_edges = igraph_ecount(graph);
+    const igraph_int_t vcount = igraph_vcount(graph);
+    const igraph_int_t ecount = igraph_ecount(graph);
 
     if (niter < 0) {
         IGRAPH_ERROR("Number of iterations must be non-negative in "
                      "Fruchterman-Reingold layout.", IGRAPH_EINVAL);
     }
 
-    if (use_seed && (igraph_matrix_nrow(res) != no_nodes ||
+    if (use_seed && (igraph_matrix_nrow(res) != vcount ||
                      igraph_matrix_ncol(res) != 2)) {
         IGRAPH_ERROR("Invalid start position matrix size in "
                      "Fruchterman-Reingold layout.", IGRAPH_EINVAL);
     }
 
-    if (weights && igraph_vector_size(weights) != no_edges) {
+    if (weights && igraph_vector_size(weights) != ecount) {
         IGRAPH_ERROR("Invalid weight vector length.", IGRAPH_EINVAL);
     }
-    if (weights && no_edges > 0 && igraph_vector_min(weights) <= 0) {
+    if (weights && ecount > 0 && igraph_vector_min(weights) <= 0) {
         IGRAPH_ERROR("Weights must be positive for Fruchterman-Reingold layout.", IGRAPH_EINVAL);
     }
 
-    if (minx && igraph_vector_size(minx) != no_nodes) {
+    if (minx && igraph_vector_size(minx) != vcount) {
         IGRAPH_ERROR("Invalid minx vector length.", IGRAPH_EINVAL);
     }
-    if (maxx && igraph_vector_size(maxx) != no_nodes) {
+    if (maxx && igraph_vector_size(maxx) != vcount) {
         IGRAPH_ERROR("Invalid maxx vector length.", IGRAPH_EINVAL);
     }
     if (minx && maxx && !igraph_vector_all_le(minx, maxx)) {
         IGRAPH_ERROR("minx must not be greater than maxx.", IGRAPH_EINVAL);
     }
-    if (miny && igraph_vector_size(miny) != no_nodes) {
+    if (miny && igraph_vector_size(miny) != vcount) {
         IGRAPH_ERROR("Invalid miny vector length.", IGRAPH_EINVAL);
     }
-    if (maxy && igraph_vector_size(maxy) != no_nodes) {
+    if (maxy && igraph_vector_size(maxy) != vcount) {
         IGRAPH_ERROR("Invalid maxy vector length.", IGRAPH_EINVAL);
     }
     if (miny && maxy && !igraph_vector_all_le(miny, maxy)) {
@@ -417,7 +403,7 @@ igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
     }
 
     if (grid == IGRAPH_LAYOUT_AUTOGRID) {
-        if (no_nodes > 1000) {
+        if (vcount > 1000) {
             grid = IGRAPH_LAYOUT_GRID;
         } else {
             grid = IGRAPH_LAYOUT_NOGRID;
@@ -443,7 +429,7 @@ igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
  * \param graph Pointer to an initialized graph object.
  * \param res Pointer to an initialized matrix object. This will
  *        contain the result and will be resized as needed.
- * \param use_seed Logical, if true the supplied values in the
+ * \param use_seed If true the supplied values in the
  *        \p res argument are used as an initial layout, if
  *        false a random initial layout is used.
  * \param niter The number of iterations to do. A reasonable
@@ -452,10 +438,11 @@ igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
  *        of movement alloved along one axis, within one step, for a
  *        vertex. Currently it is decreased linearly to zero during
  *        the iteration.
- * \param weights Pointer to a vector containing edge weights,
- *        the attraction along the edges will be multiplied by these.
- *        Weights must be positive.
- *        It will be ignored if it is a null-pointer.
+ * \param weights Pointer to a vector containing edge weights. Weights must
+ *        be positive. If \c NULL, all edges are assumed to have weight 1.
+ *        The attraction along the edges will be multiplied by the weights,
+ *        resulting in vertices connected by a high-weight edge being placed
+ *        closer together.
  * \param minx Pointer to a vector, or a \c NULL pointer. If not a
  *        \c NULL pointer then the vector gives the minimum
  *        \quote x \endquote coordinate for every vertex.
@@ -484,7 +471,7 @@ igraph_error_t igraph_layout_fruchterman_reingold(const igraph_t *graph,
 igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
         igraph_matrix_t *res,
         igraph_bool_t use_seed,
-        igraph_integer_t niter,
+        igraph_int_t niter,
         igraph_real_t start_temp,
         const igraph_vector_t *weights,
         const igraph_vector_t *minx,
@@ -494,9 +481,8 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
         const igraph_vector_t *minz,
         const igraph_vector_t *maxz) {
 
-    const igraph_integer_t no_nodes = igraph_vcount(graph);
-    const igraph_integer_t no_edges = igraph_ecount(graph);
-    igraph_integer_t i;
+    const igraph_int_t vcount = igraph_vcount(graph);
+    const igraph_int_t ecount = igraph_ecount(graph);
     igraph_vector_t dispx, dispy, dispz;
     igraph_real_t temp = start_temp;
     igraph_real_t difftemp = start_temp / niter;
@@ -508,7 +494,7 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
                      "Fruchterman-Reingold layout", IGRAPH_EINVAL);
     }
 
-    if (use_seed && (igraph_matrix_nrow(res) != no_nodes ||
+    if (use_seed && (igraph_matrix_nrow(res) != vcount ||
                      igraph_matrix_ncol(res) != 3)) {
         IGRAPH_ERROR("Invalid start position matrix size in "
                      "Fruchterman-Reingold layout", IGRAPH_EINVAL);
@@ -517,32 +503,32 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
     if (weights && igraph_vector_size(weights) != igraph_ecount(graph)) {
         IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
     }
-    if (weights && no_edges > 0 && igraph_vector_min(weights) <= 0) {
+    if (weights && ecount > 0 && igraph_vector_min(weights) <= 0) {
         IGRAPH_ERROR("Weights must be positive for Fruchterman-Reingold layout.", IGRAPH_EINVAL);
     }
 
-    if (minx && igraph_vector_size(minx) != no_nodes) {
+    if (minx && igraph_vector_size(minx) != vcount) {
         IGRAPH_ERROR("Invalid minx vector length", IGRAPH_EINVAL);
     }
-    if (maxx && igraph_vector_size(maxx) != no_nodes) {
+    if (maxx && igraph_vector_size(maxx) != vcount) {
         IGRAPH_ERROR("Invalid maxx vector length", IGRAPH_EINVAL);
     }
     if (minx && maxx && !igraph_vector_all_le(minx, maxx)) {
         IGRAPH_ERROR("minx must not be greater than maxx", IGRAPH_EINVAL);
     }
-    if (miny && igraph_vector_size(miny) != no_nodes) {
+    if (miny && igraph_vector_size(miny) != vcount) {
         IGRAPH_ERROR("Invalid miny vector length", IGRAPH_EINVAL);
     }
-    if (maxy && igraph_vector_size(maxy) != no_nodes) {
+    if (maxy && igraph_vector_size(maxy) != vcount) {
         IGRAPH_ERROR("Invalid maxy vector length", IGRAPH_EINVAL);
     }
     if (miny && maxy && !igraph_vector_all_le(miny, maxy)) {
         IGRAPH_ERROR("miny must not be greater than maxy", IGRAPH_EINVAL);
     }
-    if (minz && igraph_vector_size(minz) != no_nodes) {
+    if (minz && igraph_vector_size(minz) != vcount) {
         IGRAPH_ERROR("Invalid minz vector length", IGRAPH_EINVAL);
     }
-    if (maxz && igraph_vector_size(maxz) != no_nodes) {
+    if (maxz && igraph_vector_size(maxz) != vcount) {
         IGRAPH_ERROR("Invalid maxz vector length", IGRAPH_EINVAL);
     }
     if (minz && maxz && !igraph_vector_all_le(minz, maxz)) {
@@ -551,22 +537,18 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 
     IGRAPH_CHECK(igraph_is_connected(graph, &conn, IGRAPH_WEAK));
     if (!conn) {
-        C = no_nodes * sqrt(no_nodes);
+        C = vcount * sqrt(vcount);
     }
 
-    RNG_BEGIN();
-
     if (!use_seed) {
         igraph_i_layout_random_bounded_3d(graph, res, minx, maxx, miny, maxy, minz, maxz);
     }
 
-    IGRAPH_VECTOR_INIT_FINALLY(&dispx, no_nodes);
-    IGRAPH_VECTOR_INIT_FINALLY(&dispy, no_nodes);
-    IGRAPH_VECTOR_INIT_FINALLY(&dispz, no_nodes);
-
-    for (i = 0; i < niter; i++) {
-        igraph_integer_t v, u, e;
+    IGRAPH_VECTOR_INIT_FINALLY(&dispx, vcount);
+    IGRAPH_VECTOR_INIT_FINALLY(&dispy, vcount);
+    IGRAPH_VECTOR_INIT_FINALLY(&dispz, vcount);
 
+    for (igraph_int_t i = 0; i < niter; i++) {
         IGRAPH_ALLOW_INTERRUPTION();
 
         /* calculate repulsive forces, we have a special version
@@ -575,8 +557,8 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
         igraph_vector_null(&dispy);
         igraph_vector_null(&dispz);
         if (conn) {
-            for (v = 0; v < no_nodes; v++) {
-                for (u = v + 1; u < no_nodes; u++) {
+            for (igraph_int_t v = 0; v < vcount; v++) {
+                for (igraph_int_t u = v + 1; u < vcount; u++) {
                     igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
                     igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
                     igraph_real_t dz = MATRIX(*res, v, 2) - MATRIX(*res, u, 2);
@@ -598,8 +580,8 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
                 }
             }
         } else {
-            for (v = 0; v < no_nodes; v++) {
-                for (u = v + 1; u < no_nodes; u++) {
+            for (igraph_int_t v = 0; v < vcount; v++) {
+                for (igraph_int_t u = v + 1; u < vcount; u++) {
                     igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
                     igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
                     igraph_real_t dz = MATRIX(*res, v, 2) - MATRIX(*res, u, 2);
@@ -626,10 +608,10 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
         }
 
         /* calculate attractive forces */
-        for (e = 0; e < no_edges; e++) {
+        for (igraph_int_t e = 0; e < ecount; e++) {
             /* each edges is an ordered pair of vertices v and u */
-            igraph_integer_t v = IGRAPH_FROM(graph, e);
-            igraph_integer_t u = IGRAPH_TO(graph, e);
+            igraph_int_t v = IGRAPH_FROM(graph, e);
+            igraph_int_t u = IGRAPH_TO(graph, e);
             igraph_real_t dx = MATRIX(*res, v, 0) - MATRIX(*res, u, 0);
             igraph_real_t dy = MATRIX(*res, v, 1) - MATRIX(*res, u, 1);
             igraph_real_t dz = MATRIX(*res, v, 2) - MATRIX(*res, u, 2);
@@ -645,7 +627,7 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
 
         /* limit max displacement to temperature t and prevent from
            displacement outside frame */
-        for (v = 0; v < no_nodes; v++) {
+        for (igraph_int_t v = 0; v < vcount; v++) {
             igraph_real_t dx = VECTOR(dispx)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t dy = VECTOR(dispy)[v] + RNG_UNIF(-1e-9, 1e-9);
             igraph_real_t dz = VECTOR(dispz)[v] + RNG_UNIF(-1e-9, 1e-9);
@@ -685,8 +667,6 @@ igraph_error_t igraph_layout_fruchterman_reingold_3d(const igraph_t *graph,
         temp -= difftemp;
     }
 
-    RNG_END();
-
     igraph_vector_destroy(&dispx);
     igraph_vector_destroy(&dispy);
     igraph_vector_destroy(&dispz);
