@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2020  The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -32,18 +30,18 @@
 #include "igraph_progress.h"
 #include "igraph_structural.h"
 
-#include "core/math.h"
+#include "core/math.h" /* M_PI */
 
-static igraph_error_t igraph_i_layout_reingold_tilford_unreachable(
+static igraph_error_t layout_reingold_tilford_unreachable(
     const igraph_t *graph,
     igraph_neimode_t mode,
-    igraph_integer_t real_root,
-    igraph_integer_t no_of_nodes,
+    igraph_int_t real_root,
+    igraph_int_t no_of_nodes,
     igraph_vector_int_t *pnewedges) {
 
-    igraph_integer_t no_of_newedges;
+    igraph_int_t no_of_newedges;
     igraph_vector_bool_t visited;
-    igraph_integer_t i, j, n;
+    igraph_int_t i, j, n;
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
     igraph_adjlist_t allneis;
     igraph_vector_int_t *neis;
@@ -61,12 +59,12 @@ static igraph_error_t igraph_i_layout_reingold_tilford_unreachable(
     /* start from real_root and go BFS */
     IGRAPH_CHECK(igraph_dqueue_int_push(&q, real_root));
     while (!igraph_dqueue_int_empty(&q)) {
-        igraph_integer_t actnode = igraph_dqueue_int_pop(&q);
+        igraph_int_t actnode = igraph_dqueue_int_pop(&q);
         neis = igraph_adjlist_get(&allneis, actnode);
         n = igraph_vector_int_size(neis);
-        VECTOR(visited)[actnode] = 1;
+        VECTOR(visited)[actnode] = true;
         for (j = 0; j < n; j++) {
-            igraph_integer_t neighbor = VECTOR(*neis)[j];
+            igraph_int_t neighbor = VECTOR(*neis)[j];
             if (!VECTOR(visited)[neighbor]) {
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
             }
@@ -74,13 +72,13 @@ static igraph_error_t igraph_i_layout_reingold_tilford_unreachable(
     }
 
     for (j = 0; j < no_of_nodes; j++) {
-        no_of_newedges += 1 - VECTOR(visited)[j];
+        no_of_newedges += VECTOR(visited)[j] ? 0 : 1;
     }
 
     /* if any nodes are unreachable, add edges between them and real_root */
     if (no_of_newedges != 0) {
 
-        igraph_vector_int_resize(pnewedges, no_of_newedges * 2);
+        IGRAPH_CHECK(igraph_vector_int_resize(pnewedges, no_of_newedges * 2));
         j = 0;
         for (i = 0; i < no_of_nodes; i++) {
             if (!VECTOR(visited)[i]) {
@@ -106,41 +104,41 @@ static igraph_error_t igraph_i_layout_reingold_tilford_unreachable(
 
 
 /* Internal structure for Reingold-Tilford layout */
-struct igraph_i_reingold_tilford_vertex {
-    igraph_integer_t parent;        /* Parent node index */
-    igraph_integer_t level;         /* Level of the node */
+struct reingold_tilford_vertex {
+    igraph_int_t parent;        /* Parent node index */
+    igraph_int_t level;         /* Level of the node */
     igraph_real_t offset;     /* X offset from parent node */
-    igraph_integer_t left_contour;  /* Next left node of the contour
+    igraph_int_t left_contour;  /* Next left node of the contour
               of the subtree rooted at this node */
-    igraph_integer_t right_contour; /* Next right node of the contour
+    igraph_int_t right_contour; /* Next right node of the contour
               of the subtree rooted at this node */
     igraph_real_t offset_to_left_contour;  /* X offset when following the left contour */
     igraph_real_t offset_to_right_contour;  /* X offset when following the right contour */
-    igraph_integer_t left_extreme;  /* Leftmost node on the deepest layer of the subtree rooted at this node */
-    igraph_integer_t right_extreme; /* Rightmost node on the deepest layer of the subtree rooted at this node */
+    igraph_int_t left_extreme;  /* Leftmost node on the deepest layer of the subtree rooted at this node */
+    igraph_int_t right_extreme; /* Rightmost node on the deepest layer of the subtree rooted at this node */
     igraph_real_t offset_to_left_extreme;  /* X offset when jumping to the left extreme node */
     igraph_real_t offset_to_right_extreme;  /* X offset when jumping to the right extreme node */
 };
 
-static igraph_error_t igraph_i_layout_reingold_tilford_postorder(struct igraph_i_reingold_tilford_vertex *vdata,
-                                                      igraph_integer_t node, igraph_integer_t vcount);
-static igraph_error_t igraph_i_layout_reingold_tilford_calc_coords(struct igraph_i_reingold_tilford_vertex *vdata,
-                                                        igraph_matrix_t *res, igraph_integer_t node,
-                                                        igraph_integer_t vcount, igraph_real_t xpos);
+static void layout_reingold_tilford_postorder(struct reingold_tilford_vertex *vdata,
+                                                      igraph_int_t node, igraph_int_t vcount);
+static void layout_reingold_tilford_calc_coords(struct reingold_tilford_vertex *vdata,
+                                                        igraph_matrix_t *res, igraph_int_t node,
+                                                        igraph_int_t vcount, igraph_real_t xpos);
 
 /* uncomment the next line for debugging the Reingold-Tilford layout */
 /* #define LAYOUT_RT_DEBUG 1 */
 
-static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
+static igraph_error_t layout_reingold_tilford(const igraph_t *graph,
                                             igraph_matrix_t *res,
                                             igraph_neimode_t mode,
-                                            igraph_integer_t root) {
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t i, n, j;
+                                            igraph_int_t root) {
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t i, n, j;
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
     igraph_adjlist_t allneis;
     igraph_vector_int_t *neis;
-    struct igraph_i_reingold_tilford_vertex *vdata;
+    struct reingold_tilford_vertex *vdata;
 
     IGRAPH_CHECK(igraph_matrix_resize(res, no_of_nodes, 2));
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 100);
@@ -148,7 +146,7 @@ static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
     IGRAPH_CHECK(igraph_adjlist_init(graph, &allneis, mode, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
     IGRAPH_FINALLY(igraph_adjlist_destroy, &allneis);
 
-    vdata = IGRAPH_CALLOC(no_of_nodes, struct igraph_i_reingold_tilford_vertex);
+    vdata = IGRAPH_CALLOC(no_of_nodes, struct reingold_tilford_vertex);
     if (vdata == 0) {
         IGRAPH_ERROR("igraph_layout_reingold_tilford failed", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
     }
@@ -175,13 +173,13 @@ static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
     IGRAPH_CHECK(igraph_dqueue_int_push(&q, root));
     IGRAPH_CHECK(igraph_dqueue_int_push(&q, 0));
     while (!igraph_dqueue_int_empty(&q)) {
-        igraph_integer_t actnode = igraph_dqueue_int_pop(&q);
-        igraph_integer_t actdist = igraph_dqueue_int_pop(&q);
+        igraph_int_t actnode = igraph_dqueue_int_pop(&q);
+        igraph_int_t actdist = igraph_dqueue_int_pop(&q);
         neis = igraph_adjlist_get(&allneis, actnode);
         n = igraph_vector_int_size(neis);
 
         for (j = 0; j < n; j++) {
-            igraph_integer_t neighbor = VECTOR(*neis)[j];
+            igraph_int_t neighbor = VECTOR(*neis)[j];
             if (vdata[neighbor].parent >= 0) {
                 continue;
             }
@@ -195,10 +193,10 @@ static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
 
     /* Step 2: postorder tree traversal, determines the appropriate X
      * offsets for every node */
-    igraph_i_layout_reingold_tilford_postorder(vdata, root, no_of_nodes);
+    layout_reingold_tilford_postorder(vdata, root, no_of_nodes);
 
     /* Step 3: calculate real coordinates based on X offsets */
-    igraph_i_layout_reingold_tilford_calc_coords(vdata, res, root, no_of_nodes, vdata[root].offset);
+    layout_reingold_tilford_calc_coords(vdata, res, root, no_of_nodes, vdata[root].offset);
 
     igraph_dqueue_int_destroy(&q);
     igraph_adjlist_destroy(&allneis);
@@ -210,14 +208,14 @@ static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
 #ifdef LAYOUT_RT_DEBUG
     for (i = 0; i < no_of_nodes; i++) {
         printf(
-            "%3ld: offset = %.2f, contours = [%ld, %ld], contour offsets = [%.2f, %.2f]\n",
+            "%3" IGRAPH_PRId ": offset = %.2f, contours = [%" IGRAPH_PRId ", %" IGRAPH_PRId "], contour offsets = [%.2f, %.2f]\n",
             i, vdata[i].offset,
             vdata[i].left_contour, vdata[i].right_contour,
             vdata[i].offset_to_left_contour, vdata[i].offset_to_right_contour
         );
         if (vdata[i].left_extreme != i || vdata[i].right_extreme != i) {
             printf(
-                "     extrema = [%ld, %ld], offsets to extrema = [%.2f, %.2f]\n",
+                "     extrema = [%" IGRAPH_PRId ", %" IGRAPH_PRId "], offsets to extrema = [%.2f, %.2f]\n",
                 vdata[i].left_extreme, vdata[i].right_extreme,
                 vdata[i].offset_to_left_extreme, vdata[i].offset_to_right_extreme
             );
@@ -228,50 +226,50 @@ static igraph_error_t igraph_i_layout_reingold_tilford(const igraph_t *graph,
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_i_layout_reingold_tilford_calc_coords(
-        struct igraph_i_reingold_tilford_vertex *vdata,
-        igraph_matrix_t *res, igraph_integer_t node,
-        igraph_integer_t vcount, igraph_real_t xpos) {
-    igraph_integer_t i;
+static void layout_reingold_tilford_calc_coords(
+        struct reingold_tilford_vertex *vdata,
+        igraph_matrix_t *res, igraph_int_t node,
+        igraph_int_t vcount, igraph_real_t xpos) {
+
     MATRIX(*res, node, 0) = xpos;
-    for (i = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         if (i == node) {
             continue;
         }
         if (vdata[i].parent == node) {
-            igraph_i_layout_reingold_tilford_calc_coords(vdata, res, i, vcount,
+            layout_reingold_tilford_calc_coords(vdata, res, i, vcount,
                     xpos + vdata[i].offset);
         }
     }
-    return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
-        struct igraph_i_reingold_tilford_vertex *vdata,
-        igraph_integer_t node, igraph_integer_t vcount) {
-    igraph_integer_t i, j, childcount, leftroot, leftrootidx;
+static void layout_reingold_tilford_postorder(
+        struct reingold_tilford_vertex *vdata,
+        igraph_int_t node, igraph_int_t vcount) {
+
+    igraph_int_t childcount, leftroot, leftrootidx;
     const igraph_real_t minsep = 1;
     igraph_real_t avg;
 
 #ifdef LAYOUT_RT_DEBUG
-    printf("Starting visiting node %ld\n", node);
+    printf("Starting visiting node %" IGRAPH_PRId "\n", node);
 #endif
 
     /* Check whether this node is a leaf node */
     childcount = 0;
-    for (i = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         if (i == node) {
             continue;
         }
         if (vdata[i].parent == node) {
             /* Node i is a child, so visit it recursively */
             childcount++;
-            igraph_i_layout_reingold_tilford_postorder(vdata, i, vcount);
+            layout_reingold_tilford_postorder(vdata, i, vcount);
         }
     }
 
     if (childcount == 0) {
-        return IGRAPH_SUCCESS;
+        return;
     }
 
     /* Here we can assume that all of the subtrees have been placed and their
@@ -285,9 +283,9 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
     leftroot = leftrootidx = -1;
     avg = 0.0;
 #ifdef LAYOUT_RT_DEBUG
-    printf("Visited node %ld and arranged its subtrees\n", node);
+    printf("Visited node %" IGRAPH_PRId " and arranged its subtrees\n", node);
 #endif
-    for (i = 0, j = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0, j = 0; i < vcount; i++) {
         if (i == node) {
             continue;
         }
@@ -295,11 +293,11 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
             if (leftroot >= 0) {
                 /* Now we will follow the right contour of leftroot and the
                  * left contour of the subtree rooted at i */
-                igraph_integer_t lnode, rnode, auxnode;
+                igraph_int_t lnode, rnode, auxnode;
                 igraph_real_t loffset, roffset, rootsep, newoffset;
 
 #ifdef LAYOUT_RT_DEBUG
-                printf("  Placing child %ld on level %ld, to the right of %ld\n", i, vdata[i].level, leftroot);
+                printf("  Placing child %" IGRAPH_PRId " on level %" IGRAPH_PRId ", to the right of %" IGRAPH_PRId "\n", i, vdata[i].level, leftroot);
 #endif
                 lnode = leftroot; rnode = i;
                 rootsep = vdata[leftroot].offset + minsep;
@@ -311,7 +309,7 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
                 vdata[node].offset_to_right_contour = rootsep;
 
 #ifdef LAYOUT_RT_DEBUG
-                printf("    Contour: [%ld, %ld], offsets: [%lf, %lf], rootsep: %lf\n",
+                printf("    Contour: [%" IGRAPH_PRId ", %" IGRAPH_PRId "], offsets: [%lf, %lf], rootsep: %lf\n",
                        lnode, rnode, loffset, roffset, rootsep);
 #endif
                 while ((lnode >= 0) && (rnode >= 0)) {
@@ -341,8 +339,8 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
                             vdata[node].offset_to_left_extreme = vdata[i].offset_to_left_extreme + rootsep;
                             vdata[node].offset_to_right_extreme = vdata[i].offset_to_right_extreme + rootsep;
 #ifdef LAYOUT_RT_DEBUG
-                            printf("      Left subtree ended earlier, continuing left subtree's left and right contour on right subtree (node %ld gets connected to node %ld)\n", auxnode, vdata[rnode].left_contour);
-                            printf("      New contour following offset for node %ld is %lf\n", auxnode, vdata[auxnode].offset_to_left_contour);
+                            printf("      Left subtree ended earlier, continuing left subtree's left and right contour on right subtree (node %" IGRAPH_PRId " gets connected to node %" IGRAPH_PRId ")\n", auxnode, vdata[rnode].left_contour);
+                            printf("      New contour following offset for node %" IGRAPH_PRId " is %lf\n", auxnode, vdata[auxnode].offset_to_left_contour);
 #endif
                         } else {
                             /* Both subtrees are ending at the same time; the
@@ -376,14 +374,14 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
                              * rooted at 'node' because the right subtree was
                              * smaller */
 #ifdef LAYOUT_RT_DEBUG
-                            printf("      Right subtree ended earlier, continuing right subtree's left and right contour on left subtree (node %ld gets connected to node %ld)\n", auxnode, lnode);
-                            printf("      New contour following offset for node %ld is %lf\n", auxnode, vdata[auxnode].offset_to_left_contour);
+                            printf("      Right subtree ended earlier, continuing right subtree's left and right contour on left subtree (node %" IGRAPH_PRId " gets connected to node %" IGRAPH_PRId ")\n", auxnode, lnode);
+                            printf("      New contour following offset for node %" IGRAPH_PRId " is %lf\n", auxnode, vdata[auxnode].offset_to_left_contour);
 #endif
                         }
                         rnode = -1;
                     }
 #ifdef LAYOUT_RT_DEBUG
-                    printf("    Contour: [%ld, %ld], offsets: [%lf, %lf], rootsep: %lf\n",
+                    printf("    Contour: [%" IGRAPH_PRId ", %" IGRAPH_PRId "], offsets: [%lf, %lf], rootsep: %lf\n",
                            lnode, rnode, loffset, roffset, rootsep);
 #endif
 
@@ -399,7 +397,7 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
                 }
 
 #ifdef LAYOUT_RT_DEBUG
-                printf("  Offset of subtree with root node %ld will be %lf\n", i, rootsep);
+                printf("  Offset of subtree with root node %" IGRAPH_PRId " will be %lf\n", i, rootsep);
 #endif
                 vdata[i].offset = rootsep;
                 vdata[node].offset_to_right_contour = rootsep;
@@ -407,10 +405,10 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
                 leftrootidx = j;
                 leftroot = i;
             } else {
-                /* This is the first child of the node being considered so we
-                 * can simply place the subtree on our virtual canvas */
+                /* This is the first child of the node being considered,
+                 * so we can simply place the subtree on our virtual canvas. */
 #ifdef LAYOUT_RT_DEBUG
-                printf("  Placing child %ld on level %ld as first child\n", i, vdata[i].level);
+                printf("  Placing child %" IGRAPH_PRId " on level %" IGRAPH_PRId " as first child\n", i, vdata[i].level);
 #endif
                 leftrootidx = j;
                 leftroot = i;
@@ -428,13 +426,13 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
         }
     }
 #ifdef LAYOUT_RT_DEBUG
-    printf("Shifting node %ld to be centered above children. Shift amount: %lf\n", node, avg);
+    printf("Shifting node %" IGRAPH_PRId " to be centered above children. Shift amount: %lf\n", node, avg);
 #endif
     vdata[node].offset_to_left_contour -= avg;
     vdata[node].offset_to_right_contour -= avg;
     vdata[node].offset_to_left_extreme -= avg;
     vdata[node].offset_to_right_extreme -= avg;
-    for (i = 0, j = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         if (i == node) {
             continue;
         }
@@ -442,17 +440,15 @@ static igraph_error_t igraph_i_layout_reingold_tilford_postorder(
             vdata[i].offset -= avg;
         }
     }
-
-    return IGRAPH_SUCCESS;
 }
 
 /* This function computes the number of outgoing (or incoming) connections
  * of clusters, represented as a membership vector. It only works with
  * directed graphs. */
-igraph_error_t igraph_i_layout_reingold_tilford_cluster_degrees_directed(
+static igraph_error_t layout_reingold_tilford_cluster_degrees_directed(
         const igraph_t *graph,
         const igraph_vector_int_t *membership,
-        igraph_integer_t no_comps,
+        igraph_int_t no_comps,
         igraph_neimode_t mode,
         igraph_vector_int_t *degrees) {
 
@@ -469,15 +465,15 @@ igraph_error_t igraph_i_layout_reingold_tilford_cluster_degrees_directed(
     IGRAPH_FINALLY(igraph_eit_destroy, &eit);
 
     for (; !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
-        igraph_integer_t eid = IGRAPH_EIT_GET(eit);
+        igraph_int_t eid = IGRAPH_EIT_GET(eit);
 
-        igraph_integer_t from = IGRAPH_FROM(graph, eid);
-        igraph_integer_t to   = IGRAPH_TO(graph, eid);
+        igraph_int_t from = IGRAPH_FROM(graph, eid);
+        igraph_int_t to   = IGRAPH_TO(graph, eid);
 
-        igraph_integer_t from_cl = VECTOR(*membership)[from];
-        igraph_integer_t to_cl   = VECTOR(*membership)[to];
+        igraph_int_t from_cl = VECTOR(*membership)[from];
+        igraph_int_t to_cl   = VECTOR(*membership)[to];
 
-        igraph_integer_t cl = mode == IGRAPH_OUT ? from_cl : to_cl;
+        igraph_int_t cl = mode == IGRAPH_OUT ? from_cl : to_cl;
 
         if (from_cl != to_cl) {
             VECTOR(*degrees)[cl] += 1;
@@ -499,7 +495,7 @@ igraph_error_t igraph_i_layout_reingold_tilford_cluster_degrees_directed(
  * In the directed case, one root is chosen from each strongly connected component
  * that has no incoming (or outgoing) edges (depending on 'mode').
  * When more than one root choice is possible, nodes are prioritized based on
- * either lowest ecccentricity (if 'use_ecccentricity' is true) or based on
+ * either lowest eccentricity (if 'use_eccentricity' is true) or based on
  * highest degree (out- or in-degree in directed mode).
  */
 
@@ -531,7 +527,7 @@ igraph_error_t igraph_i_layout_reingold_tilford_cluster_degrees_directed(
  *           Choose the vertices with the lowest eccentricity. This usually results
  *           in a "wide and shallow" tree layout. While this heuristic produces
  *           high-quality results, it is slow for large graphs: computing the
- *           eccentricities has quadractic complexity in the number of vertices.
+ *           eccentricities has quadratic complexity in the number of vertices.
  *          \endclist
  * \return Error code.
  *
@@ -543,17 +539,17 @@ igraph_error_t igraph_roots_for_tree_layout(
         igraph_vector_int_t *roots,
         igraph_root_choice_t heuristic) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t order, membership;
-    igraph_integer_t no_comps;
-    igraph_integer_t i, j;
+    igraph_int_t no_comps;
+    igraph_int_t i, j;
     igraph_bool_t use_eccentricity;
 
     switch (heuristic) {
     case IGRAPH_ROOT_CHOICE_DEGREE:
-        use_eccentricity = 0; break;
+        use_eccentricity = false; break;
     case IGRAPH_ROOT_CHOICE_ECCENTRICITY:
-        use_eccentricity = 1; break;
+        use_eccentricity = true; break;
     default:
         IGRAPH_ERROR("Invalid root choice heuristic given.", IGRAPH_EINVAL);
     }
@@ -569,13 +565,13 @@ igraph_error_t igraph_roots_for_tree_layout(
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&order, no_of_nodes);
     if (use_eccentricity) {
-        /* Sort vertices by decreasing eccenticity. */
+        /* Sort vertices by decreasing eccentricity. */
 
         igraph_vector_t ecc;
 
         IGRAPH_VECTOR_INIT_FINALLY(&ecc, no_of_nodes);
-        IGRAPH_CHECK(igraph_eccentricity(graph, &ecc, igraph_vss_all(), mode));
-        IGRAPH_CHECK(igraph_vector_qsort_ind(&ecc, &order, IGRAPH_ASCENDING));
+        IGRAPH_CHECK(igraph_eccentricity(graph, NULL, &ecc, igraph_vss_all(), mode));
+        IGRAPH_CHECK(igraph_vector_sort_ind(&ecc, &order, IGRAPH_ASCENDING));
 
         igraph_vector_destroy(&ecc);
         IGRAPH_FINALLY_CLEAN(1);
@@ -606,7 +602,7 @@ igraph_error_t igraph_roots_for_tree_layout(
         igraph_vector_int_t cluster_degrees;
 
         IGRAPH_VECTOR_INT_INIT_FINALLY(&cluster_degrees, no_of_nodes);
-        IGRAPH_CHECK(igraph_i_layout_reingold_tilford_cluster_degrees_directed(
+        IGRAPH_CHECK(layout_reingold_tilford_cluster_degrees_directed(
                          graph, &membership, no_comps,
                          mode == IGRAPH_OUT ? IGRAPH_IN : IGRAPH_OUT, /* reverse direction */
                          &cluster_degrees));
@@ -615,8 +611,8 @@ igraph_error_t igraph_roots_for_tree_layout(
          * and record largest degree node in each strongly-connected component
          * which has no incoming (outgoing) edges. */
         for (i = 0; i < no_of_nodes; ++i) {
-            igraph_integer_t v  = VECTOR(order)[i];
-            igraph_integer_t cl = VECTOR(membership)[v];
+            igraph_int_t v  = VECTOR(order)[i];
+            igraph_int_t cl = VECTOR(membership)[v];
             if (VECTOR(cluster_degrees)[cl] == 0 && VECTOR(*roots)[cl] == -1) {
                 VECTOR(*roots)[cl] = v;
             }
@@ -633,7 +629,7 @@ igraph_error_t igraph_roots_for_tree_layout(
             }
             VECTOR(*roots)[j++] = VECTOR(*roots)[i];
         }
-        igraph_vector_int_resize(roots, j);
+        igraph_vector_int_resize(roots, j); /* shrinks */
 
     } else {
         /* Undirected case:
@@ -641,11 +637,11 @@ igraph_error_t igraph_roots_for_tree_layout(
          * Select the highest degree node from each component.
          */
 
-        igraph_integer_t no_seen = 0;
+        igraph_int_t no_seen = 0;
 
         for (i=0; i < no_of_nodes; ++i) {
-            igraph_integer_t v  = VECTOR(order)[i];
-            igraph_integer_t cl = VECTOR(membership)[v];
+            igraph_int_t v  = VECTOR(order)[i];
+            igraph_int_t cl = VECTOR(membership)[v];
             if (VECTOR(*roots)[cl] == -1) {
                 no_seen += 1;
                 VECTOR(*roots)[cl] = v;
@@ -693,12 +689,12 @@ igraph_error_t igraph_roots_for_tree_layout(
  *   vertices are calculated, if they are not given. See the \p roots parameter.
  * \param roots The index of the root vertex or root vertices. The set of roots
  *   should be specified so that all vertices of the graph are reachable from them.
- *   Simply put, in the udirected case, one root should be given from each
+ *   Simply put, in the undirected case, one root should be given from each
  *   connected component. If \p roots is \c NULL or a pointer to an empty vector,
  *   then the roots will be selected automatically. Currently, automatic root
  *   selection prefers low eccentricity vertices in graphs with fewer than
- *   500 vertices, and high degree vertices (acording to \p mode) in larger graphs.
- *   The root selecton heuristic may change without notice. To ensure a consistent
+ *   500 vertices, and high degree vertices (according to \p mode) in larger graphs.
+ *   The root selection heuristic may change without notice. To ensure a consistent
  *   output, please specify the roots manually. The \ref igraph_roots_for_tree_layout()
  *   function gives more control over automatic root selection.
  * \param rootlevel This argument can be useful when drawing forests which are
@@ -720,14 +716,13 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
                                    const igraph_vector_int_t *roots,
                                    const igraph_vector_int_t *rootlevel) {
 
-    igraph_integer_t no_of_nodes_orig = igraph_vcount(graph);
-    igraph_integer_t no_of_nodes = no_of_nodes_orig;
-    igraph_integer_t real_root;
+    const igraph_int_t no_of_nodes_orig = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = no_of_nodes_orig;
+    igraph_int_t real_root;
     igraph_t extended;
     const igraph_t *pextended = graph;
     igraph_vector_int_t myroots;
     const igraph_vector_int_t *proots = roots;
-    igraph_integer_t i;
     igraph_vector_int_t newedges;
 
 
@@ -765,8 +760,7 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
         /* ----------------------------------------------------------------------- */
         /* Many roots were given to us, check 'rootlevel' */
 
-        igraph_integer_t plus_levels = 0;
-        igraph_integer_t i;
+        igraph_int_t plus_levels = 0;
 
         if (igraph_vector_int_size(roots) != igraph_vector_int_size(rootlevel)) {
             IGRAPH_ERROR("Reingold-Tilford: 'roots' and 'rootlevel' lengths differ",
@@ -774,25 +768,25 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
         }
 
         /* count the rootlevels that are not zero */
-        for (i = 0; i < igraph_vector_int_size(roots); i++) {
+        for (igraph_int_t i = 0; i < igraph_vector_int_size(roots); i++) {
             plus_levels += VECTOR(*rootlevel)[i];
         }
 
         /* make copy of graph, add vertices/edges */
         if (plus_levels != 0) {
-            igraph_integer_t edgeptr = 0;
+            igraph_int_t edgeptr = 0;
 
             pextended = &extended;
             IGRAPH_CHECK(igraph_copy(&extended, graph));
             IGRAPH_FINALLY(igraph_destroy, &extended);
             IGRAPH_CHECK(igraph_add_vertices(&extended, plus_levels, 0));
 
-            igraph_vector_int_resize(&newedges, plus_levels * 2);
+            IGRAPH_CHECK(igraph_vector_int_resize(&newedges, plus_levels * 2));
 
-            for (i = 0; i < igraph_vector_int_size(roots); i++) {
-                igraph_integer_t rl = VECTOR(*rootlevel)[i];
-                igraph_integer_t rn = VECTOR(*roots)[i];
-                igraph_integer_t j;
+            for (igraph_int_t i = 0; i < igraph_vector_int_size(roots); i++) {
+                igraph_int_t rl = VECTOR(*rootlevel)[i];
+                igraph_int_t rn = VECTOR(*roots)[i];
+                igraph_int_t j;
 
                 /* zero-level roots don't get anything special */
                 if (rl == 0) {
@@ -864,7 +858,7 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
 
         /* else, we need to make real_root */
     } else {
-        igraph_integer_t no_of_newedges;
+        igraph_int_t no_of_newedges;
 
         /* Make copy of the graph unless it exists already */
         if (pextended == graph) {
@@ -880,8 +874,8 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
 
         /* add edges from the roots to real_root */
         no_of_newedges = igraph_vector_int_size(proots);
-        igraph_vector_int_resize(&newedges, no_of_newedges * 2);
-        for (i = 0; i < no_of_newedges; i++) {
+        IGRAPH_CHECK(igraph_vector_int_resize(&newedges, no_of_newedges * 2));
+        for (igraph_int_t i = 0; i < no_of_newedges; i++) {
             VECTOR(newedges)[2 * i] = no_of_nodes - 1;
             VECTOR(newedges)[2 * i + 1] = VECTOR(*proots)[i];
         }
@@ -890,7 +884,7 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
     }
 
     /* prepare edges to unreachable parts of the graph */
-    IGRAPH_CHECK(igraph_i_layout_reingold_tilford_unreachable(pextended, mode, real_root, no_of_nodes, &newedges));
+    IGRAPH_CHECK(layout_reingold_tilford_unreachable(pextended, mode, real_root, no_of_nodes, &newedges));
 
     if (igraph_vector_int_size(&newedges) != 0) {
         /* Make copy of the graph unless it exists already */
@@ -907,7 +901,7 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
 
     /* ----------------------------------------------------------------------- */
     /* Layout */
-    IGRAPH_CHECK(igraph_i_layout_reingold_tilford(pextended, res, mode, real_root));
+    IGRAPH_CHECK(layout_reingold_tilford(pextended, res, mode, real_root));
 
     /* Remove the new vertices from the layout */
     if (no_of_nodes != no_of_nodes_orig) {
@@ -915,7 +909,7 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
             IGRAPH_CHECK(igraph_matrix_remove_row(res, no_of_nodes_orig));
         } else {
             igraph_matrix_t tmp;
-            igraph_integer_t i;
+            igraph_int_t i;
             IGRAPH_MATRIX_INIT_FINALLY(&tmp, no_of_nodes_orig, 2);
             for (i = 0; i < no_of_nodes_orig; i++) {
                 MATRIX(tmp, i, 0) = MATRIX(*res, i, 0);
@@ -945,7 +939,6 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
  * \function igraph_layout_reingold_tilford_circular
  * \brief Circular Reingold-Tilford layout for trees.
  *
- * </para><para>
  * This layout is almost the same as \ref igraph_layout_reingold_tilford(), but
  * the tree is drawn in a circular way, with the root vertex in the center.
  *
@@ -960,12 +953,12 @@ igraph_error_t igraph_layout_reingold_tilford(const igraph_t *graph,
  *   vertices are calculated, if they are not given. See the \p roots parameter.
  * \param roots The index of the root vertex or root vertices. The set of roots
  *   should be specified so that all vertices of the graph are reachable from them.
- *   Simply put, in the udirected case, one root should be given from each
+ *   Simply put, in the undirected case, one root should be given from each
  *   connected component. If \p roots is \c NULL or a pointer to an empty vector,
  *   then the roots will be selected automatically. Currently, automatic root
- *   selection prefers low ecccentricity vertices in graphs with fewer than
- *   500 vertices, and high degree vertices (acording to \p mode) in larger graphs.
- *   The root selecton heuristic may change without notice. To ensure a consistent
+ *   selection prefers low eccentricity vertices in graphs with fewer than
+ *   500 vertices, and high degree vertices (according to \p mode) in larger graphs.
+ *   The root selection heuristic may change without notice. To ensure a consistent
  *   output, please specify the roots manually.
  * \param rootlevel This argument can be useful when drawing forests which are
  *   not trees (i.e. they are unconnected and have tree components). It specifies
@@ -982,8 +975,7 @@ igraph_error_t igraph_layout_reingold_tilford_circular(const igraph_t *graph,
         const igraph_vector_int_t *roots,
         const igraph_vector_int_t *rootlevel) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t i;
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_real_t ratio;
     igraph_real_t minx, maxx;
 
@@ -996,7 +988,7 @@ igraph_error_t igraph_layout_reingold_tilford_circular(const igraph_t *graph,
     ratio = 2 * M_PI * (no_of_nodes - 1.0) / no_of_nodes;
 
     minx = maxx = MATRIX(*res, 0, 0);
-    for (i = 1; i < no_of_nodes; i++) {
+    for (igraph_int_t i = 1; i < no_of_nodes; i++) {
         if (MATRIX(*res, i, 0) > maxx) {
             maxx = MATRIX(*res, i, 0);
         }
@@ -1007,7 +999,7 @@ igraph_error_t igraph_layout_reingold_tilford_circular(const igraph_t *graph,
     if (maxx > minx) {
         ratio /= (maxx - minx);
     }
-    for (i = 0; i < no_of_nodes; i++) {
+    for (igraph_int_t i = 0; i < no_of_nodes; i++) {
         igraph_real_t phi = (MATRIX(*res, i, 0) - minx) * ratio;
         igraph_real_t r = MATRIX(*res, i, 1);
         MATRIX(*res, i, 0) = r * cos(phi);
