@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2007-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -31,7 +30,7 @@
 #include "igraph_decls.h"
 #include "config.h" /* INTERNAL_ARPACK */
 
-__BEGIN_DECLS
+IGRAPH_BEGIN_C_DECLS
 
 #ifndef INTERNAL_ARPACK
     #define igraphdsaupd_   dsaupd_
@@ -228,6 +227,6 @@ int igraphdsortc_(char *which, int *apply, int* n, double *xreal,
 
 #endif
 
-__END_DECLS
+IGRAPH_END_C_DECLS
 
 #endif  /* ARPACK_INTERNAL_H */
