@@ -1,9 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph R package.
-   Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
-   334 Harvard street, Cambridge, MA 02139 USA
+   igraph library.
+   Copyright (C) 2005-2023  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,16 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "igraph_cocitation.h"
 
-#include "igraph_memory.h"
 #include "igraph_adjlist.h"
+#include "igraph_bitset.h"
 #include "igraph_interface.h"
 
 #include "core/interruption.h"
@@ -41,12 +35,12 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
  * \function igraph_cocitation
  * \brief Cocitation coupling.
  *
- * </para><para>
  * Two vertices are cocited if there is another vertex citing both of
  * them. \ref igraph_cocitation() simply counts how many times two vertices are
  * cocited.
  * The cocitation score for each given vertex and all other vertices
  * in the graph will be calculated.
+ *
  * \param graph The graph object to analyze.
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows is the same as the
@@ -69,7 +63,7 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
 
 igraph_error_t igraph_cocitation(const igraph_t *graph, igraph_matrix_t *res,
                       const igraph_vs_t vids) {
-    return igraph_i_cocitation_real(graph, res, vids, IGRAPH_OUT, 0);
+    return igraph_i_cocitation_real(graph, res, vids, IGRAPH_OUT, NULL);
 }
 
 /**
@@ -77,12 +71,12 @@ igraph_error_t igraph_cocitation(const igraph_t *graph, igraph_matrix_t *res,
  * \function igraph_bibcoupling
  * \brief Bibliographic coupling.
  *
- * </para><para>
  * The bibliographic coupling of two vertices is the number
  * of other vertices they both cite, \ref igraph_bibcoupling() calculates
  * this.
  * The bibliographic coupling  score for each given vertex and all
  * other vertices in the graph will be calculated.
+ *
  * \param graph The graph object to analyze.
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows is the same as the
@@ -105,7 +99,7 @@ igraph_error_t igraph_cocitation(const igraph_t *graph, igraph_matrix_t *res,
 
 igraph_error_t igraph_bibcoupling(const igraph_t *graph, igraph_matrix_t *res,
                        const igraph_vs_t vids) {
-    return igraph_i_cocitation_real(graph, res, vids, IGRAPH_IN, 0);
+    return igraph_i_cocitation_real(graph, res, vids, IGRAPH_IN, NULL);
 }
 
 /**
@@ -113,7 +107,6 @@ igraph_error_t igraph_bibcoupling(const igraph_t *graph, igraph_matrix_t *res,
  * \function igraph_similarity_inverse_log_weighted
  * \brief Vertex similarity based on the inverse logarithm of vertex degrees.
  *
- * </para><para>
  * The inverse log-weighted similarity of two vertices is the number of
  * their common neighbors, weighted by the inverse logarithm of their degrees.
  * It is based on the assumption that two vertices should be considered
@@ -125,8 +118,20 @@ igraph_error_t igraph_bibcoupling(const igraph_t *graph, igraph_matrix_t *res,
  * Self-similarities are not calculated.
  *
  * </para><para>
+ * Note that the presence of loop edges may yield counter-intuitive
+ * results. A node with a loop edge is considered to be a neighbor of itself
+ * \em twice (because there are two edge stems incident on the node). Adding a
+ * loop edge to a node may decrease its similarity to other nodes, but it may
+ * also \em increase it. For instance, if nodes A and B are connected but share
+ * no common neighbors, their similarity is zero. However, if a loop edge is
+ * added to B, then B itself becomes a common neighbor of A and B and thus the
+ * similarity of A and B will be increased. Consider removing loop edges
+ * explicitly before invoking this function using \ref igraph_simplify().
+ *
+ * </para><para>
  * See the following paper for more details: Lada A. Adamic and Eytan Adar:
  * Friends and neighbors on the Web. Social Networks, 25(3):211-230, 2003.
+ * https://doi.org/10.1016/S0378-8733(03)00009-1
  *
  * \param graph The graph object to analyze.
  * \param res Pointer to a matrix, the result of the calculation will
@@ -164,21 +169,17 @@ igraph_error_t igraph_similarity_inverse_log_weighted(const igraph_t *graph,
         igraph_matrix_t *res, const igraph_vs_t vids, igraph_neimode_t mode) {
     igraph_vector_t weights;
     igraph_vector_int_t degrees;
-    igraph_neimode_t mode0;
-    igraph_integer_t i, no_of_nodes;
+    igraph_neimode_t mode0 = IGRAPH_REVERSE_MODE(mode);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
 
-    switch (mode) {
-    case IGRAPH_OUT: mode0 = IGRAPH_IN; break;
-    case IGRAPH_IN: mode0 = IGRAPH_OUT; break;
-    default: mode0 = IGRAPH_ALL;
+    if (mode != IGRAPH_OUT && mode != IGRAPH_IN && mode != IGRAPH_ALL) {
+        IGRAPH_ERROR("Invalid mode for inverse log weighted similarity.", IGRAPH_EINVMODE);
     }
 
-    no_of_nodes = igraph_vcount(graph);
-
     IGRAPH_VECTOR_INIT_FINALLY(&weights, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&degrees, no_of_nodes);
-    IGRAPH_CHECK(igraph_degree(graph, &degrees, igraph_vss_all(), mode0, 1));
-    for (i = 0; i < no_of_nodes; i++) {
+    IGRAPH_CHECK(igraph_degree(graph, &degrees, igraph_vss_all(), mode0, IGRAPH_LOOPS));
+    for (igraph_int_t i = 0; i < no_of_nodes; i++) {
         VECTOR(weights)[i] = VECTOR(degrees)[i];
         if (VECTOR(weights)[i] > 1) {
             VECTOR(weights)[i] = 1.0 / log(VECTOR(weights)[i]);
@@ -189,6 +190,7 @@ igraph_error_t igraph_similarity_inverse_log_weighted(const igraph_t *graph,
     igraph_vector_int_destroy(&degrees);
     igraph_vector_destroy(&weights);
     IGRAPH_FINALLY_CLEAN(2);
+
     return IGRAPH_SUCCESS;
 }
 
@@ -197,10 +199,10 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
                            igraph_neimode_t mode,
                            igraph_vector_t *weights) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_vids;
-    igraph_integer_t from, i, j, k, l, u, v;
-    igraph_vector_int_t neis = IGRAPH_VECTOR_NULL;
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_vids;
+    igraph_int_t i;
+    igraph_vector_int_t neis;
     igraph_vector_int_t vid_reverse_index;
     igraph_vit_t vit;
 
@@ -214,9 +216,9 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
     IGRAPH_VECTOR_INT_INIT_FINALLY(&vid_reverse_index, no_of_nodes);
     igraph_vector_int_fill(&vid_reverse_index, -1);
     for (IGRAPH_VIT_RESET(vit), i = 0; !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit), i++) {
-        v = IGRAPH_VIT_GET(vit);
+        igraph_int_t v = IGRAPH_VIT_GET(vit);
         if (v < 0 || v >= no_of_nodes) {
-            IGRAPH_ERROR("invalid vertex ID in vertex selector", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Invalid vertex ID in vertex selector.", IGRAPH_EINVVID);
         }
         VECTOR(vid_reverse_index)[v] = i;
     }
@@ -227,22 +229,20 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
 
     /* The result */
 
-    for (from = 0; from < no_of_nodes; from++) {
-        igraph_real_t weight = 1;
-
+    for (igraph_int_t from = 0; from < no_of_nodes; from++) {
         IGRAPH_ALLOW_INTERRUPTION();
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, from, mode));
-        igraph_integer_t nei_count = igraph_vector_int_size(&neis);
-        if (weights) {
-            weight = VECTOR(*weights)[from];
-        }
+
+        const igraph_real_t weight = weights ? VECTOR(*weights)[from] : 1;
+
+        IGRAPH_CHECK(igraph_neighbors(graph, &neis, from, mode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
+        const igraph_int_t nei_count = igraph_vector_int_size(&neis);
 
         for (i = 0; i < nei_count - 1; i++) {
-            u = VECTOR(neis)[i];
-            k = VECTOR(vid_reverse_index)[u];
-            for (j = i + 1; j < nei_count; j++) {
-                v = VECTOR(neis)[j];
-                l = VECTOR(vid_reverse_index)[v];
+            igraph_int_t u = VECTOR(neis)[i];
+            igraph_int_t k = VECTOR(vid_reverse_index)[u];
+            for (igraph_int_t j = i + 1; j < nei_count; j++) {
+                igraph_int_t v = VECTOR(neis)[j];
+                igraph_int_t l = VECTOR(vid_reverse_index)[v];
                 if (k != -1) {
                     MATRIX(*res, k, v) += weight;
                 }
@@ -265,23 +265,12 @@ static igraph_error_t igraph_i_cocitation_real(const igraph_t *graph, igraph_mat
 
 static igraph_error_t igraph_i_neisets_intersect(
     const igraph_vector_int_t *v1, const igraph_vector_int_t *v2,
-    igraph_integer_t *len_union, igraph_integer_t *len_intersection
+    igraph_int_t *len_union, igraph_int_t *len_intersection
 ) {
     /* ASSERT: v1 and v2 are sorted */
-    igraph_integer_t i, j, i0, jj0;
-    i0 = igraph_vector_int_size(v1); jj0 = igraph_vector_int_size(v2);
-    *len_union = i0 + jj0; *len_intersection = 0;
-    i = 0; j = 0;
-    while (i < i0 && j < jj0) {
-        if (VECTOR(*v1)[i] == VECTOR(*v2)[j]) {
-            (*len_intersection)++; (*len_union)--;
-            i++; j++;
-        } else if (VECTOR(*v1)[i] < VECTOR(*v2)[j]) {
-            i++;
-        } else {
-            j++;
-        }
-    }
+    igraph_int_t n1 = igraph_vector_int_size(v1), n2 = igraph_vector_int_size(v2);
+    *len_intersection = igraph_vector_int_intersection_size_sorted(v1, v2);
+    *len_union = n1 + n2 - *len_intersection;
     return IGRAPH_SUCCESS;
 }
 
@@ -290,7 +279,6 @@ static igraph_error_t igraph_i_neisets_intersect(
  * \function igraph_similarity_jaccard
  * \brief Jaccard similarity coefficient for the given vertices.
  *
- * </para><para>
  * The Jaccard similarity coefficient of two vertices is the number of common
  * neighbors divided by the number of vertices that are neighbors of at
  * least one of the two vertices being considered. This function calculates
@@ -299,9 +287,11 @@ static igraph_error_t igraph_i_neisets_intersect(
  * \param graph The graph object to analyze
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows and columns is the same
- *        as the number of vertex IDs in \p vids.
- * \param vids The vertex IDs of the vertices for which the
- *        calculation will be done.
+ *        as the number of vertex IDs in \p vit_from and \p vit_to, respectively.
+ * \param from The vertex IDs of the first set of vertices of the
+ *        pairs for which the calculation will be done.
+ * \param to The vertex IDs of the second set of vertices of the
+ *        pairs for which the calculation will be done.
  * \param mode The type of neighbors to be used for the calculation in
  *        directed graphs. Possible values:
  *        \clist
@@ -335,27 +325,27 @@ static igraph_error_t igraph_i_neisets_intersect(
  * \example examples/simple/igraph_similarity.c
  */
 igraph_error_t igraph_similarity_jaccard(const igraph_t *graph, igraph_matrix_t *res,
-                              const igraph_vs_t vids, igraph_neimode_t mode, igraph_bool_t loops) {
+                              const igraph_vs_t from, const igraph_vs_t to, igraph_neimode_t mode, igraph_bool_t loops) {
     igraph_lazy_adjlist_t al;
-    igraph_vit_t vit, vit2;
-    igraph_integer_t i, j;
-    igraph_integer_t len_union, len_intersection;
+    igraph_vit_t vit_from, vit_to;
+    igraph_int_t i, j;
+    igraph_int_t len_union, len_intersection;
     igraph_vector_int_t *v1, *v2;
-    igraph_integer_t k;
+    igraph_int_t k;
 
-    IGRAPH_CHECK(igraph_vit_create(graph, vids, &vit));
-    IGRAPH_FINALLY(igraph_vit_destroy, &vit);
-    IGRAPH_CHECK(igraph_vit_create(graph, vids, &vit2));
-    IGRAPH_FINALLY(igraph_vit_destroy, &vit2);
+    IGRAPH_CHECK(igraph_vit_create(graph, from, &vit_from));
+    IGRAPH_FINALLY(igraph_vit_destroy, &vit_from);
+    IGRAPH_CHECK(igraph_vit_create(graph, to, &vit_to));
+    IGRAPH_FINALLY(igraph_vit_destroy, &vit_to);
 
     IGRAPH_CHECK(igraph_lazy_adjlist_init(graph, &al, mode, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
     IGRAPH_FINALLY(igraph_lazy_adjlist_destroy, &al);
 
-    IGRAPH_CHECK(igraph_matrix_resize(res, IGRAPH_VIT_SIZE(vit), IGRAPH_VIT_SIZE(vit)));
+    IGRAPH_CHECK(igraph_matrix_resize(res, IGRAPH_VIT_SIZE(vit_from), IGRAPH_VIT_SIZE(vit_to)));
 
     if (loops) {
-        for (IGRAPH_VIT_RESET(vit); !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit)) {
-            i = IGRAPH_VIT_GET(vit);
+        for (IGRAPH_VIT_RESET(vit_from); !IGRAPH_VIT_END(vit_from); IGRAPH_VIT_NEXT(vit_from)) {
+            i = IGRAPH_VIT_GET(vit_from);
             v1 = igraph_lazy_adjlist_get(&al, i);
             IGRAPH_CHECK_OOM(v1, "Failed to query neighbors.");
             if (!igraph_vector_int_binsearch(v1, i, &k)) {
@@ -364,18 +354,20 @@ igraph_error_t igraph_similarity_jaccard(const igraph_t *graph, igraph_matrix_t
         }
     }
 
-    for (IGRAPH_VIT_RESET(vit), i = 0;
-         !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit), i++) {
+    for (IGRAPH_VIT_RESET(vit_from), i = 0;
+         !IGRAPH_VIT_END(vit_from); IGRAPH_VIT_NEXT(vit_from), i++) {
         MATRIX(*res, i, i) = 1.0;
-        for (IGRAPH_VIT_RESET(vit2), j = 0;
-             !IGRAPH_VIT_END(vit2); IGRAPH_VIT_NEXT(vit2), j++) {
+        for (IGRAPH_VIT_RESET(vit_to), j = 0;
+             !IGRAPH_VIT_END(vit_to); IGRAPH_VIT_NEXT(vit_to), j++) {
             if (j <= i) {
                 continue;
             }
-            v1 = igraph_lazy_adjlist_get(&al, IGRAPH_VIT_GET(vit));
-            v2 = igraph_lazy_adjlist_get(&al, IGRAPH_VIT_GET(vit2));
+
+            v1 = igraph_lazy_adjlist_get(&al, IGRAPH_VIT_GET(vit_from));
             IGRAPH_CHECK_OOM(v1, "Failed to query neighbors.");
+            v2 = igraph_lazy_adjlist_get(&al, IGRAPH_VIT_GET(vit_to));
             IGRAPH_CHECK_OOM(v2, "Failed to query neighbors.");
+
             IGRAPH_CHECK(igraph_i_neisets_intersect(v1, v2, &len_union, &len_intersection));
             if (len_union > 0) {
                 MATRIX(*res, i, j) = ((igraph_real_t)len_intersection) / len_union;
@@ -387,8 +379,8 @@ igraph_error_t igraph_similarity_jaccard(const igraph_t *graph, igraph_matrix_t
     }
 
     igraph_lazy_adjlist_destroy(&al);
-    igraph_vit_destroy(&vit);
-    igraph_vit_destroy(&vit2);
+    igraph_vit_destroy(&vit_from);
+    igraph_vit_destroy(&vit_to);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -399,7 +391,6 @@ igraph_error_t igraph_similarity_jaccard(const igraph_t *graph, igraph_matrix_t
  * \function igraph_similarity_jaccard_pairs
  * \brief Jaccard similarity coefficient for given vertex pairs.
  *
- * </para><para>
  * The Jaccard similarity coefficient of two vertices is the number of common
  * neighbors divided by the number of vertices that are neighbors of at
  * least one of the two vertices being considered. This function calculates
@@ -448,15 +439,18 @@ igraph_error_t igraph_similarity_jaccard(const igraph_t *graph, igraph_matrix_t
  */
 igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vector_t *res,
                                     const igraph_vector_int_t *pairs, igraph_neimode_t mode, igraph_bool_t loops) {
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_lazy_adjlist_t al;
-    igraph_integer_t i, j, k, u, v;
-    igraph_integer_t len_union, len_intersection;
+    igraph_int_t u, v;
+    igraph_int_t len_union, len_intersection;
     igraph_vector_int_t *v1, *v2;
-    igraph_bool_t *seen;
 
-    k = igraph_vector_int_size(pairs);
+    igraph_int_t k = igraph_vector_int_size(pairs);
     if (k % 2 != 0) {
-        IGRAPH_ERROR("number of elements in `pairs' must be even", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Number of elements in `pairs' must be even.", IGRAPH_EINVAL);
+    }
+    if (!igraph_vector_int_isininterval(pairs, 0, no_of_nodes - 1)) {
+        IGRAPH_ERROR("Invalid vertex ID in pairs.", IGRAPH_EINVVID);
     }
     IGRAPH_CHECK(igraph_vector_resize(res, k / 2));
 
@@ -465,19 +459,16 @@ igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vec
 
     if (loops) {
         /* Add the loop edges */
-        i = igraph_vcount(graph);
-        seen = IGRAPH_CALLOC(i, igraph_bool_t);
-        if (seen == 0) {
-            IGRAPH_ERROR("cannot calculate Jaccard similarity", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-        }
-        IGRAPH_FINALLY(igraph_free, seen);
 
-        for (i = 0; i < k; i++) {
-            j = VECTOR(*pairs)[i];
-            if (seen[j]) {
+        igraph_bitset_t seen;
+        IGRAPH_BITSET_INIT_FINALLY(&seen, no_of_nodes);
+
+        for (igraph_int_t i = 0; i < k; i++) {
+            igraph_int_t j = VECTOR(*pairs)[i];
+            if (IGRAPH_BIT_TEST(seen, j)) {
                 continue;
             }
-            seen[j] = 1;
+            IGRAPH_BIT_SET(seen, j);
             v1 = igraph_lazy_adjlist_get(&al, j);
             IGRAPH_CHECK_OOM(v1, "Failed to query neighbors.");
             if (!igraph_vector_int_binsearch(v1, j, &u)) {
@@ -485,11 +476,11 @@ igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vec
             }
         }
 
-        IGRAPH_FREE(seen);
+        igraph_bitset_destroy(&seen);
         IGRAPH_FINALLY_CLEAN(1);
     }
 
-    for (i = 0, j = 0; i < k; i += 2, j++) {
+    for (igraph_int_t i = 0, j = 0; i < k; i += 2, j++) {
         u = VECTOR(*pairs)[i];
         v = VECTOR(*pairs)[i + 1];
 
@@ -499,9 +490,10 @@ igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vec
         }
 
         v1 = igraph_lazy_adjlist_get(&al, u);
-        v2 = igraph_lazy_adjlist_get(&al, v);
         IGRAPH_CHECK_OOM(v1, "Failed to query neighbors.");
+        v2 = igraph_lazy_adjlist_get(&al, v);
         IGRAPH_CHECK_OOM(v2, "Failed to query neighbors.");
+
         IGRAPH_CHECK(igraph_i_neisets_intersect(v1, v2, &len_union, &len_intersection));
         if (len_union > 0) {
             VECTOR(*res)[j] = ((igraph_real_t)len_intersection) / len_union;
@@ -521,7 +513,6 @@ igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vec
  * \function igraph_similarity_jaccard_es
  * \brief Jaccard similarity coefficient for a given edge selector.
  *
- * </para><para>
  * The Jaccard similarity coefficient of two vertices is the number of common
  * neighbors divided by the number of vertices that are neighbors of at
  * least one of the two vertices being considered. This function calculates
@@ -570,26 +561,13 @@ igraph_error_t igraph_similarity_jaccard_pairs(const igraph_t *graph, igraph_vec
  */
 igraph_error_t igraph_similarity_jaccard_es(const igraph_t *graph, igraph_vector_t *res,
                                  const igraph_es_t es, igraph_neimode_t mode, igraph_bool_t loops) {
-    igraph_vector_int_t v;
-    igraph_eit_t eit;
 
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&v, 0);
+    igraph_vector_int_t pairs;
 
-    IGRAPH_CHECK(igraph_eit_create(graph, es, &eit));
-    IGRAPH_FINALLY(igraph_eit_destroy, &eit);
-
-    while (!IGRAPH_EIT_END(eit)) {
-        igraph_integer_t eid = IGRAPH_EIT_GET(eit);
-        igraph_vector_int_push_back(&v, IGRAPH_FROM(graph, eid));
-        igraph_vector_int_push_back(&v, IGRAPH_TO(graph, eid));
-        IGRAPH_EIT_NEXT(eit);
-    }
-
-    igraph_eit_destroy(&eit);
-    IGRAPH_FINALLY_CLEAN(1);
-
-    IGRAPH_CHECK(igraph_similarity_jaccard_pairs(graph, res, &v, mode, loops));
-    igraph_vector_int_destroy(&v);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&pairs, 0);
+    IGRAPH_CHECK(igraph_edges(graph, es, &pairs, 0));
+    IGRAPH_CHECK(igraph_similarity_jaccard_pairs(graph, res, &pairs, mode, loops));
+    igraph_vector_int_destroy(&pairs);
     IGRAPH_FINALLY_CLEAN(1);
 
     return IGRAPH_SUCCESS;
@@ -607,9 +585,11 @@ igraph_error_t igraph_similarity_jaccard_es(const igraph_t *graph, igraph_vector
  * \param graph The graph object to analyze.
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows and columns is the same
- *        as the number of vertex IDs in \p vids.
- * \param vids The vertex IDs of the vertices for which the
- *        calculation will be done.
+ *        as the number of vertex IDs in \p vit_from and \p vit_to, respectively.
+ * \param vit_from The vertex IDs of the first vertices of the
+ *        pairs for which the calculation will be done.
+ * \param vit_to The vertex IDs of the second vertices of the
+ *        pairs for which the calculation will be done.
  * \param mode The type of neighbors to be used for the calculation in
  *        directed graphs. Possible values:
  *        \clist
@@ -643,16 +623,15 @@ igraph_error_t igraph_similarity_jaccard_es(const igraph_t *graph, igraph_vector
  * \example examples/simple/igraph_similarity.c
  */
 igraph_error_t igraph_similarity_dice(const igraph_t *graph, igraph_matrix_t *res,
-                                      const igraph_vs_t vids,
+                                      const igraph_vs_t vit_from, const igraph_vs_t vit_to,
                                       igraph_neimode_t mode, igraph_bool_t loops) {
-    igraph_integer_t i, j, nr, nc;
 
-    IGRAPH_CHECK(igraph_similarity_jaccard(graph, res, vids, mode, loops));
+    IGRAPH_CHECK(igraph_similarity_jaccard(graph, res, vit_from, vit_to, mode, loops));
 
-    nr = igraph_matrix_nrow(res);
-    nc = igraph_matrix_ncol(res);
-    for (i = 0; i < nr; i++) {
-        for (j = 0; j < nc; j++) {
+    igraph_int_t nr = igraph_matrix_nrow(res);
+    igraph_int_t nc = igraph_matrix_ncol(res);
+    for (igraph_int_t i = 0; i < nr; i++) {
+        for (igraph_int_t j = 0; j < nc; j++) {
             igraph_real_t x = MATRIX(*res, i, j);
             MATRIX(*res, i, j) = 2 * x / (1 + x);
         }
@@ -666,7 +645,6 @@ igraph_error_t igraph_similarity_dice(const igraph_t *graph, igraph_matrix_t *re
  * \function igraph_similarity_dice_pairs
  * \brief Dice similarity coefficient for given vertex pairs.
  *
- * </para><para>
  * The Dice similarity coefficient of two vertices is twice the number of common
  * neighbors divided by the sum of the degrees of the vertices. This function
  * calculates the pairwise Dice similarities for a list of vertex pairs.
@@ -714,11 +692,10 @@ igraph_error_t igraph_similarity_dice(const igraph_t *graph, igraph_matrix_t *re
  */
 igraph_error_t igraph_similarity_dice_pairs(const igraph_t *graph, igraph_vector_t *res,
                                  const igraph_vector_int_t *pairs, igraph_neimode_t mode, igraph_bool_t loops) {
-    igraph_integer_t i, n;
 
     IGRAPH_CHECK(igraph_similarity_jaccard_pairs(graph, res, pairs, mode, loops));
-    n = igraph_vector_size(res);
-    for (i = 0; i < n; i++) {
+    igraph_int_t n = igraph_vector_size(res);
+    for (igraph_int_t i = 0; i < n; i++) {
         igraph_real_t x = VECTOR(*res)[i];
         VECTOR(*res)[i] = 2 * x / (1 + x);
     }
@@ -731,7 +708,6 @@ igraph_error_t igraph_similarity_dice_pairs(const igraph_t *graph, igraph_vector
  * \function igraph_similarity_dice_es
  * \brief Dice similarity coefficient for a given edge selector.
  *
- * </para><para>
  * The Dice similarity coefficient of two vertices is twice the number of common
  * neighbors divided by the sum of the degrees of the vertices. This function
  * calculates the pairwise Dice similarities for the endpoints of edges in a given
@@ -779,11 +755,10 @@ igraph_error_t igraph_similarity_dice_pairs(const igraph_t *graph, igraph_vector
  */
 igraph_error_t igraph_similarity_dice_es(const igraph_t *graph, igraph_vector_t *res,
                               const igraph_es_t es, igraph_neimode_t mode, igraph_bool_t loops) {
-    igraph_integer_t i, n;
 
     IGRAPH_CHECK(igraph_similarity_jaccard_es(graph, res, es, mode, loops));
-    n = igraph_vector_size(res);
-    for (i = 0; i < n; i++) {
+    igraph_int_t n = igraph_vector_size(res);
+    for (igraph_int_t i = 0; i < n; i++) {
         igraph_real_t x = VECTOR(*res)[i];
         VECTOR(*res)[i] = 2 * x / (1 + x);
     }
