@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -40,7 +39,9 @@
  */
 static igraph_error_t igraph_i_motifs_randesu_update_hist(
         const igraph_t *graph,
-        igraph_vector_int_t *vids, igraph_integer_t isoclass, void* extra) {
+        const igraph_vector_int_t *vids,
+        igraph_int_t isoclass, void* extra) {
+
     igraph_vector_t *hist = (igraph_vector_t*)extra;
     IGRAPH_UNUSED(graph); IGRAPH_UNUSED(vids);
     VECTOR(*hist)[isoclass]++;
@@ -115,9 +116,9 @@ static igraph_error_t igraph_i_motifs_randesu_update_hist(
  * \example examples/simple/igraph_motifs_randesu.c
  */
 igraph_error_t igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
-                          igraph_integer_t size, const igraph_vector_t *cut_prob) {
+                          igraph_int_t size, const igraph_vector_t *cut_prob) {
     igraph_bool_t directed = igraph_is_directed(graph);
-    igraph_integer_t histlen;
+    igraph_int_t histlen;
 
     if (directed) {
         switch (size) {
@@ -237,22 +238,23 @@ igraph_error_t igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *his
  * \example examples/simple/igraph_motifs_randesu.c
  */
 
-igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_integer_t size,
-                                   const igraph_vector_t *cut_prob, igraph_motifs_handler_t *callback,
-                                   void* extra) {
+igraph_error_t igraph_motifs_randesu_callback(
+        const igraph_t *graph,
+        igraph_int_t size, const igraph_vector_t *cut_prob,
+        igraph_motifs_handler_t *callback, void* extra) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_adjlist_t allneis, alloutneis;
     igraph_vector_int_t *neis;
-    igraph_integer_t father;
-    igraph_integer_t i, j, s;
-    igraph_integer_t motifs = 0;
+    igraph_int_t father;
+    igraph_int_t i, j, s;
+    igraph_int_t motifs = 0;
     IGRAPH_UNUSED(motifs);    /* We mark it as unused to prevent warnings about unused-but-set-variables. */
 
     igraph_vector_int_t vids;     /* this is G */
     igraph_vector_int_t adjverts; /* this is V_E */
     igraph_stack_int_t stack;     /* this is S */
-    igraph_integer_t *added;
+    igraph_int_t *added;
     char *subg;
 
     const unsigned int *arr_idx, *arr_code;
@@ -310,7 +312,7 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
                       IGRAPH_EINVAL, igraph_vector_size(cut_prob), size);
     }
 
-    added = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
+    added = IGRAPH_CALLOC(no_of_nodes, igraph_int_t);
     IGRAPH_CHECK_OOM(added, "Insufficient memory to find motifs.");
     IGRAPH_FINALLY(igraph_free, added);
 
@@ -328,10 +330,8 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
     IGRAPH_CHECK(igraph_stack_int_init(&stack, 0));
     IGRAPH_FINALLY(igraph_stack_int_destroy, &stack);
 
-    RNG_BEGIN();
-
     for (father = 0; father < no_of_nodes; father++) {
-        igraph_integer_t level;
+        igraph_int_t level;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -351,7 +351,7 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
         neis = igraph_adjlist_get(&allneis, father);
         s = igraph_vector_int_size(neis);
         for (i = 0; i < s; i++) {
-            igraph_integer_t nei = VECTOR(*neis)[i];
+            igraph_int_t nei = VECTOR(*neis)[i];
             if (!added[nei] && nei > father) {
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, father));
@@ -368,8 +368,8 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
             if (level == size - 1) {
                 s = igraph_vector_int_size(&adjverts) / 2;
                 for (i = 0; i < s; i++) {
-                    igraph_integer_t k, s2;
-                    igraph_integer_t last;
+                    igraph_int_t k, s2;
+                    igraph_int_t last;
                     igraph_error_t ret;
 
                     if (cp != 0 && RNG_UNIF01() < cp) {
@@ -383,11 +383,11 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
 
                     code = 0; idx = 0;
                     for (k = 0; k < size; k++) {
-                        igraph_integer_t from = VECTOR(vids)[k];
+                        igraph_int_t from = VECTOR(vids)[k];
                         neis = igraph_adjlist_get(&alloutneis, from);
                         s2 = igraph_vector_int_size(neis);
                         for (j = 0; j < s2; j++) {
-                            igraph_integer_t nei = VECTOR(*neis)[j];
+                            igraph_int_t nei = VECTOR(*neis)[j];
                             if (subg[nei] && k != subg[nei] - 1) {
                                 idx = (unsigned char) (mul * k + (subg[nei] - 1));
                                 code |= arr_idx[idx];
@@ -419,8 +419,8 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
             if (level < size - 1 &&
                 !igraph_vector_int_empty(&adjverts)) {
                 /* we might step down */
-                igraph_integer_t neifather = igraph_vector_int_pop_back(&adjverts);
-                igraph_integer_t nei = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t neifather = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t nei = igraph_vector_int_pop_back(&adjverts);
 
                 if (cp == 0 || RNG_UNIF01() > cp) {
                     /* yes, step down */
@@ -434,7 +434,7 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
                     neis = igraph_adjlist_get(&allneis, nei);
                     s = igraph_vector_int_size(neis);
                     for (i = 0; i < s; i++) {
-                        igraph_integer_t nei2 = VECTOR(*neis)[i];
+                        igraph_int_t nei2 = VECTOR(*neis)[i];
                         if (!added[nei2] && nei2 > father) {
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei2));
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
@@ -444,7 +444,7 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
                 }
             } else {
                 /* no, step back */
-                igraph_integer_t nei, neifather;
+                igraph_int_t nei, neifather;
                 while (!igraph_stack_int_empty(&stack) &&
                        level == igraph_stack_int_top(&stack) - 1) {
                     igraph_stack_int_pop(&stack);
@@ -486,8 +486,6 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
 
     } /* for father */
 
-    RNG_END();
-
     IGRAPH_FREE(added);
     IGRAPH_FREE(subg);
     igraph_vector_int_destroy(&vids);
@@ -522,7 +520,10 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
  * the search tree.
  *
  * \param graph The graph object to study.
- * \param est Pointer to an integer, the result will be stored here.
+ * \param est Pointer to an \c igraph_real_t, the result will be stored here.
+ *        Note that even though the result is an integer, we need to use
+ *        \c igraph_real_t to avoid overflow when igraph is compiled with
+ *        32-bit integers.
  * \param size The size of the subgraphs to look for.
  * \param cut_prob Vector of probabilities for cutting the search tree
  *        at a given level. The first element is the first level, etc.
@@ -544,21 +545,21 @@ igraph_error_t igraph_motifs_randesu_callback(const igraph_t *graph, igraph_inte
  * Time complexity: TODO.
  */
 
-igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_integer_t *est,
-                                   igraph_integer_t size, const igraph_vector_t *cut_prob,
-                                   igraph_integer_t sample_size,
+igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_real_t *est,
+                                   igraph_int_t size, const igraph_vector_t *cut_prob,
+                                   igraph_int_t sample_size,
                                    const igraph_vector_int_t *parsample) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t neis;
 
     igraph_vector_int_t vids;     /* this is G */
     igraph_vector_int_t adjverts; /* this is V_E */
     igraph_stack_int_t stack;     /* this is S */
-    igraph_integer_t *added;
+    igraph_int_t *added;
     igraph_vector_int_t *sample;
-    igraph_integer_t sam;
-    igraph_integer_t i;
+    igraph_int_t sam;
+    igraph_int_t i;
 
     if (size < 3) {
         IGRAPH_ERRORF("Motif size must be at least 3, received %" IGRAPH_PRId ".",
@@ -579,7 +580,7 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
         return IGRAPH_SUCCESS;
     }
 
-    added = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
+    added = IGRAPH_CALLOC(no_of_nodes, igraph_int_t);
     IGRAPH_CHECK_OOM(added, "Insufficient memory to count motifs.");
     IGRAPH_FINALLY(igraph_free, added);
 
@@ -608,11 +609,9 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
 
     *est = 0;
 
-    RNG_BEGIN();
-
     for (sam = 0; sam < sample_size; sam++) {
-        igraph_integer_t father = VECTOR(*sample)[sam];
-        igraph_integer_t level, s;
+        igraph_int_t father = VECTOR(*sample)[sam];
+        igraph_int_t level, s;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -629,10 +628,12 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
 
         /* init V_E */
         igraph_vector_int_clear(&adjverts);
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, father, IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &neis, father, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+        ));
         s = igraph_vector_int_size(&neis);
         for (i = 0; i < s; i++) {
-            igraph_integer_t nei = VECTOR(neis)[i];
+            igraph_int_t nei = VECTOR(neis)[i];
             if (!added[nei] && nei > father) {
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, father));
@@ -659,8 +660,8 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
             if (level < size - 1 &&
                 !igraph_vector_int_empty(&adjverts)) {
                 /* We might step down */
-                igraph_integer_t neifather = igraph_vector_int_pop_back(&adjverts);
-                igraph_integer_t nei = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t neifather = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t nei = igraph_vector_int_pop_back(&adjverts);
 
                 if (cp == 0 || RNG_UNIF01() > cp) {
                     /* Yes, step down */
@@ -671,10 +672,12 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
                     IGRAPH_CHECK(igraph_stack_int_push(&stack, nei));
                     IGRAPH_CHECK(igraph_stack_int_push(&stack, level));
 
-                    IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+                    IGRAPH_CHECK(igraph_neighbors(
+                        graph, &neis, nei, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+                    ));
                     s = igraph_vector_int_size(&neis);
                     for (i = 0; i < s; i++) {
-                        igraph_integer_t nei2 = VECTOR(neis)[i];
+                        igraph_int_t nei2 = VECTOR(neis)[i];
                         if (!added[nei2] && nei2 > father) {
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei2));
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
@@ -684,7 +687,7 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
                 }
             } else {
                 /* no, step back */
-                igraph_integer_t nei, neifather;
+                igraph_int_t nei, neifather;
                 while (!igraph_stack_int_empty(&stack) &&
                        level == igraph_stack_int_top(&stack) - 1) {
                     igraph_stack_int_pop(&stack);
@@ -696,7 +699,9 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
 
                 nei = igraph_vector_int_pop_back(&vids);
                 added[nei] -= 1; level -= 1;
-                IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+                IGRAPH_CHECK(igraph_neighbors(
+                    graph, &neis, nei, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+                ));
                 s = igraph_vector_int_size(&neis);
                 for (i = 0; i < s; i++) {
                     added[ VECTOR(neis)[i] ] -= 1;
@@ -712,7 +717,9 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
 
         /* clear the added vector */
         added[father] -= 1;
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, father, IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &neis, father, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+        ));
         s = igraph_vector_int_size(&neis);
         for (i = 0; i < s; i++) {
             added[ VECTOR(neis)[i] ] -= 1;
@@ -720,8 +727,6 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
 
     } /* for father */
 
-    RNG_END();
-
     (*est) *= ((igraph_real_t) no_of_nodes / sample_size);
 
     if (parsample == 0) {
@@ -748,8 +753,10 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
  * classes to them. Arbitrarily large motif sizes are supported.
  *
  * \param graph The graph object to study.
- * \param no Pointer to an integer type, the result will be stored
- *        here.
+ * \param no Pointer to an \c igraph_real_t, the result will be stored here.
+ *        Note that even though the result is an integer, we need to use
+ *        \c igraph_real_t to avoid overflow when igraph is compiled with
+ *        32-bit integers.
  * \param size The size of the motifs to count.
  * \param cut_prob Vector of probabilities for cutting the search tree
  *        at a given level. The first element is the first level, etc.
@@ -763,17 +770,19 @@ igraph_error_t igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_inte
  * Time complexity: TODO.
  */
 
-igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t *no,
-                             igraph_integer_t size, const igraph_vector_t *cut_prob) {
+igraph_error_t igraph_motifs_randesu_no(
+    const igraph_t *graph, igraph_real_t *no, igraph_int_t size,
+    const igraph_vector_t *cut_prob
+) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t neis;
     igraph_vector_int_t vids;     /* this is G */
     igraph_vector_int_t adjverts; /* this is V_E */
     igraph_stack_int_t stack;     /* this is S */
-    igraph_integer_t *added;
-    igraph_integer_t father;
-    igraph_integer_t i;
+    igraph_int_t *added;
+    igraph_int_t father;
+    igraph_int_t i;
 
     if (size < 3) {
         IGRAPH_ERRORF("Motif size must be at least 3, received %" IGRAPH_PRId ".",
@@ -784,7 +793,7 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
         IGRAPH_ERRORF("Cut probability vector size (%" IGRAPH_PRId ") must agree with motif size (%" IGRAPH_PRId ").",
                       IGRAPH_EINVAL, igraph_vector_size(cut_prob), size);
     }
-    added = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
+    added = IGRAPH_CALLOC(no_of_nodes, igraph_int_t);
     IGRAPH_CHECK_OOM(added, "Insufficient memory to count motifs.");
     IGRAPH_FINALLY(igraph_free, added);
 
@@ -796,10 +805,8 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
 
     *no = 0;
 
-    RNG_BEGIN();
-
     for (father = 0; father < no_of_nodes; father++) {
-        igraph_integer_t level, s;
+        igraph_int_t level, s;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -816,10 +823,12 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
 
         /* init V_E */
         igraph_vector_int_clear(&adjverts);
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, father, IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &neis, father, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+        ));
         s = igraph_vector_int_size(&neis);
         for (i = 0; i < s; i++) {
-            igraph_integer_t nei = VECTOR(neis)[i];
+            igraph_int_t nei = VECTOR(neis)[i];
             if (!added[nei] && nei > father) {
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
                 IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, father));
@@ -846,8 +855,8 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
             if (level < size - 1 &&
                 !igraph_vector_int_empty(&adjverts)) {
                 /* We might step down */
-                igraph_integer_t neifather = igraph_vector_int_pop_back(&adjverts);
-                igraph_integer_t nei = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t neifather = igraph_vector_int_pop_back(&adjverts);
+                igraph_int_t nei = igraph_vector_int_pop_back(&adjverts);
 
                 if (cp == 0 || RNG_UNIF01() > cp) {
                     /* Yes, step down */
@@ -858,10 +867,12 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
                     IGRAPH_CHECK(igraph_stack_int_push(&stack, nei));
                     IGRAPH_CHECK(igraph_stack_int_push(&stack, level));
 
-                    IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+                    IGRAPH_CHECK(igraph_neighbors(
+                        graph, &neis, nei, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+                    ));
                     s = igraph_vector_int_size(&neis);
                     for (i = 0; i < s; i++) {
-                        igraph_integer_t nei2 = VECTOR(neis)[i];
+                        igraph_int_t nei2 = VECTOR(neis)[i];
                         if (!added[nei2] && nei2 > father) {
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei2));
                             IGRAPH_CHECK(igraph_vector_int_push_back(&adjverts, nei));
@@ -871,7 +882,7 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
                 }
             } else {
                 /* no, step back */
-                igraph_integer_t nei, neifather;
+                igraph_int_t nei, neifather;
                 while (!igraph_stack_int_empty(&stack) &&
                        level == igraph_stack_int_top(&stack) - 1) {
                     igraph_stack_int_pop(&stack);
@@ -883,7 +894,9 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
 
                 nei = igraph_vector_int_pop_back(&vids);
                 added[nei] -= 1; level -= 1;
-                IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+                IGRAPH_CHECK(igraph_neighbors(
+                    graph, &neis, nei, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+                ));
                 s = igraph_vector_int_size(&neis);
                 for (i = 0; i < s; i++) {
                     added[ VECTOR(neis)[i] ] -= 1;
@@ -899,7 +912,9 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
 
         /* clear the added vector */
         added[father] -= 1;
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, father, IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &neis, father, IGRAPH_ALL, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+        ));
         s = igraph_vector_int_size(&neis);
         for (i = 0; i < s; i++) {
             added[ VECTOR(neis)[i] ] -= 1;
@@ -907,8 +922,6 @@ igraph_error_t igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t
 
     } /* for father */
 
-    RNG_END();
-
     IGRAPH_FREE(added);
     igraph_vector_int_destroy(&vids);
     igraph_vector_int_destroy(&adjverts);
@@ -957,18 +970,18 @@ igraph_error_t igraph_dyad_census(const igraph_t *graph, igraph_real_t *mut,
 
     igraph_real_t nonrec = 0, rec = 0;
     igraph_vector_int_t inneis, outneis;
-    igraph_integer_t vc = igraph_vcount(graph);
-    igraph_integer_t i;
+    igraph_int_t vc = igraph_vcount(graph);
+    igraph_int_t i;
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&inneis, 0);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&outneis, 0);
 
     for (i = 0; i < vc; i++) {
-        igraph_integer_t ideg, odeg;
-        igraph_integer_t ip, op;
+        igraph_int_t ideg, odeg;
+        igraph_int_t ip, op;
 
-        IGRAPH_CHECK(igraph_i_neighbors(graph, &inneis, i, IGRAPH_IN, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
-        IGRAPH_CHECK(igraph_i_neighbors(graph, &outneis, i, IGRAPH_OUT, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
+        IGRAPH_CHECK(igraph_neighbors(graph, &inneis, i, IGRAPH_IN, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
+        IGRAPH_CHECK(igraph_neighbors(graph, &outneis, i, IGRAPH_OUT, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
 
         ideg = igraph_vector_int_size(&inneis);
         odeg = igraph_vector_int_size(&outneis);
@@ -1006,10 +1019,10 @@ static igraph_error_t igraph_i_triad_census_24(const igraph_t *graph,
                                                igraph_real_t *res2,
                                                igraph_real_t *res4) {
 
-    const igraph_integer_t vcount = igraph_vcount(graph);
+    const igraph_int_t vcount = igraph_vcount(graph);
     igraph_vector_int_t seen;
     igraph_vector_int_t *neis, *neis2;
-    igraph_integer_t s, neilen, neilen2, ign;
+    igraph_int_t s, neilen, neilen2, ign;
     igraph_adjlist_t adjlist;
     int iter = 0;
 
@@ -1018,7 +1031,7 @@ static igraph_error_t igraph_i_triad_census_24(const igraph_t *graph,
     IGRAPH_FINALLY(igraph_adjlist_destroy, &adjlist);
     *res2 = *res4 = 0;
 
-    for (igraph_integer_t i = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 12);
 
         neis = igraph_adjlist_get(&adjlist, i);
@@ -1026,8 +1039,8 @@ static igraph_error_t igraph_i_triad_census_24(const igraph_t *graph,
         /* mark neighbors of i & i itself */
         VECTOR(seen)[i] = i + 1;
         ign = 0;
-        for (igraph_integer_t j = 0; j < neilen; j++) {
-            igraph_integer_t nei = VECTOR(*neis)[j];
+        for (igraph_int_t j = 0; j < neilen; j++) {
+            igraph_int_t nei = VECTOR(*neis)[j];
             if (VECTOR(seen)[nei] == i + 1 || VECTOR(seen)[nei] == -(i + 1)) {
                 /* multiple edges or loop edge */
                 VECTOR(seen)[nei] = -(i + 1);
@@ -1037,16 +1050,16 @@ static igraph_error_t igraph_i_triad_census_24(const igraph_t *graph,
             }
         }
 
-        for (igraph_integer_t j = 0; j < neilen; j++) {
-            igraph_integer_t nei = VECTOR(*neis)[j];
+        for (igraph_int_t j = 0; j < neilen; j++) {
+            igraph_int_t nei = VECTOR(*neis)[j];
             if (nei <= i || (j > 0 && nei == VECTOR(*neis)[j - 1])) {
                 continue;
             }
             neis2 = igraph_adjlist_get(&adjlist, nei);
             neilen2 = igraph_vector_int_size(neis2);
             s = 0;
-            for (igraph_integer_t k = 0; k < neilen2; k++) {
-                igraph_integer_t nei2 = VECTOR(*neis2)[k];
+            for (igraph_int_t k = 0; k < neilen2; k++) {
+                igraph_int_t nei2 = VECTOR(*neis2)[k];
                 if (k > 0 && nei2 == VECTOR(*neis2)[k - 1]) {
                     continue;
                 }
@@ -1148,7 +1161,7 @@ static igraph_error_t igraph_i_triad_census_24(const igraph_t *graph,
 
 igraph_error_t igraph_triad_census(const igraph_t *graph, igraph_vector_t *res) {
 
-    const igraph_integer_t vcount = igraph_vcount(graph);
+    const igraph_int_t vcount = igraph_vcount(graph);
     igraph_vector_t cut_prob, tmp;
     igraph_real_t m2, m4;
     igraph_real_t total;
