@@ -1,8 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
-   Copyright (C) 2005-2021 The igraph development team
+   igraph library.
+   Copyright (C) 2005-2025 The igraph development team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,11 +19,13 @@
 #include "igraph_paths.h"
 
 #include "igraph_adjlist.h"
+#include "igraph_bitset.h"
 #include "igraph_dqueue.h"
 #include "igraph_interface.h"
 #include "igraph_memory.h"
 
 #include "core/interruption.h"
+#include "paths/paths_internal.h"
 
 /**
  * \function igraph_distances_bellman_ford
@@ -44,10 +44,10 @@
  *    vertices the matrix contains \c IGRAPH_INFINITY.
  * \param from The source vertices.
  * \param to The target vertices.
- * \param weights The edge weights. There must not be any closed loop in
+ * \param weights The edge weights. There must not be any cycle in
  *    the graph that has a negative total weight (since this would allow
  *    us to decrease the weight of any path containing at least a single
- *    vertex of this loop infinitely). Additionally, no edge weight may
+ *    vertex of this cycle infinitely). Additionally, no edge weight may
  *    be NaN. If either case does not hold, an error is returned. If this
  *    is a null pointer, then the unweighted version,
  *    \ref igraph_distances() is called.
@@ -60,49 +60,56 @@
  * Time complexity: O(s*|E|*|V|), where |V| is the number of
  * vertices, |E| the number of edges and s the number of sources.
  *
- * \sa \ref igraph_distances() for a faster unweighted version
- * or \ref igraph_distances_dijkstra() if you do not have negative
+ * \sa \ref igraph_distances() for a non-algorithm-specific interface;
+ * \ref igraph_distances_dijkstra() if you do not have negative
  * edge weights.
  *
  * \example examples/simple/bellman_ford.c
  */
-igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
-                                       igraph_matrix_t *res,
-                                       const igraph_vs_t from,
-                                       const igraph_vs_t to,
-                                       const igraph_vector_t *weights,
-                                       igraph_neimode_t mode) {
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
+igraph_error_t igraph_distances_bellman_ford(
+        const igraph_t *graph,
+        igraph_matrix_t *res,
+        const igraph_vs_t from,
+        const igraph_vs_t to,
+        const igraph_vector_t *weights,
+        igraph_neimode_t mode) {
+
+    igraph_bool_t negative_weights;
+    IGRAPH_CHECK(igraph_i_validate_distance_weights(graph, weights, &negative_weights));
+    return igraph_i_distances_bellman_ford(graph, res, from, to, weights, mode);
+}
+
+igraph_error_t igraph_i_distances_bellman_ford(
+        const igraph_t *graph,
+        igraph_matrix_t *res,
+        const igraph_vs_t from, const igraph_vs_t to,
+        const igraph_vector_t *weights,
+        igraph_neimode_t mode) {
+
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
     igraph_lazy_inclist_t inclist;
-    igraph_integer_t i, j, k;
-    igraph_integer_t no_of_from, no_of_to;
+    igraph_int_t i;
+    igraph_int_t no_of_from, no_of_to;
     igraph_dqueue_int_t Q;
-    igraph_vector_int_t clean_vertices;
+    igraph_bitset_t clean_vertices;
     igraph_vector_int_t num_queued;
     igraph_vit_t fromvit, tovit;
     igraph_bool_t all_to;
     igraph_vector_t dist;
+    int iter = 0;
 
     /*
        - speedup: a vertex is marked clean if its distance from the source
          did not change during the last phase. Neighbors of a clean vertex
          are not relaxed again, since it would mean no change in the
-         shortest path values. Dirty vertices are queued. Negative loops can
+         shortest path values. Dirty vertices are queued. Negative cycles can
          be detected by checking whether a vertex has been queued at least
          n times.
     */
-    if (!weights) {
-        return igraph_distances(graph, res, from, to, mode);
-    }
 
-    if (igraph_vector_size(weights) != no_of_edges) {
-        IGRAPH_ERRORF("Weight vector length (%" IGRAPH_PRId ") does not match number of edges (%" IGRAPH_PRId ").",
-                      IGRAPH_EINVAL,
-                      igraph_vector_size(weights), no_of_edges);
-    }
-    if (no_of_edges > 0 && igraph_vector_is_any_nan(weights)) {
-        IGRAPH_ERROR("Weight vector must not contain NaN values.", IGRAPH_EINVAL);
+    if (!weights || no_of_edges == 0) {
+        return igraph_i_distances_unweighted_cutoff(graph, res, from, to, mode, -1);
     }
 
     IGRAPH_CHECK(igraph_vit_create(graph, from, &fromvit));
@@ -110,7 +117,7 @@ igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
     no_of_from = IGRAPH_VIT_SIZE(fromvit);
 
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&Q, no_of_nodes);
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&clean_vertices, no_of_nodes);
+    IGRAPH_BITSET_INIT_FINALLY(&clean_vertices, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&num_queued, no_of_nodes);
     IGRAPH_CHECK(igraph_lazy_inclist_init(graph, &inclist, mode, IGRAPH_LOOPS));
     IGRAPH_FINALLY(igraph_lazy_inclist_destroy, &inclist);
@@ -135,49 +142,48 @@ igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
     for (IGRAPH_VIT_RESET(fromvit), i = 0;
          !IGRAPH_VIT_END(fromvit);
          IGRAPH_VIT_NEXT(fromvit), i++) {
-        igraph_integer_t source = IGRAPH_VIT_GET(fromvit);
+        igraph_int_t source = IGRAPH_VIT_GET(fromvit);
 
         igraph_vector_fill(&dist, IGRAPH_INFINITY);
         VECTOR(dist)[source] = 0;
-        igraph_vector_int_null(&clean_vertices);
+        igraph_bitset_null(&clean_vertices);
         igraph_vector_int_null(&num_queued);
 
         /* Fill the queue with vertices to be checked */
-        for (j = 0; j < no_of_nodes; j++) {
+        for (igraph_int_t j = 0; j < no_of_nodes; j++) {
             IGRAPH_CHECK(igraph_dqueue_int_push(&Q, j));
         }
 
         while (!igraph_dqueue_int_empty(&Q)) {
-            igraph_vector_int_t *neis;
-            igraph_integer_t nlen;
+            IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 13);
 
-            j = igraph_dqueue_int_pop(&Q);
-            VECTOR(clean_vertices)[j] = 1;
+            igraph_int_t j = igraph_dqueue_int_pop(&Q);
+            IGRAPH_BIT_SET(clean_vertices, j);
             VECTOR(num_queued)[j] += 1;
             if (VECTOR(num_queued)[j] > no_of_nodes) {
-                IGRAPH_ERROR("Negative loop in graph while calculating distances with Bellman-Ford algorithm.",
-                             IGRAPH_ENEGLOOP);
+                IGRAPH_ERROR("Negative cycle in graph while calculating distances with Bellman-Ford algorithm.",
+                             IGRAPH_ENEGCYCLE);
             }
 
             /* If we cannot get to j in finite time yet, there is no need to relax
              * its edges */
-            if (!isfinite(VECTOR(dist)[j])) {
+            if (VECTOR(dist)[j] == IGRAPH_INFINITY) {
                 continue;
             }
 
-            neis = igraph_lazy_inclist_get(&inclist, j);
+            igraph_vector_int_t *neis = igraph_lazy_inclist_get(&inclist, j);
             IGRAPH_CHECK_OOM(neis, "Failed to query incident edges.");
 
-            nlen = igraph_vector_int_size(neis);
-
-            for (k = 0; k < nlen; k++) {
-                igraph_integer_t nei = VECTOR(*neis)[k];
-                igraph_integer_t target = IGRAPH_OTHER(graph, nei, j);
-                if (VECTOR(dist)[target] > VECTOR(dist)[j] + VECTOR(*weights)[nei]) {
+            igraph_int_t nlen = igraph_vector_int_size(neis);
+            for (igraph_int_t k = 0; k < nlen; k++) {
+                igraph_int_t nei = VECTOR(*neis)[k];
+                igraph_int_t target = IGRAPH_OTHER(graph, nei, j);
+                igraph_real_t altdist = VECTOR(dist)[j] + VECTOR(*weights)[nei];
+                if (VECTOR(dist)[target] > altdist) {
                     /* relax the edge */
-                    VECTOR(dist)[target] = VECTOR(dist)[j] + VECTOR(*weights)[nei];
-                    if (VECTOR(clean_vertices)[target]) {
-                        VECTOR(clean_vertices)[target] = 0;
+                    VECTOR(dist)[target] = altdist;
+                    if (IGRAPH_BIT_TEST(clean_vertices, target)) {
+                        IGRAPH_BIT_CLEAR(clean_vertices, target);
                         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, target));
                     }
                 }
@@ -186,11 +192,12 @@ igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
 
         /* Copy it to the result */
         if (all_to) {
-            igraph_matrix_set_row(res, &dist, i);
+            IGRAPH_CHECK(igraph_matrix_set_row(res, &dist, i));
         } else {
+            igraph_int_t j;
             for (IGRAPH_VIT_RESET(tovit), j = 0; !IGRAPH_VIT_END(tovit);
                  IGRAPH_VIT_NEXT(tovit), j++) {
-                igraph_integer_t v = IGRAPH_VIT_GET(tovit);
+                igraph_int_t v = IGRAPH_VIT_GET(tovit);
                 MATRIX(*res, i, j) = VECTOR(dist)[v];
             }
         }
@@ -206,7 +213,7 @@ igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
 
     igraph_vit_destroy(&fromvit);
     igraph_dqueue_int_destroy(&Q);
-    igraph_vector_int_destroy(&clean_vertices);
+    igraph_bitset_destroy(&clean_vertices);
     igraph_vector_int_destroy(&num_queued);
     igraph_lazy_inclist_destroy(&inclist);
     IGRAPH_FINALLY_CLEAN(5);
@@ -214,32 +221,17 @@ igraph_error_t igraph_distances_bellman_ford(const igraph_t *graph,
     return IGRAPH_SUCCESS;
 }
 
-/**
- * \function igraph_shortest_paths_bellman_ford
- * \brief Weighted shortest path lengths between vertices, allowing negative weights (deprecated).
- *
- * \deprecated-by igraph_distances_bellman_ford 0.10.0
- */
-igraph_error_t igraph_shortest_paths_bellman_ford(const igraph_t *graph,
-                                       igraph_matrix_t *res,
-                                       const igraph_vs_t from,
-                                       const igraph_vs_t to,
-                                       const igraph_vector_t *weights,
-                                       igraph_neimode_t mode) {
-    return igraph_distances_bellman_ford(graph, res, from, to, weights, mode);
-}
 
 /**
  * \ingroup structural
  * \function igraph_get_shortest_paths_bellman_ford
  * \brief Weighted shortest paths from a vertex, allowing negative weights.
  *
- * This function calculates weighted shortest paths from or to a single vertex,
- * and allows negative weights. When there is more than one shortest path between
- * two vertices, only one of them is returned.
- *
- * If there are no negative weights, you are better off with
- * \ref igraph_get_shortest_paths_dijkstra() .
+ * This function calculates weighted shortest paths from or to a single vertex
+ * using the Bellman-Ford algorithm, whihc can handle negative weights. When
+ * there is more than one shortest path between two vertices, only one of them
+ * is returned. When there are no negative weights,
+ * \ref igraph_get_shortest_paths_dijkstra() is likely to be faster.
  *
  * \param graph The input graph, can be directed.
  * \param vertices The result, the IDs of the vertices along the paths.
@@ -255,11 +247,12 @@ igraph_error_t igraph_shortest_paths_bellman_ford(const igraph_t *graph,
  * \param to Vertex sequence with the IDs of the vertices to/from which the
  *        shortest paths will be calculated. A vertex might be given multiple
  *        times.
- * \param weights The edge weights. There must not be any closed loop in
+ * \param weights The edge weights. There must not be any cycle in
  *    the graph that has a negative total weight (since this would allow
  *    us to decrease the weight of any path containing at least a single
- *    vertex of this loop infinitely). If this is a null pointer, then the
+ *    vertex of this cycle infinitely). If this is a null pointer, then the
  *    unweighted version, \ref igraph_get_shortest_paths() is called.
+ *    Edges with positive infinite weights are ignored.
  * \param mode For directed graphs; whether to follow paths along edge
  *    directions (\c IGRAPH_OUT), or the opposite (\c IGRAPH_IN), or
  *    ignore edge directions completely (\c IGRAPH_ALL). It is ignored
@@ -289,49 +282,65 @@ igraph_error_t igraph_shortest_paths_bellman_ford(const igraph_t *graph,
  *           The weight vector doesn't math the number of edges.
  *         \cli IGRAPH_EINVVID
  *           \p from is invalid vertex ID
- *         \cli IGRAPH_ENEGLOOP
- *           Bellman-ford algorithm encounted a negative loop.
+ *         \cli IGRAPH_ENEGCYCLE
+ *           Bellman-ford algorithm encounted a negative cycle.
  *         \endclist
  *
  * Time complexity: O(|E|*|V|), where |V| is the number of
  * vertices, |E| the number of edges.
  *
- * \sa \ref igraph_get_shortest_paths() for a faster unweighted version
- * or \ref igraph_get_shortest_paths_dijkstra() if you do not have negative
- * edge weights.
+ * \sa \ref igraph_distances_bellman_ford() to compute only shortest path
+ * lengths, but not the paths themselves; \ref igraph_get_shortest_paths() for
+ * a non-algorithm-specific interface.
  */
+igraph_error_t igraph_get_shortest_paths_bellman_ford(
+        const igraph_t *graph,
+        igraph_vector_int_list_t *vertices,
+        igraph_vector_int_list_t *edges,
+        igraph_int_t from,
+        igraph_vs_t to,
+        const igraph_vector_t *weights,
+        igraph_neimode_t mode,
+        igraph_vector_int_t *parents,
+        igraph_vector_int_t *inbound_edges) {
 
-igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
-                                        igraph_vector_int_list_t *vertices,
-                                        igraph_vector_int_list_t *edges,
-                                        igraph_integer_t from,
-                                        igraph_vs_t to,
-                                        const igraph_vector_t *weights,
-                                        igraph_neimode_t mode,
-                                        igraph_vector_int_t *parents,
-                                        igraph_vector_int_t *inbound_edges) {
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
-    igraph_integer_t *parent_eids;
+    igraph_bool_t negative_weights;
+    IGRAPH_CHECK(igraph_i_validate_distance_weights(graph, weights, &negative_weights));
+    return igraph_i_get_shortest_paths_bellman_ford(graph, vertices, edges, from, to, weights, mode, parents, inbound_edges);
+}
+
+igraph_error_t igraph_i_get_shortest_paths_bellman_ford(
+        const igraph_t *graph,
+        igraph_vector_int_list_t *vertices,
+        igraph_vector_int_list_t *edges,
+        igraph_int_t from,
+        igraph_vs_t to,
+        const igraph_vector_t *weights,
+        igraph_neimode_t mode,
+        igraph_vector_int_t *parents,
+        igraph_vector_int_t *inbound_edges) {
+
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t *parent_eids;
     igraph_lazy_inclist_t inclist;
-    igraph_integer_t i, j, k;
+    igraph_int_t i, j, k;
     igraph_dqueue_int_t Q;
-    igraph_vector_int_t clean_vertices;
+    igraph_bitset_t clean_vertices;
     igraph_vector_int_t num_queued;
     igraph_vit_t tovit;
     igraph_vector_t dist;
+    int iter = 0;
 
     if (!weights) {
-        return  igraph_get_shortest_paths(graph, vertices, edges, from, to, mode,
-                                         parents, inbound_edges);
+        return igraph_i_get_shortest_paths_unweighted(graph, vertices, edges, from, to, mode, parents, inbound_edges);
     }
 
-    if (igraph_vector_size(weights) != no_of_edges) {
-        IGRAPH_ERROR("Weight vector length must match number of edges.", IGRAPH_EINVAL);
+    if (from < 0 || from >= no_of_nodes) {
+        IGRAPH_ERROR("Index of source vertex is out of range.", IGRAPH_EINVVID);
     }
 
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&Q, no_of_nodes);
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&clean_vertices, no_of_nodes);
+    IGRAPH_BITSET_INIT_FINALLY(&clean_vertices, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&num_queued, no_of_nodes);
     IGRAPH_CHECK(igraph_lazy_inclist_init(graph, &inclist, mode, IGRAPH_LOOPS));
     IGRAPH_FINALLY(igraph_lazy_inclist_destroy, &inclist);
@@ -346,17 +355,14 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
         IGRAPH_CHECK(igraph_vector_int_list_resize(edges, IGRAPH_VIT_SIZE(tovit)));
     }
 
-    parent_eids = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
-    if (parent_eids == 0) {
-        IGRAPH_ERROR("Insufficient memory for shortest paths with Bellman-Ford.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    parent_eids = IGRAPH_CALLOC(no_of_nodes, igraph_int_t);
+    IGRAPH_CHECK_OOM(parent_eids, "Insufficient memory for shortest paths with Bellman-Ford.");
     IGRAPH_FINALLY(igraph_free, parent_eids);
+
     IGRAPH_VECTOR_INIT_FINALLY(&dist, no_of_nodes);
 
     igraph_vector_fill(&dist, IGRAPH_INFINITY);
     VECTOR(dist)[from] = 0;
-    igraph_vector_int_null(&clean_vertices);
-    igraph_vector_int_null(&num_queued);
 
     /* Fill the queue with vertices to be checked */
     for (j = 0; j < no_of_nodes; j++) {
@@ -364,37 +370,41 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
     }
 
     while (!igraph_dqueue_int_empty(&Q)) {
-        igraph_vector_int_t *neis;
-        igraph_integer_t nlen;
+        IGRAPH_ALLOW_INTERRUPTION_LIMITED(iter, 1 << 13);
 
         j = igraph_dqueue_int_pop(&Q);
-        VECTOR(clean_vertices)[j] = 1;
+        IGRAPH_BIT_SET(clean_vertices, j);
         VECTOR(num_queued)[j] += 1;
         if (VECTOR(num_queued)[j] > no_of_nodes) {
-            IGRAPH_ERROR("Negative loop in graph while calculating distances with Bellman-Ford algorithm.",
-                         IGRAPH_ENEGLOOP);
+            IGRAPH_ERROR("Negative cycle in graph while calculating distances with Bellman-Ford algorithm.",
+                         IGRAPH_ENEGCYCLE);
         }
 
-        /* If we cannot get to j in finite time yet, there is no need to relax
-            * its edges */
-        if (!isfinite(VECTOR(dist)[j])) {
+        /* If we cannot get to j in finite time yet, there is no need to relax its edges */
+        if (VECTOR(dist)[j] == IGRAPH_INFINITY) {
             continue;
         }
 
-        neis = igraph_lazy_inclist_get(&inclist, j);
+        igraph_vector_int_t *neis = igraph_lazy_inclist_get(&inclist, j);
         IGRAPH_CHECK_OOM(neis, "Failed to query incident edges.");
 
-        nlen = igraph_vector_int_size(neis);
-
+        igraph_int_t nlen = igraph_vector_int_size(neis);
         for (k = 0; k < nlen; k++) {
-            igraph_integer_t nei = VECTOR(*neis)[k];
-            igraph_integer_t target = IGRAPH_OTHER(graph, nei, j);
-            if (VECTOR(dist)[target] > VECTOR(dist)[j] + VECTOR(*weights)[nei]) {
+            igraph_int_t nei = VECTOR(*neis)[k];
+            igraph_int_t target = IGRAPH_OTHER(graph, nei, j);
+            igraph_real_t weight = VECTOR(*weights)[nei];
+            igraph_real_t altdist = VECTOR(dist)[j] + weight;
+
+            /* infinite weights are handled correctly here; if an edge has
+             * infinite weight, altdist will also be infinite so the condition
+             * will never be true as if the edge was ignored */
+
+            if (VECTOR(dist)[target] > altdist) {
                 /* relax the edge */
-                VECTOR(dist)[target] = VECTOR(dist)[j] + VECTOR(*weights)[nei];
+                VECTOR(dist)[target] = altdist;
                 parent_eids[target] = nei + 1;
-                if (VECTOR(clean_vertices)[target]) {
-                    VECTOR(clean_vertices)[target] = 0;
+                if (IGRAPH_BIT_TEST(clean_vertices, target)) {
+                    IGRAPH_BIT_CLEAR(clean_vertices, target);
                     IGRAPH_CHECK(igraph_dqueue_int_push(&Q, target));
                 }
             }
@@ -437,8 +447,8 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
     /* Reconstruct the shortest paths based on vertex and/or edge IDs */
     if (vertices || edges) {
         for (IGRAPH_VIT_RESET(tovit), i = 0; !IGRAPH_VIT_END(tovit); IGRAPH_VIT_NEXT(tovit), i++) {
-            igraph_integer_t node = IGRAPH_VIT_GET(tovit);
-            igraph_integer_t size, act, edge;
+            igraph_int_t node = IGRAPH_VIT_GET(tovit);
+            igraph_int_t size, act, edge;
             igraph_vector_int_t *vvec = 0, *evec = 0;
             if (vertices) {
                 vvec = igraph_vector_int_list_get_ptr(vertices, i);
@@ -488,7 +498,7 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
 
     IGRAPH_FREE(parent_eids);
     igraph_dqueue_int_destroy(&Q);
-    igraph_vector_int_destroy(&clean_vertices);
+    igraph_bitset_destroy(&clean_vertices);
     igraph_vector_int_destroy(&num_queued);
     igraph_lazy_inclist_destroy(&inclist);
     IGRAPH_FINALLY_CLEAN(5);
@@ -498,10 +508,10 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
 
 /**
  * \function igraph_get_shortest_path_bellman_ford
- * \brief Weighted shortest path from one vertex to another one.
+ * \brief Weighted shortest path from one vertex to another one (Bellman-Ford).
  *
- * Calculates a single (positively) weighted shortest path from
- * a single vertex to another one, using Bellman-Ford algorithm.
+ * Finds a weighted shortest path from a single source vertex to
+ * a single target using the Bellman-Ford algorithm.
  *
  * </para><para>
  * This function is a special case (and a wrapper) to
@@ -515,12 +525,12 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
  * \param edges Pointer to an initialized vector or a null
  *        pointer. If not a null pointer, then the edge IDs along the
  *        path are stored here.
- * \param from The id of the source vertex.
- * \param to The id of the target vertex.
- * \param weights The edge weights. There must not be any closed loop in
+ * \param from The ID of the source vertex.
+ * \param to The ID of the target vertex.
+ * \param weights The edge weights. There must not be any cycle in
  *        the graph that has a negative total weight (since this would allow
  *        us to decrease the weight of any path containing at least a single
- *        vertex of this loop infinitely). If this is a null pointer, then the
+ *        vertex of this cycle infinitely). If this is a null pointer, then the
  *        unweighted version is called.
  * \param mode A constant specifying how edge directions are
  *        considered in directed graphs. \c IGRAPH_OUT follows edge
@@ -539,8 +549,8 @@ igraph_error_t igraph_get_shortest_paths_bellman_ford(const igraph_t *graph,
 igraph_error_t igraph_get_shortest_path_bellman_ford(const igraph_t *graph,
                                           igraph_vector_int_t *vertices,
                                           igraph_vector_int_t *edges,
-                                          igraph_integer_t from,
-                                          igraph_integer_t to,
+                                          igraph_int_t from,
+                                          igraph_int_t to,
                                           const igraph_vector_t *weights,
                                           igraph_neimode_t mode) {
 
@@ -564,13 +574,15 @@ igraph_error_t igraph_get_shortest_path_bellman_ford(const igraph_t *graph,
                                                         from, igraph_vss_1(to),
                                                         weights, mode, NULL, NULL));
 
+    /* We use the constant time vector_swap() instead of the linear-time vector_update() to move the
+       result to the output parameter. */
     if (edges) {
-        IGRAPH_CHECK(igraph_vector_int_update(edges, igraph_vector_int_list_get_ptr(&edges2, 0)));
+        igraph_vector_int_swap(edges, igraph_vector_int_list_get_ptr(&edges2, 0));
         igraph_vector_int_list_destroy(&edges2);
         IGRAPH_FINALLY_CLEAN(1);
     }
     if (vertices) {
-        IGRAPH_CHECK(igraph_vector_int_update(vertices, igraph_vector_int_list_get_ptr(&vertices2, 0)));
+        igraph_vector_int_swap(vertices, igraph_vector_int_list_get_ptr(&vertices2, 0));
         igraph_vector_int_list_destroy(&vertices2);
         IGRAPH_FINALLY_CLEAN(1);
     }
