@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -29,7 +28,7 @@
 #include "igraph_types.h"
 #include "igraph_vector.h"
 
-#include "core/math.h"
+#include "core/interruption.h"
 #include "math/safe_intop.h"
 #include "random/random_internal.h"
 
@@ -42,6 +41,7 @@
 #include <assert.h>
 #include <math.h>
 #include <float.h> /* DBL_MANT_DIG */
+#include <stdbool.h>
 
 /**
  * \section about_rngs
@@ -145,32 +145,32 @@
  * igraph_rng_set_default() function.
  */
 
-extern IGRAPH_THREAD_LOCAL igraph_rng_t igraph_i_rng_default; /* defined in rng_pcg32.c */
+extern igraph_rng_t igraph_i_rng_default; /* defined in rng_pcg32.c */
+IGRAPH_THREAD_LOCAL igraph_rng_t *igraph_i_rng_default_ptr = &igraph_i_rng_default;
 
 /**
  * \function igraph_rng_set_default
  * \brief Set the default igraph random number generator.
  *
- * This function \em copies the internal structure of the given \type igraph_rng_t
- * object to igraph's internal default RNG structure. The structure itself
- * contains two pointers only, one to the "methods" of the RNG and one to the
- * memory buffer holding the internal state of the RNG. This means that if you
- * keep on generating random numbers from the RNG after setting it as the
- * default, it will affect the state of the default RNG as well because the two
- * share the same state pointer. However, do \em not expect
- * \ref igraph_rng_default() to return the same pointer as the one you passed
- * in here - the state is shared, but the entire structure is not.
+ * This function updates the default RNG used by igraph to be the one
+ * pointed to by \p rng, and returns a pointer to the previous default
+ * RNG. Future calls to \ref igraph_rng_default() will return the same
+ * pointer as \p rng. The RNG pointed to by \p rng must not be destroyed
+ * for as long as it is used as the default.
  *
  * \param rng The random number generator to use as default from now
  *    on. Calling \ref igraph_rng_destroy() on it, while it is still
  *    being used as the default will result in crashes and/or
  *    unpredictable results.
+ * \return Pointer the previous default RNG.
  *
  * Time complexity: O(1).
  */
 
-void igraph_rng_set_default(igraph_rng_t *rng) {
-    igraph_i_rng_default = (*rng);
+igraph_rng_t *igraph_rng_set_default(igraph_rng_t *rng) {
+    igraph_rng_t *old_rng = igraph_i_rng_default_ptr;
+    igraph_i_rng_default_ptr = rng;
+    return old_rng;
 }
 
 
@@ -186,7 +186,7 @@ void igraph_rng_set_default(igraph_rng_t *rng) {
  */
 
 igraph_rng_t *igraph_rng_default(void) {
-    return &igraph_i_rng_default;
+    return igraph_i_rng_default_ptr;
 }
 
 /* ------------------------------------ */
@@ -207,7 +207,7 @@ static uint64_t igraph_i_rng_get_uint64_bounded(igraph_rng_t *rng, uint64_t rang
 
 static double igraph_i_norm_rand(igraph_rng_t *rng);
 static double igraph_i_exp_rand(igraph_rng_t *rng);
-static double igraph_i_rbinom(igraph_rng_t *rng, igraph_integer_t n, double pp);
+static double igraph_i_rbinom(igraph_rng_t *rng, igraph_int_t n, double pp);
 static double igraph_i_rexp(igraph_rng_t *rng, double rate);
 static double igraph_i_rgamma(igraph_rng_t *rng, double shape, double scale);
 static double igraph_i_rpois(igraph_rng_t *rng, double rate);
@@ -260,7 +260,7 @@ void igraph_rng_destroy(igraph_rng_t *rng) {
 igraph_error_t igraph_rng_seed(igraph_rng_t *rng, igraph_uint_t seed) {
     const igraph_rng_type_t *type = rng->type;
     IGRAPH_CHECK(type->seed(rng->state, seed));
-    rng->is_seeded = 1;
+    rng->is_seeded = true;
     return IGRAPH_SUCCESS;
 }
 
@@ -274,7 +274,7 @@ igraph_error_t igraph_rng_seed(igraph_rng_t *rng, igraph_uint_t seed) {
  *
  * Time complexity: O(1).
  */
-IGRAPH_EXPORT igraph_integer_t igraph_rng_bits(const igraph_rng_t* rng) {
+igraph_int_t igraph_rng_bits(const igraph_rng_t* rng) {
     return rng->type->bits;
 }
 
@@ -335,7 +335,7 @@ const char *igraph_rng_name(const igraph_rng_t *rng) {
  */
 static igraph_uint_t igraph_i_rng_get_random_bits(igraph_rng_t *rng, uint8_t bits) {
     const igraph_rng_type_t *type = rng->type;
-    igraph_integer_t rng_bitwidth = igraph_rng_bits(rng);
+    igraph_int_t rng_bitwidth = igraph_rng_bits(rng);
     igraph_uint_t result;
 
     if (rng_bitwidth >= bits) {
@@ -374,7 +374,7 @@ static igraph_uint_t igraph_i_rng_get_random_bits(igraph_rng_t *rng, uint8_t bit
  */
 static uint64_t igraph_i_rng_get_random_bits_uint64(igraph_rng_t *rng, uint8_t bits) {
     const igraph_rng_type_t *type = rng->type;
-    igraph_integer_t rng_bitwidth = igraph_rng_bits(rng);
+    igraph_int_t rng_bitwidth = igraph_rng_bits(rng);
     uint64_t result;
 
     if (rng_bitwidth >= bits) {
@@ -546,23 +546,50 @@ static igraph_uint_t igraph_i_rng_get_uint_bounded(igraph_rng_t *rng, igraph_uin
 #endif
 }
 
+
+/**
+ * \function igraph_rng_get_bool
+ * \brief Generate a random boolean.
+ *
+ * Use this function only when a single random boolean, i.e. a single bit
+ * is needed at a time. It is not efficient for generating multiple bits.
+ *
+ * \param rng Pointer to the RNG to use for the generation. Use \ref
+ *        igraph_rng_default() here to use the default igraph RNG.
+ * \return The generated bit, as a truth value.
+ */
+
+igraph_bool_t igraph_rng_get_bool(igraph_rng_t *rng) {
+    const igraph_rng_type_t *type = rng->type;
+    const igraph_int_t rng_bitwidth = igraph_rng_bits(rng);
+    /* Keep the highest bit as RNGs sometimes tend to have lower entropy in
+     * low bits than in high bits.
+     *
+     * Ensure that the return value is stricly 0 or 1 even if igraph_bool_t is
+     * defined to something else than a native bool.
+     */
+    return (type->get(rng->state) >> (rng_bitwidth - 1)) & (igraph_uint_t)1;
+}
+
 /**
  * \function igraph_rng_get_integer
  * \brief Generate an integer random number from an interval.
  *
+ * Generate uniformly distributed integers from the interval <code>[l, h]</code>.
+ *
  * \param rng Pointer to the RNG to use for the generation. Use \ref
  *        igraph_rng_default() here to use the default igraph RNG.
  * \param l Lower limit, inclusive, it can be negative as well.
  * \param h Upper limit, inclusive, it can be negative as well, but it
- *        should be at least <code>l</code>.
+ *        must be at least <code>l</code>.
  * \return The generated random integer.
  *
- * Time complexity: O(log2(h-l) / bits) where bits is the value of
+ * Time complexity: O(log2(h-l+1) / bits) where bits is the value of
  * \ref igraph_rng_bits(rng).
  */
 
-igraph_integer_t igraph_rng_get_integer(
-    igraph_rng_t *rng, igraph_integer_t l, igraph_integer_t h
+igraph_int_t igraph_rng_get_integer(
+    igraph_rng_t *rng, igraph_int_t l, igraph_int_t h
 ) {
     const igraph_rng_type_t *type = rng->type;
     igraph_uint_t range;
@@ -580,9 +607,9 @@ igraph_integer_t igraph_rng_get_integer(
     if (IGRAPH_UNLIKELY(l == IGRAPH_INTEGER_MIN && h == IGRAPH_INTEGER_MAX)) {
         /* Full uint range is needed, we can just grab a random number from
          * the uint range and cast it to a signed integer */
-        return (igraph_integer_t) igraph_i_rng_get_uint(rng);
+        return (igraph_int_t) igraph_i_rng_get_uint(rng);
     } else if (l >= 0 || h < 0) {
-        /* this is okay, (h - l) will not overflow an igraph_integer_t */
+        /* this is okay, (h - l) will not overflow an igraph_int_t */
         range = (igraph_uint_t)(h - l) + 1;
     } else {
         /* (h - l) could potentially overflow so we need to play it safe. If we
@@ -680,8 +707,13 @@ igraph_real_t igraph_rng_get_unif01(igraph_rng_t *rng) {
          * Then we subtract 1 to arrive at the [0; 1) interval. This is fast
          * but we lose one bit of precision as there are 2^53 possible doubles
          * between 0 and 1. */
-        uint64_t r = (igraph_i_rng_get_random_bits_uint64(rng, 52) & 0xFFFFFFFFFFFFFull) | 0x3FF0000000000000ull;
-        return *(double *)(&r) - 1.0;
+        union {
+            uint64_t as_uint64_t;
+            double as_double;
+        } value;
+        value.as_uint64_t =
+            (igraph_i_rng_get_random_bits_uint64(rng, 52) & 0xFFFFFFFFFFFFFull) | 0x3FF0000000000000ull;
+        return value.as_double - 1.0;
     }
 }
 
@@ -735,7 +767,7 @@ igraph_real_t igraph_rng_get_geom(igraph_rng_t *rng, igraph_real_t p) {
  * Time complexity: depends on the RNG.
  */
 
-igraph_real_t igraph_rng_get_binom(igraph_rng_t *rng, igraph_integer_t n, igraph_real_t p) {
+igraph_real_t igraph_rng_get_binom(igraph_rng_t *rng, igraph_int_t n, igraph_real_t p) {
     const igraph_rng_type_t *type = rng->type;
     if (type->get_binom) {
         return type->get_binom(rng->state, n, p);
@@ -840,16 +872,16 @@ igraph_real_t igraph_rng_get_pois(igraph_rng_t *rng, igraph_real_t rate) {
  */
 
 static void igraph_i_random_sample_alga(igraph_vector_int_t *res,
-                                        igraph_integer_t l, igraph_integer_t h,
-                                        igraph_integer_t length) {
+                                        igraph_int_t l, igraph_int_t h,
+                                        igraph_int_t length) {
     /* Vitter: Variables V, quot, Nreal, and top are of type real */
 
-    igraph_integer_t N = h - l + 1;
-    igraph_integer_t n = length;
+    igraph_int_t N = h - l + 1;
+    igraph_int_t n = length;
 
     igraph_real_t top = N - n;
     igraph_real_t Nreal = N;
-    igraph_integer_t S = 0;
+    igraph_int_t S = 0;
     igraph_real_t V, quot;
 
     l = l - 1;
@@ -914,23 +946,23 @@ static void igraph_i_random_sample_alga(igraph_vector_int_t *res,
  * \example examples/simple/igraph_random_sample.c
  */
 
-igraph_error_t igraph_random_sample(igraph_vector_int_t *res, igraph_integer_t l, igraph_integer_t h,
-                         igraph_integer_t length) {
-    igraph_integer_t N; /* := h - l + 1 */
+igraph_error_t igraph_random_sample(igraph_vector_int_t *res, igraph_int_t l, igraph_int_t h,
+                         igraph_int_t length) {
+    igraph_int_t N; /* := h - l + 1 */
     IGRAPH_SAFE_ADD(h, -l, &N);
     IGRAPH_SAFE_ADD(N, 1, &N);
 
-    igraph_integer_t n = length;
+    igraph_int_t n = length;
 
     igraph_real_t nreal = length;
     igraph_real_t ninv = (nreal != 0) ? 1.0 / nreal : 0.0;
     igraph_real_t Nreal = N;
     igraph_real_t Vprime;
-    igraph_integer_t qu1 = -n + 1 + N;
+    igraph_int_t qu1 = -n + 1 + N;
     igraph_real_t qu1real = -nreal + 1.0 + Nreal;
     igraph_real_t negalphainv = -13;
     igraph_real_t threshold = -negalphainv * n;
-    igraph_integer_t S;
+    igraph_int_t S;
 
     /* getting back some sense of sanity */
     if (l > h) {
@@ -953,7 +985,7 @@ igraph_error_t igraph_random_sample(igraph_vector_int_t *res, igraph_integer_t l
     }
     if (length == N) {
         IGRAPH_CHECK(igraph_vector_int_resize(res, length));
-        for (igraph_integer_t i = 0; i < length; i++) {
+        for (igraph_int_t i = 0; i < length; i++) {
             VECTOR(*res)[i] = l++;
         }
         return IGRAPH_SUCCESS;
@@ -962,8 +994,6 @@ igraph_error_t igraph_random_sample(igraph_vector_int_t *res, igraph_integer_t l
     igraph_vector_int_clear(res);
     IGRAPH_CHECK(igraph_vector_int_reserve(res, length));
 
-    RNG_BEGIN();
-
     Vprime = exp(log(RNG_UNIF01()) * ninv);
     l = l - 1;
 
@@ -1028,8 +1058,6 @@ igraph_error_t igraph_random_sample(igraph_vector_int_t *res, igraph_integer_t l
         igraph_vector_int_push_back(res, l);    /* allocated */
     }
 
-    RNG_END();
-
     return IGRAPH_SUCCESS;
 }
 
@@ -1068,13 +1096,13 @@ static void igraph_i_random_sample_alga_real(igraph_vector_t *res,
 
 /**
  * \ingroup nongraph
- * \function igraph_random_sample_real
+ * \function igraph_i_random_sample_real
  * \brief Generates an increasing random sequence of integers (igraph_real_t version).
  *
  * This function is the 'real' version of \ref igraph_random_sample(), and was added
  * so \ref igraph_erdos_renyi_game_gnm() and related functions can use a random sample
  * of doubles instead of integers to prevent overflows on systems with 32-bit
- * \type igraph_integer_t.
+ * \type igraph_int_t.
  *
  * \param res Pointer to an initialized vector. This will hold the
  *        result. It will be resized to the proper size.
@@ -1093,11 +1121,11 @@ static void igraph_i_random_sample_alga_real(igraph_vector_t *res,
  *         size of the candidate pool.
  */
 
-igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
-                    igraph_real_t h, igraph_integer_t length) {
+igraph_error_t igraph_i_random_sample_real(igraph_vector_t *res, igraph_real_t l,
+                    igraph_real_t h, igraph_int_t length) {
     /* This function is the 'real' version of igraph_random_sample, and was added
      * so erdos_renyi_game_gnm can use a random sample of doubles instead of integers
-     * to prevent overflows on systems with 32-bits igraph_integer_t.
+     * to prevent overflows on systems with 32-bits igraph_int_t.
      */
     igraph_real_t N = h - l + 1;
     igraph_real_t n = length;
@@ -1111,6 +1139,7 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
     igraph_real_t negalphainv = -13;
     igraph_real_t threshold = -negalphainv * n;
     igraph_real_t S;
+    int iter = 0;
 
     /* getting back some sense of sanity */
     if (l > h) {
@@ -1138,7 +1167,7 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
     }
     if (length == N) {
         IGRAPH_CHECK(igraph_vector_resize(res, length));
-        for (igraph_integer_t i = 0; i < length; i++) {
+        for (igraph_int_t i = 0; i < length; i++) {
             VECTOR(*res)[i] = l++;
         }
         return IGRAPH_SUCCESS;
@@ -1147,8 +1176,6 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
     igraph_vector_clear(res);
     IGRAPH_CHECK(igraph_vector_reserve(res, length));
 
-    RNG_BEGIN();
-
     Vprime = exp(log(RNG_UNIF01()) * ninv);
     l = l - 1;
 
@@ -1203,6 +1230,11 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
         n = -1 + n;   nreal = -1.0 + nreal; ninv = nmin1inv;
         qu1 = -S + qu1; qu1real = negSreal + qu1real;
         threshold = threshold + negalphainv;
+
+        if (++iter >= (1 << 14)) {
+            iter = 0;
+            IGRAPH_ALLOW_INTERRUPTION();
+        }
     }
 
     if (n > 1) {
@@ -1213,8 +1245,6 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
         igraph_vector_push_back(res, l);    /* allocated */
     }
 
-    RNG_END();
-
     return IGRAPH_SUCCESS;
 }
 
@@ -1365,7 +1395,7 @@ igraph_error_t igraph_random_sample_real(igraph_vector_t *res, igraph_real_t l,
 #define R_DT_log(p) (lower_tail? R_D_log(p) : R_D_LExp(p))/* log(p) in qF */
 #define R_DT_Clog(p)    (lower_tail? R_D_LExp(p): R_D_log(p))/* log(1-p) in qF*/
 #define R_DT_Log(p) (lower_tail? (p) : R_Log1_Exp(p))
-/* ==   R_DT_log when we already "know" log_p == TRUE :*/
+/* ==   R_DT_log when we already "know" log_p == true :*/
 
 #define R_Q_P01_check(p)            \
     if ((log_p  && p > 0) ||            \
@@ -1485,11 +1515,11 @@ static double igraph_i_qnorm5(double p, double mu, double sigma, igraph_bool_t l
     return mu + sigma * val;
 }
 
-static igraph_integer_t imax2(igraph_integer_t x, igraph_integer_t y) {
+static igraph_int_t imax2(igraph_int_t x, igraph_int_t y) {
     return (x < y) ? y : x;
 }
 
-static igraph_integer_t imin2(igraph_integer_t x, igraph_integer_t y) {
+static igraph_int_t imin2(igraph_int_t x, igraph_int_t y) {
     return (x < y) ? x : y;
 }
 
@@ -1506,7 +1536,7 @@ static double igraph_i_norm_rand(igraph_rng_t *rng) {
         r = igraph_rng_get_unif01(rng);
     } while (r == 0.0);
 
-    return igraph_i_qnorm5(r, 0.0, 1.0, 1, 0);
+    return igraph_i_qnorm5(r, 0.0, 1.0, true, false);
 }
 
 /*
@@ -1576,8 +1606,6 @@ static double igraph_i_exp_rand(igraph_rng_t *rng) {
 
 #define repeat for(;;)
 
-#define FALSE 0
-#define TRUE  1
 #define M_1_SQRT_2PI    0.398942280401432677939946059934     /* 1/sqrt(2pi) */
 
 static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
@@ -1588,7 +1616,7 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
 
     /* These are static --- persistent between calls for same mu : */
     static IGRAPH_THREAD_LOCAL int l;
-    static IGRAPH_THREAD_LOCAL igraph_integer_t m;
+    static IGRAPH_THREAD_LOCAL igraph_int_t m;
 
     static IGRAPH_THREAD_LOCAL double b1, b2, c, c0, c1, c2, c3;
     static IGRAPH_THREAD_LOCAL double pp[36], p0, p, q, s, d, omega;
@@ -1598,7 +1626,9 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
     /* Local Vars  [initialize some for -Wall]: */
     double del, difmuk = 0., E = 0., fk = 0., fx, fy, g, px, py, t, u = 0., v, x;
     double pois = -1.;
-    int k, kflag, big_mu, new_big_mu = FALSE;
+    int k, big_mu;
+    bool new_big_mu = false;
+    bool kflag;
 
     if (!isfinite(mu) || mu < 0) {
         ML_ERR_return_NAN;
@@ -1610,13 +1640,13 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
 
     big_mu = mu >= 10.;
     if (big_mu) {
-        new_big_mu = FALSE;
+        new_big_mu = false;
     }
 
     if (!(big_mu && mu == muprev)) {/* maybe compute new persistent par.s */
 
         if (big_mu) {
-            new_big_mu = TRUE;
+            new_big_mu = true;
             /* Case A. (recalculation of s,d,l  because mu has changed):
              * The Poisson probabilities pk exceed the discrete normal
              * probabilities fk whenever k >= m(mu).
@@ -1633,7 +1663,7 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
             /*muprev = 0.;-* such that next time, mu != muprev ..*/
             if (mu != muprev) {
                 muprev = mu;
-                m = imax2(1, (igraph_integer_t) mu);
+                m = imax2(1, (igraph_int_t) mu);
                 l = 0; /* pp[] is already ok up to pp[l] */
                 q = p0 = p = exp(-mu);
             }
@@ -1718,7 +1748,7 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
 
     if (g >= 0.) {
         /* 'Subroutine' F is called (kflag=0 for correct return) */
-        kflag = 0;
+        kflag = false;
         goto Step_F;
     }
 
@@ -1738,7 +1768,7 @@ static double igraph_i_rpois(igraph_rng_t *rng, double mu) {
             difmuk = mu - fk;
 
             /* 'subroutine' F is called (kflag=1 for correct return) */
-            kflag = 1;
+            kflag = true;
 
 Step_F: /* 'subroutine' F : calculation of px,py,fx,fy. */
 
@@ -1764,7 +1794,7 @@ Step_F: /* 'subroutine' F : calculation of px,py,fx,fy. */
             x *= x;/* x^2 */
             fx = -0.5 * x;
             fy = omega * (((c3 * x + c2) * x + c1) * x + c0);
-            if (kflag > 0) {
+            if (kflag) {
                 /* Step H. Hat acceptance (E is repeated on rejection) */
                 if (c * fabs(u) <= py * exp(px + E) - fy * exp(fx + E)) {
                     break;
@@ -1791,18 +1821,18 @@ Step_F: /* 'subroutine' F : calculation of px,py,fx,fy. */
 
 #define repeat for(;;)
 
-static double igraph_i_rbinom(igraph_rng_t *rng, igraph_integer_t n, double pp) {
+static double igraph_i_rbinom(igraph_rng_t *rng, igraph_int_t n, double pp) {
 
     static IGRAPH_THREAD_LOCAL double c, fm, npq, p1, p2, p3, p4, qn;
     static IGRAPH_THREAD_LOCAL double xl, xll, xlr, xm, xr;
 
     static IGRAPH_THREAD_LOCAL double psave = -1.0;
-    static IGRAPH_THREAD_LOCAL igraph_integer_t nsave = -1;
-    static IGRAPH_THREAD_LOCAL igraph_integer_t m;
+    static IGRAPH_THREAD_LOCAL igraph_int_t nsave = -1;
+    static IGRAPH_THREAD_LOCAL igraph_int_t m;
 
     double f, f1, f2, u, v, w, w2, x, x1, x2, z, z2;
     double p, q, np, g, r, al, alv, amaxp, ffm, ynorm;
-    igraph_integer_t i, ix, k;
+    igraph_int_t i, ix, k;
 
     if (!isfinite(pp) ||
         /* n=0, p=0, p=1 are not errors <TSL>*/
@@ -2197,37 +2227,3 @@ static double igraph_i_rgamma(igraph_rng_t *rng, double a, double scale) {
     x = s + 0.5 * t;
     return scale * x * x;
 }
-
-igraph_error_t igraph_rng_get_dirichlet(igraph_rng_t *rng,
-                             const igraph_vector_t *alpha,
-                             igraph_vector_t *result) {
-
-    igraph_integer_t len = igraph_vector_size(alpha);
-    igraph_integer_t j;
-    igraph_real_t sum = 0.0;
-
-    if (len < 2) {
-        IGRAPH_ERROR("Dirichlet parameter vector too short, must have at least two entries.",
-                     IGRAPH_EINVAL);
-    }
-    if (igraph_vector_min(alpha) <= 0) {
-        IGRAPH_ERROR("Dirichlet concentration parameters must be positive.",
-                     IGRAPH_EINVAL);
-    }
-
-    IGRAPH_CHECK(igraph_vector_resize(result, len));
-
-    RNG_BEGIN();
-
-    for (j = 0; j < len; j++) {
-        VECTOR(*result)[j] = igraph_rng_get_gamma(rng, VECTOR(*alpha)[j], 1.0);
-        sum += VECTOR(*result)[j];
-    }
-    for (j = 0; j < len; j++) {
-        VECTOR(*result)[j] /= sum;
-    }
-
-    RNG_END();
-
-    return IGRAPH_SUCCESS;
-}
