@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2009-2021  The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -25,11 +24,11 @@
 #include "test_utilities.h"
 
 igraph_error_t bfs_callback(const igraph_t *graph,
-                           igraph_integer_t vid,
-                           igraph_integer_t pred,
-                           igraph_integer_t succ,
-                           igraph_integer_t rank,
-                           igraph_integer_t dist,
+                           igraph_int_t vid,
+                           igraph_int_t pred,
+                           igraph_int_t succ,
+                           igraph_int_t rank,
+                           igraph_int_t dist,
                            void *extra) {
     IGRAPH_UNUSED(graph);
     IGRAPH_UNUSED(pred);
@@ -45,7 +44,7 @@ int main(void) {
 
     igraph_t graph, ring;
     igraph_vector_int_t restricted, order, rank, father, pred, succ, dist, roots;
-    igraph_integer_t i;
+    igraph_int_t i;
 
     igraph_ring(&ring, 10, /*directed=*/ 0, /*mutual=*/ 0, /*circular=*/ 1);
     igraph_disjoint_union(&graph, &ring, &ring);
@@ -58,7 +57,7 @@ int main(void) {
     igraph_vector_int_init(&succ, 0);
     igraph_vector_int_init(&dist, 0);
 
-    igraph_bfs(&graph, /*root=*/0, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/0, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 1, /*restricted=*/ 0,
                &order, &rank, &father, &pred, &succ, &dist,
                /*callback=*/ 0, /*extra=*/ 0);
@@ -80,7 +79,7 @@ int main(void) {
     /* Test the callback */
 
     printf("(");
-    igraph_bfs(&graph, /*root=*/ 0, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ 0, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 1, /*restricted=*/ 0,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
@@ -88,7 +87,7 @@ int main(void) {
     /* Test different roots */
 
     printf("(");
-    igraph_bfs(&graph, /*root=*/ 2, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ 2, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 1, /*restricted=*/ 0,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
@@ -100,7 +99,7 @@ int main(void) {
         igraph_vector_int_push_back(&restricted, i);
     }
     printf("(");
-    igraph_bfs(&graph, /*root=*/ 5, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ 5, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 1, &restricted,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
@@ -108,13 +107,13 @@ int main(void) {
     /* Root not in restricted set */
 
     printf("(");
-    igraph_bfs(&graph, /*root=*/ 4, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ 4, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 1, &restricted,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
 
     printf("(");
-    igraph_bfs(&graph, /*root=*/ 3, /*roots=*/ 0, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ 3, /*roots=*/ 0, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 0, &restricted,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
@@ -126,7 +125,7 @@ int main(void) {
     VECTOR(roots)[1] = 4;
     VECTOR(roots)[2] = 6;
     printf("(");
-    igraph_bfs(&graph, /*root=*/ -1, &roots, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ -1, &roots, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 0, &restricted,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
@@ -135,7 +134,7 @@ int main(void) {
 
     igraph_vector_int_clear(&roots);
     printf("(");
-    igraph_bfs(&graph, /*root=*/ -1, &roots, /*neimode=*/ IGRAPH_OUT,
+    igraph_bfs(&graph, /*root=*/ -1, &roots, /*mode=*/ IGRAPH_OUT,
                /*unreachable=*/ 0, &restricted,
                0, 0, 0, 0, 0, 0, &bfs_callback, 0);
     printf(" )\n");
