@@ -8,13 +8,12 @@
 static clock_t start;
 
 /* Wait for at least a second before attempting interruption */
-igraph_error_t interruption_handler(void *data) {
-    IGRAPH_UNUSED(data);
+igraph_bool_t interruption_handler(void) {
     if ( ((double) (clock() - start)) / CLOCKS_PER_SEC > 1.0 ) {
         IGRAPH_FINALLY_FREE();
-        return IGRAPH_INTERRUPTED;
+        return true;
     } else {
-        return IGRAPH_SUCCESS;
+        return false;
     }
 }
 
@@ -28,7 +27,7 @@ int main(void) {
     /* Skip test when igraph does not have GLPK support. */
     igraph_small(&graph, 0, IGRAPH_DIRECTED, 0,1, -1);
     ehandler = igraph_set_error_handler(igraph_error_handler_ignore);
-    if (igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP) == IGRAPH_UNIMPLEMENTED) {
+    if (igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP_TI) == IGRAPH_UNIMPLEMENTED) {
         igraph_destroy(&graph);
         igraph_vector_int_destroy(&res);
         return 77;
@@ -43,7 +42,7 @@ int main(void) {
     igraph_full(&graph, 700, IGRAPH_DIRECTED, IGRAPH_NO_LOOPS);
 
     ehandler = igraph_set_error_handler(igraph_error_handler_printignore);
-    IGRAPH_ASSERT(igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP) == IGRAPH_EGLP);
+    IGRAPH_ASSERT(igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP_TI) == IGRAPH_FAILURE);
     igraph_set_error_handler(ehandler);
 
     igraph_destroy(&graph);
@@ -54,15 +53,12 @@ int main(void) {
     igraph_rng_seed(igraph_rng_default(), 42);
 
     igraph_vector_int_init(&res, 0);
-    igraph_erdos_renyi_game(
-                      &graph, IGRAPH_ERDOS_RENYI_GNM,
-                      100, 200,
-                      IGRAPH_DIRECTED, IGRAPH_NO_LOOPS);
+    igraph_erdos_renyi_game_gnm(&graph, 100, 200, IGRAPH_DIRECTED, IGRAPH_SIMPLE_SW, IGRAPH_EDGE_UNLABELED);
 
     igraph_set_interruption_handler(interruption_handler);
     ehandler = igraph_set_error_handler(igraph_error_handler_printignore);
     start = clock();
-    IGRAPH_ASSERT(igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP) == IGRAPH_INTERRUPTED);
+    IGRAPH_ASSERT(igraph_feedback_arc_set(&graph, &res, NULL, IGRAPH_FAS_EXACT_IP_TI) == IGRAPH_INTERRUPTED);
     igraph_set_error_handler(ehandler);
     igraph_set_interruption_handler(NULL);
 
