@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2021  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -116,11 +116,11 @@ void test_extreme_values(void) {
 }
 
 void test_infinities(void) {
-    ASSERT_ALMOST_EQUAL(IGRAPH_POSINFINITY, IGRAPH_POSINFINITY);
-    ASSERT_ALMOST_EQUAL(IGRAPH_NEGINFINITY, IGRAPH_NEGINFINITY);
-    ASSERT_NOT_EQUAL(IGRAPH_NEGINFINITY, IGRAPH_POSINFINITY);
-    ASSERT_NOT_EQUAL(IGRAPH_POSINFINITY, DBL_MAX);
-    ASSERT_NOT_EQUAL(IGRAPH_NEGINFINITY, -DBL_MAX);
+    ASSERT_ALMOST_EQUAL(IGRAPH_INFINITY, IGRAPH_INFINITY);
+    ASSERT_ALMOST_EQUAL(-IGRAPH_INFINITY, -IGRAPH_INFINITY);
+    ASSERT_NOT_EQUAL(-IGRAPH_INFINITY, IGRAPH_INFINITY);
+    ASSERT_NOT_EQUAL(IGRAPH_INFINITY, DBL_MAX);
+    ASSERT_NOT_EQUAL(-IGRAPH_INFINITY, -DBL_MAX);
 }
 
 void test_nans(void) {
@@ -128,10 +128,10 @@ void test_nans(void) {
     ASSERT_NOT_EQUAL(IGRAPH_NAN, 0);
     ASSERT_NOT_EQUAL(-0.0, IGRAPH_NAN);
     ASSERT_NOT_EQUAL(IGRAPH_NAN, -0.0);
-    ASSERT_NOT_EQUAL(IGRAPH_NAN, IGRAPH_POSINFINITY);
-    ASSERT_NOT_EQUAL(IGRAPH_POSINFINITY, IGRAPH_NAN);
-    ASSERT_NOT_EQUAL(IGRAPH_NAN, IGRAPH_NEGINFINITY);
-    ASSERT_NOT_EQUAL(IGRAPH_NEGINFINITY, IGRAPH_NAN);
+    ASSERT_NOT_EQUAL(IGRAPH_NAN, IGRAPH_INFINITY);
+    ASSERT_NOT_EQUAL(IGRAPH_INFINITY, IGRAPH_NAN);
+    ASSERT_NOT_EQUAL(IGRAPH_NAN, -IGRAPH_INFINITY);
+    ASSERT_NOT_EQUAL(-IGRAPH_INFINITY, IGRAPH_NAN);
     ASSERT_NOT_EQUAL(IGRAPH_NAN, DBL_MAX);
     ASSERT_NOT_EQUAL(DBL_MAX, IGRAPH_NAN);
     ASSERT_NOT_EQUAL(IGRAPH_NAN, -DBL_MAX);
