@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2021  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 #include <igraph.h>
 #include "test_utilities.h"
 
-void print_and_destroy(igraph_matrix_t *vectors, igraph_matrix_t *values, igraph_integer_t nev, igraph_error_t error) {
+void print_and_destroy(igraph_matrix_t *vectors, igraph_matrix_t *values, igraph_int_t nev, igraph_error_t error) {
     printf("vectors in:\n");
     print_matrix_format(vectors, stdout, "%6.2f");
     printf("values in:\n");
