@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2021  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,9 @@ int main(void) {
     igraph_vector_t bet, bet2, weights;
     igraph_vector_int_t node_vec, source_vec, target_vec;
     igraph_vs_t vs, vs_source, vs_target;
-    igraph_integer_t i, n;
+    igraph_int_t i, n;
 
-    igraph_integer_t nontriv[] = { 0, 19, 0, 16, 0, 20, 1, 19, 2, 5, 3, 7, 3, 8,
+    igraph_int_t nontriv[] = { 0, 19, 0, 16, 0, 20, 1, 19, 2, 5, 3, 7, 3, 8,
                                 4, 15, 4, 11, 5, 8, 5, 19, 6, 7, 6, 10, 6, 8,
                                 6, 9, 7, 20, 9, 10, 9, 20, 10, 19,
                                 11, 12, 11, 20, 12, 15, 13, 15,
@@ -63,12 +63,12 @@ int main(void) {
     igraph_vs_range(&vs_target, 500, 1000);
 
     igraph_betweenness_subset(/* graph=     */ &g,
-        /* res=       */ &bet,
-        /* vids=      */ igraph_vss_all(),
-        /* directed = */ IGRAPH_UNDIRECTED,
-        /* sources = */ vs_source,
-        /* target = */ vs_target,
-        /* weights=   */ NULL);
+            /* weights=   */ NULL,
+            /* res=       */ &bet,
+            /* sources = */ vs_source,
+            /* target = */ vs_target,
+            /* vids=      */ igraph_vss_all(),
+            /* directed = */ IGRAPH_UNDIRECTED, false);
 
     igraph_vector_destroy(&bet);
     igraph_vs_destroy(&vs_source);
@@ -103,20 +103,19 @@ int main(void) {
     igraph_vector_init(&bet, 0);
 
     igraph_betweenness_subset(
-        /* graph=     */ &g,
-        /* res=       */ &bet,
-        /* vids=      */ igraph_vss_all(),
-        /* directed = */ IGRAPH_UNDIRECTED,
-        /* sources =  */ vs_source,
-        /* target =   */ vs_target,
-        /* weights=   */ NULL
-    );
+            /* graph=     */ &g,
+            /* weights=   */ NULL,
+            /* res=       */ &bet,
+            /* sources =  */ vs_source,
+            /* target =   */ vs_target,
+            /* vids=      */ igraph_vss_all(),
+            /* directed = */ IGRAPH_UNDIRECTED, false);
 
     printf("Max betweenness: %f\n", igraph_vector_max(&bet));
 
     n = igraph_vcount(&g);
     for (i = 0; i < n; i++) {
-        igraph_integer_t expected;
+        igraph_int_t expected;
 
         if (i >= 10911) {
             /* layer 5, in the subset. There are 199 shortest paths that
@@ -148,7 +147,7 @@ int main(void) {
         if (VECTOR(bet)[i] != expected) {
             printf(
                 "Invalid betweenness for vertex %" IGRAPH_PRId ", expected %" IGRAPH_PRId ", got %" IGRAPH_PRId "\n",
-                i, expected, (igraph_integer_t) VECTOR(bet)[i]
+                i, expected, (igraph_int_t) VECTOR(bet)[i]
             );
             break;
         }
@@ -159,12 +158,12 @@ int main(void) {
     igraph_vector_fill(&weights, 1.0);
 
     igraph_betweenness_subset(/* graph=     */ &g,
+            /* weights=   */ &weights,
             /* res=       */ &bet2,
-            /* vids=      */ igraph_vss_all(),
-            /* directed = */ IGRAPH_UNDIRECTED,
             /* sources = */ vs_source,
             /* target = */ vs_target,
-            /* weights=   */ &weights);
+            /* vids=      */ igraph_vss_all(),
+            /* directed = */ IGRAPH_UNDIRECTED, false);
 
     IGRAPH_ASSERT(igraph_vector_all_e(&bet, &bet2));
 
@@ -177,19 +176,19 @@ int main(void) {
 
     printf("\nNon-trivial weighted graph using subset algorithm\n");
     printf("==========================================================\n");
-    igraph_vector_int_view(&edges, nontriv, sizeof(nontriv) / sizeof(nontriv[0]));
+    edges = igraph_vector_int_view(nontriv, sizeof(nontriv) / sizeof(nontriv[0]));
     igraph_create(&g, &edges, 0, /* directed= */ IGRAPH_UNDIRECTED);
-    igraph_vector_view(&weights, nontriv_weights,
+    weights = igraph_vector_view(nontriv_weights,
                        sizeof(nontriv_weights) / sizeof(nontriv_weights[0]));
 
     igraph_vector_init(&bet, 0);
     igraph_betweenness_subset(/* graph=     */ &g,
-        /* res=       */ &bet,
-        /* vids=      */ igraph_vss_all(),
-        /* directed = */ IGRAPH_UNDIRECTED,
-        /* sources = */ igraph_vss_all(),
-        /* target = */ igraph_vss_all(),
-        /* weights=   */ &weights);
+            /* weights=   */ &weights,
+            /* res=       */ &bet,
+            /* sources = */ igraph_vss_all(),
+            /* target = */ igraph_vss_all(),
+            /* vids=      */ igraph_vss_all(),
+            /* directed = */ IGRAPH_UNDIRECTED, false);
 
     print_vector(&bet);
 
@@ -219,22 +218,22 @@ int main(void) {
         printf("subset without %" IGRAPH_PRId "\n", i);
         printf("Unweighted\n");
         igraph_betweenness_subset(/* graph=     */ &g,
-            /* res=       */ &bet,
-            /* vids=      */ vs,
-            /* directed = */ IGRAPH_UNDIRECTED,
-            /* sources = */ vs_source,
-            /* target = */ igraph_vss_all(),
-            /* weights=   */ NULL);
+                /* weights=   */ NULL,
+                /* res=       */ &bet,
+                /* sources = */ vs_source,
+                /* target = */ igraph_vss_all(),
+                /* vids=      */ vs,
+                /* directed = */ IGRAPH_UNDIRECTED, false);
         print_vector(&bet);
 
         printf("Weighted\n");
         igraph_betweenness_subset(/* graph=     */ &g,
-            /* res=       */ &bet2,
-            /* vids=      */ vs,
-            /* directed = */ IGRAPH_UNDIRECTED,
-            /* sources = */ vs_source,
-            /* target = */ igraph_vss_all(),
-            /* weights */ &weights);
+                /* weights */ &weights,
+                /* res=       */ &bet2,
+                /* sources = */ vs_source,
+                /* target = */ igraph_vss_all(),
+                /* vids=      */ vs,
+                /* directed = */ IGRAPH_UNDIRECTED, false);
         print_vector(&bet2);
         printf("\n");
 
@@ -273,22 +272,22 @@ int main(void) {
         printf("subset without %" IGRAPH_PRId "\n", i);
         printf("Unweighted\n");
         igraph_betweenness_subset(/* graph=     */ &g,
-            /* res=       */ &bet,
-            /* vids=      */ igraph_vss_all(),
-            /* directed = */ 0,
-            /* sources = */ igraph_vss_all(),
-            /* target = */ vs_target,
-            /* weights */ NULL);
+                /* weights */ NULL,
+                /* res=       */ &bet,
+                /* sources = */ igraph_vss_all(),
+                /* target = */ vs_target,
+                /* vids=      */ igraph_vss_all(),
+                /* directed = */ 0, false);
         print_vector(&bet);
 
         printf("Weighted\n");
         igraph_betweenness_subset(/* graph=     */ &g,
-            /* res=       */ &bet2,
-            /* vids=      */ igraph_vss_all(),
-            /* directed = */ 0,
-            /* sources = */ igraph_vss_all(),
-            /* target = */ vs_target,
-            /* weights */ &weights);
+                /* weights */ &weights,
+                /* res=       */ &bet2,
+                /* sources = */ igraph_vss_all(),
+                /* target = */ vs_target,
+                /* vids=      */ igraph_vss_all(),
+                /* directed = */ 0, false);
         print_vector(&bet2);
         printf("\n");
 
@@ -305,13 +304,13 @@ int main(void) {
     printf("==========================================================\n");
     igraph_empty(&g, 2, IGRAPH_UNDIRECTED);
     igraph_vector_init(&bet, 0);
-    igraph_betweenness_subset (/* graph=     */ &g,
-        /* res=       */ &bet,
-        /* vids=      */ igraph_vss_all(),
-        /* directed = */ IGRAPH_UNDIRECTED,
-        /* sources = */ igraph_vss_all(),
-        /* target = */ igraph_vss_all(),
-        /* weights=   */ NULL);
+    igraph_betweenness_subset(/* graph=     */ &g,
+            /* weights=   */ NULL,
+            /* res=       */ &bet,
+            /* sources = */ igraph_vss_all(),
+            /* target = */ igraph_vss_all(),
+            /* vids=      */ igraph_vss_all(),
+            /* directed = */ IGRAPH_UNDIRECTED, false);
 
     print_vector(&bet);
 
@@ -338,13 +337,13 @@ int main(void) {
         igraph_vector_int_remove(&source_vec, 0);
         igraph_vs_vector(&vs_source, &source_vec);
 
-        igraph_betweenness_subset (/* graph=     */ &g,
-        /* res=       */ &bet,
-        /* vids=      */ igraph_vss_all(),
-        /* directed = */ IGRAPH_UNDIRECTED,
-        /* sources = */ vs_source,
-        /* target = */ vs_target,
-        /* weights=   */ NULL);;
+        igraph_betweenness_subset(/* graph=     */ &g,
+                /* weights=   */ NULL,
+                /* res=       */ &bet,
+                /* sources = */ vs_source,
+                /* target = */ vs_target,
+                /* vids=      */ igraph_vss_all(),
+                /* directed = */ IGRAPH_UNDIRECTED, false);;
         printf("Max betweenness: %f\n", igraph_vector_max(&bet));
 
         igraph_vector_destroy(&bet);
