@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -26,7 +25,7 @@
 
 void show_results(igraph_t *g, igraph_vector_t *mod, igraph_matrix_int_t *merges,
                   igraph_vector_int_t *membership, FILE* f) {
-    igraph_integer_t i = 0;
+    igraph_int_t i = 0;
     igraph_vector_int_t our_membership;
 
     igraph_vector_int_init(&our_membership, 0);
