@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -25,12 +25,20 @@ int main(void) {
 
     igraph_rng_seed(igraph_rng_default(), 9275);
 
-    igraph_erdos_renyi_game_gnp(&g1, 10, 0.3, IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
-    igraph_correlated_game(&g1, &g2, 0.9, 0.3, /* permutation=*/ NULL);
+    igraph_erdos_renyi_game_gnp(&g1, 10, 0.3, IGRAPH_UNDIRECTED, IGRAPH_SIMPLE_SW, IGRAPH_EDGE_UNLABELED);
 
+    igraph_correlated_game(&g2, &g1, 0.9, 0.3, /* permutation=*/ NULL);
     IGRAPH_ASSERT(igraph_vcount(&g1) == igraph_vcount(&g2));
-
     igraph_destroy(&g2);
+
+    igraph_correlated_game(&g2, &g1, 0.0, 0.3, /* permutation=*/ NULL);
+    IGRAPH_ASSERT(igraph_vcount(&g1) == igraph_vcount(&g2));
+    igraph_destroy(&g2);
+
+    igraph_correlated_game(&g2, &g1, 1.0, 0.3, /* permutation=*/ NULL);
+    IGRAPH_ASSERT(igraph_vcount(&g1) == igraph_vcount(&g2));
+    igraph_destroy(&g2);
+
     igraph_destroy(&g1);
 
     VERIFY_FINALLY_STACK();
