@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2015-2022  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
 void random_permutation(igraph_vector_int_t *vec) {
     /* We just do size(vec) * 2 swaps */
-    igraph_integer_t one, two, tmp, i, n = igraph_vector_int_size(vec);
+    igraph_int_t one, two, tmp, i, n = igraph_vector_int_size(vec);
     for (i = 0; i < 2 * n; i++) {
         one = RNG_INTEGER(0, n - 1);
         two = RNG_INTEGER(0, n - 1);
@@ -36,7 +36,7 @@ void random_permutation(igraph_vector_int_t *vec) {
 
 
 void test3(void) {
-    igraph_integer_t i, j;
+    igraph_int_t i, j;
     igraph_graph_list_t graphs3;
     igraph_t g;
 
@@ -80,7 +80,7 @@ void test3(void) {
 
 
 void test4(void) {
-    igraph_integer_t i, j;
+    igraph_int_t i, j;
     igraph_graph_list_t graphs4;
     igraph_t g;
 
@@ -130,6 +130,7 @@ void test_bliss(void) {
     igraph_bliss_info_t info;
     igraph_vector_int_t color;
     igraph_vector_int_list_t generators;
+    igraph_real_t num_automorphisms;
 
     igraph_ring(&ring1, 100, /*directed=*/ 0, /*mutual=*/ 0, /*circular=*/1);
     igraph_vector_int_init_range(&perm, 0, igraph_vcount(&ring1));
@@ -145,30 +146,27 @@ void test_bliss(void) {
         printf("Bliss failed on ring isomorphism.\n");
     }
 
-    igraph_count_automorphisms(&ring1, NULL, IGRAPH_BLISS_F, &info);
-    if (strcmp(info.group_size, "200") != 0) {
+    igraph_count_automorphisms(&ring1, NULL, &num_automorphisms);
+    if (num_automorphisms != 200) {
         printf("Biss automorphism count failed: ring1.\n");
     }
-    igraph_free(info.group_size);
 
-    igraph_count_automorphisms(&ring2, NULL, IGRAPH_BLISS_F, &info);
-    if (strcmp(info.group_size, "200") != 0) {
+    igraph_count_automorphisms(&ring2, NULL, &num_automorphisms);
+    if (num_automorphisms != 200) {
         printf("Biss automorphism count failed: ring2.\n");
     }
-    igraph_free(info.group_size);
 
-    igraph_count_automorphisms(&directed_ring, NULL, IGRAPH_BLISS_F, &info);
-    if (strcmp(info.group_size, "100") != 0) {
+    igraph_count_automorphisms(&directed_ring, NULL, &num_automorphisms);
+    if (num_automorphisms != 100) {
         printf("Biss automorphism count failed: directed_ring.\n");
     }
-    igraph_free(info.group_size);
 
-    // The follwing test is included so there is at least one call to igraph_automorphism_group
+    // The following test is included so there is at least one call to igraph_automorphism_group_bliss
     // in the test suite. However, the generator set returned may depend on the splitting
     // heursitics as well as on the Bliss version. If the test fails, please verify manually
     // that the generating set is valid. For a undirected cycle graph like ring2, there should
     // be two generators: a cyclic permutation and a reversal of the vertex order.
-    igraph_automorphism_group(&ring2, NULL, &generators, IGRAPH_BLISS_F, NULL);
+    igraph_automorphism_group_bliss(&ring2, NULL, &generators, IGRAPH_BLISS_F, NULL);
     if (igraph_vector_int_list_size(&generators) != 2)
         printf("Bliss automorphism generators may have failed with ring2. "
                "Please verify the generators manually. "
@@ -176,7 +174,7 @@ void test_bliss(void) {
     igraph_vector_int_list_clear(&generators);
 
     // For a directed ring, the only generator should be a cyclic permutation.
-    igraph_automorphism_group(&directed_ring, NULL, &generators, IGRAPH_BLISS_F, NULL);
+    igraph_automorphism_group_bliss(&directed_ring, NULL, &generators, IGRAPH_BLISS_F, NULL);
     if (igraph_vector_int_list_size(&generators) != 1)
         printf("Bliss automorphism generators may have failed with directed_ring. "
                "Please verify the generators manually. "
@@ -185,14 +183,14 @@ void test_bliss(void) {
 
     igraph_vector_int_init_range(&color, 0, igraph_vcount(&ring1));
 
-    igraph_count_automorphisms(&ring1, &color, IGRAPH_BLISS_F, &info);
+    igraph_count_automorphisms_bliss(&ring1, &color, IGRAPH_BLISS_F, &info);
     if (strcmp(info.group_size, "1") != 0) {
         printf("Bliss automorphism count with color failed: ring1.\n");
     }
     igraph_free(info.group_size);
 
     // There's only one automorphism for this coloured graph, so the generating set is empty.
-    igraph_automorphism_group(&ring1, &color, &generators, IGRAPH_BLISS_F, NULL);
+    igraph_automorphism_group_bliss(&ring1, &color, &generators, IGRAPH_BLISS_F, NULL);
     if (igraph_vector_int_list_size(&generators) != 0) {
         printf("Bliss automorphism generators failed with colored graph.\n");
     }
@@ -228,15 +226,11 @@ int main(void) {
 
     igraph_rng_seed(igraph_rng_default(), 293847); /* make tests deterministic */
 
-    RNG_BEGIN();
-
     test3();
     test4();
     test_bliss();
     test_bug_995();
 
-    RNG_END();
-
     VERIFY_FINALLY_STACK();
 
     return 0;
